package com.seeyon.apps.govdoc.dao;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.google.common.collect.Lists;
import com.seeyon.apps.edoc.constants.EdocConstant;
import com.seeyon.apps.edoc.constants.EdocConstant.flowState;
import com.seeyon.apps.govdoc.constant.GovdocEnum;
import com.seeyon.apps.govdoc.constant.GovdocEnum.GovdocExchangeTypeEnum;
import com.seeyon.apps.govdoc.constant.GovdocEnum.OldEdocTypeEnum;
import com.seeyon.apps.govdoc.constant.GovdocEnum.TransferStatus;
import com.seeyon.apps.govdoc.constant.GovdocListEnum.GovdocListTypeEnum;
import com.seeyon.apps.govdoc.helper.GovdocHelper;
import com.seeyon.apps.govdoc.helper.GovdocOrgHelper;
import com.seeyon.apps.govdoc.po.GovdocExchangeMain;
import com.seeyon.apps.govdoc.util.GovdocUtil;
import com.seeyon.apps.govdoc.vo.GovdocListVO;
import com.seeyon.apps.secret.util.SecretUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.util.WFComponentUtil;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.constants.ApplicationSubCategoryEnum;
import com.seeyon.ctp.common.constants.SystemProperties;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.content.affair.constants.SubStateEnum;
import com.seeyon.ctp.common.dao.BaseHibernateDao;
import com.seeyon.ctp.common.dump.vo.DumpDataVO;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.flag.SysFlag;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.po.affair.CtpAffairExtend;
import com.seeyon.ctp.common.workflowmanage.vo.WorkflowData;
import com.seeyon.ctp.form.util.Enums.FormType;
import com.seeyon.ctp.organization.OrgConstants;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.DBAgent;
import com.seeyon.ctp.util.Datetimes;
import com.seeyon.ctp.util.FlipInfo;
import com.seeyon.ctp.util.SQLWildcardUtil;
import com.seeyon.ctp.util.Strings;
import com.seeyon.v3x.edoc.domain.EdocSummary;
import com.seeyon.v3x.edoc.listener.EdocWorkflowManageHandler;


public class GovdocListDao extends BaseHibernateDao{
	
    private OrgManager orgManager = null;
    
    public void setOrgManager(OrgManager orgManager) {
        this.orgManager = orgManager;
    }
    
    //公文列表主表Hql
	private static final String SummaryListHql = "summary.id," +
		        "summary.subject," +
		        "summary.identifier," +
		        "summary.govdocType," +
		        "summary.state," +
			    "summary.isQuickSend," +
		        "summary.templeteId," +
		        "summary.archiveId," +
		        "summary.processId," +
		        "summary.caseId," +
		        "summary.startUserId," +//11
		        "summary.startTime," +
		        "summary.createTime," +
		        "summary.completeTime," +
				"summary.packTime," +
		        "summary.deadline," +
		        "summary.deadlineDatetime," + 
		        "summary.coverTime," +
			    "summary.docMark,"+
			    "summary.serialNo,"+
			    "summary.docMark2,"+
			    "summary.docType,"+//21
			    "summary.sendType,"+
			    "summary.secretLevel,"+
			    "summary.urgentLevel,"+
			    "summary.unitLevel,"+
			    "summary.keepPeriod,"+
			    "summary.copies,"+
			    "summary.issuer,"+
			    "summary.signingDate,"+
			    "summary.createPerson,"+
			    "summary.printer,"+//31
			    "summary.phone,"+
			    "summary.orgAccountId,"+
			    "summary.exchangeSendAffairId,"+
			    "summary.undertaker," + 
			    "summary.undertakenoffice," +
			    "summary.bodyType," +
			    "summary.hasArchive,"+
			    "summary.signPerson,"+
			    "summary.receiptDate,"+
			    "summary.registrationDate,"+//41
			    "summary.fromType,"+
			    "summary.newflowType";
	
	//公文列表主表Clob字段Hql
	private static final String SummaryColbListHql = "summary.currentNodesInfo, "
			+ "summary.sendTo, "
			+ "summary.copyTo, "
			+ "summary.reportTo, "
			+ "summary.printUnit, "
			+ "summary.sendUnit, "
			+ "summary.sendDepartment";
	
	//公文列表待办Hql
	private static final String AffariListHql = "affair.id," +
	        "affair.identifier,"+
	        "affair.subject,"+
	        "affair.app,"+
		    "affair.subApp,"+
	        "affair.state," +
	        "affair.subState," +
	        "affair.finish," +
	        "affair.objectId,"+
	        "affair.subObjectId," +
	        "affair.activityId,"+
	        "affair.nodePolicy," +
	        "affair.memberId," +
	        "affair.transactorId," +
	        
	        "affair.hastenTimes," +
	        "affair.coverTime," +
	        "affair.remindDate," +
	        "affair.deadlineDate," +
	        "affair.receiveTime," +
	        "affair.completeTime," +
	        "affair.createDate," +
	        "affair.updateDate," +
	        "affair.expectedProcessTime," +
	        "affair.importantLevel," +
		    "affair.archiveId,"+
	        "affair.track," +
		    "affair.backFromId,"+
		    "affair.preApprover,"+
		    "affair.proxyMemberId,"+
		    "affair.transactorId,"+
		    "affair.senderId";
	
	//公文登记列表主表字段Hql(非Clob)
	private static final String RegSummaryHql = "summary.id"
			+ ",summary.subject"
			+ ",summary.identifier"
			+ ",summary.govdocType"
			+ ",summary.edocType"
			+ ",summary.state"
			+ ",summary.transferStatus"
			+ ",summary.sendType"
			+ ",summary.docType"
			+ ",summary.unitLevel"//10
			+ ",summary.urgentLevel"
			+ ",summary.secretLevel"
			+ ",summary.edocSecretLevel"
			+ ",summary.keepPeriod"
			+ ",summary.createTime"
			+ ",summary.completeTime"
			+ ",summary.packTime"
			+ ",summary.receiptDate"
			+ ",summary.registrationDate"
			+ ",summary.startTime"//20
			+ ",summary.startUserId"
			+ ",summary.orgAccountId"
			+ ",summary.orgDepartmentId"
			+ ",summary.docMark"
			+ ",summary.docMark2"
			+ ",summary.serialNo"
			+ ",summary.copies"
			+ ",summary.copies2"
			+ ",summary.createPerson"
			+ ",summary.review"//30
			+ ",summary.issuer"
			+ ",summary.signPerson"
			+ ",summary.signingDate"
			+ ",summary.bodyType"
	        + ",summary.undertaker"
            + ",summary.exchangeSendAffairId";

	//公文登记列表Clob字段Hql(Clob)
	private static final String RegClobSummaryHql = "summary.sendTo"
			+ ",summary.sendTo2"
			+ ",summary.sendToId"
			+ ",summary.sendToId2"
			+ ",summary.copyTo"
			+ ",summary.copyTo2"
			+ ",summary.copyToId"
			+ ",summary.copyToId2"
			+ ",summary.reportTo"
			+ ",summary.reportTo2"//10
			+ ",summary.reportToId"
			+ ",summary.reportToId2"
			+ ",summary.sendUnit"
			+ ",summary.sendUnitId"
			+ ",summary.sendUnit2"
			+ ",summary.sendUnitId2"
			+ ",summary.sendDepartment"
			+ ",summary.sendDepartmentId"
			+ ",summary.sendDepartment2"//20
			+ ",summary.sendDepartmentId2"
			+ ",summary.currentNodesInfo"
	        + ",summary.printUnit"
	        + ",summary.printer"
	        + ",summary.auditor"
	        + ",summary.undertakenoffice";

	//公文查询自定义元素 EdocSummaryExtentHql
	private static final String SummaryExtentListHql = "summaryExt.id,"+
				        "summaryExt.summaryId," +
				        "summaryExt.appType," +
				        "summaryExt.varchar1,"+
					    "summaryExt.varchar2,"+
					    "summaryExt.varchar3,"+
					    "summaryExt.varchar4,"+
					    "summaryExt.varchar5,"+
					    "summaryExt.varchar6,"+
					    "summaryExt.varchar7,"+
					    "summaryExt.varchar8,"+
					    "summaryExt.varchar9,"+
					    "summaryExt.varchar10,"+
					    "summaryExt.varchar11,"+
					    "summaryExt.varchar12,"+
					    "summaryExt.varchar13,"+
					    "summaryExt.varchar14,"+
					    "summaryExt.varchar15,"+
					    "summaryExt.varchar16,"+
					    "summaryExt.varchar17," + 
					    "summaryExt.varchar18," +
					    "summaryExt.varchar19," +
					    "summaryExt.varchar20,"+
					    "summaryExt.varchar21,"+
					    "summaryExt.varchar22,"+
					    "summaryExt.varchar23,"+
					    "summaryExt.varchar24,"+
					    "summaryExt.varchar25,"+
					    "summaryExt.varchar26,"+
					    "summaryExt.varchar27,"+
					    "summaryExt.varchar28,"+
					    "summaryExt.varchar29,"+
					    "summaryExt.varchar30,"+
					    "summaryExt.integer1,"+
					    "summaryExt.integer2,"+
					    "summaryExt.integer3,"+
					    "summaryExt.integer4,"+
					    "summaryExt.integer5,"+
					    "summaryExt.integer6,"+
					    "summaryExt.integer7,"+
					    "summaryExt.integer8,"+
					    "summaryExt.integer9,"+
					    "summaryExt.integer10,"+
					    "summaryExt.integer11,"+
					    "summaryExt.integer12,"+
					    "summaryExt.integer13,"+
					    "summaryExt.integer14,"+
					    "summaryExt.integer15,"+
					    "summaryExt.integer16,"+
					    "summaryExt.integer17," + 
					    "summaryExt.integer18," +
					    "summaryExt.integer19," +
					    "summaryExt.integer20,"+
					    "summaryExt.decimal1,"+
					    "summaryExt.decimal2,"+
					    "summaryExt.decimal3,"+
					    "summaryExt.decimal4,"+
					    "summaryExt.decimal5,"+
					    "summaryExt.decimal6,"+
					    "summaryExt.decimal7,"+
					    "summaryExt.decimal8,"+
					    "summaryExt.decimal9,"+
					    "summaryExt.decimal10,"+
					    "summaryExt.decimal11,"+
					    "summaryExt.decimal12,"+
					    "summaryExt.decimal13,"+
					    "summaryExt.decimal14,"+
					    "summaryExt.decimal15,"+
					    "summaryExt.decimal16,"+
					    "summaryExt.decimal17,"+
					    "summaryExt.decimal18,"+
					    "summaryExt.decimal19," + 
					    "summaryExt.decimal20," +
					    "summaryExt.date1,"+
					    "summaryExt.date2,"+
					    "summaryExt.date3,"+
					    "summaryExt.date4,"+
					    "summaryExt.date5,"+
					    "summaryExt.date6,"+
					    "summaryExt.date7,"+
					    "summaryExt.date8,"+
					    "summaryExt.date9,"+
					    "summaryExt.date10,"+
					    "summaryExt.date11,"+
					    "summaryExt.date12,"+
					    "summaryExt.date13,"+
					    "summaryExt.date14,"+
					    "summaryExt.date15,"+
					    "summaryExt.date16,"+
					    "summaryExt.date17," + 
					    "summaryExt.date18," +
					    "summaryExt.date19," +
					    "summaryExt.date20,"+
					    "summaryExt.list1,"+
					    "summaryExt.list2,"+
					    "summaryExt.list3,"+
					    "summaryExt.list4,"+
					    "summaryExt.list5,"+
					    "summaryExt.list6,"+
					    "summaryExt.list7,"+
					    "summaryExt.list8,"+
					    "summaryExt.list9,"+
					    "summaryExt.list10,"+
					    "summaryExt.list11,"+
					    "summaryExt.list12,"+
					    "summaryExt.list13,"+
					    "summaryExt.list14,"+
					    "summaryExt.list15,"+
					    "summaryExt.list16,"+
					    "summaryExt.list17," + 
					    "summaryExt.list18," +
					    "summaryExt.list19," +
					    "summaryExt.list20,"+
				        "summaryExt.text1," +
				        "summaryExt.text2," +
					    "summaryExt.text3," +
				        "summaryExt.text4," +
				        "summaryExt.text5," +
				        "summaryExt.text6," +
				        "summaryExt.text7," +
				        "summaryExt.text8," +
				        "summaryExt.text9," +
				        "summaryExt.text10," +
				        "summaryExt.text11," +
				        "summaryExt.text12," +
				        "summaryExt.text13," + 
				        "summaryExt.text14," +
					    "summaryExt.text15";
	
	private static final String AdminManagerHql = "affair.subApp,summary.id,affair.app,summary.subject,summary.startUserId,u.name,summary.isQuickSend,summary.createTime,summary.processId,summary.caseId, "
																		+ " affair.deadlineDate,affair.remindDate,summary.orgAccountId,affair.finish,summary.templeteId,summary.completeTime,summary.deadlineDatetime,summary.currentNodesInfo " 
																		+ ", summary.secretLevel,affair.formAppId,affair.formRecordid ";
	
	//公文登记列表登记字段Hql
	private static final String RegRecHql = "register.id," + "register.recNo";
	
	//公文主表的是否归档、联系电话字段Hql
	private static final String SummaryArchive_PhoneHql = "summary.hasArchive,"+"summary.phone";
	
	//公文列表/公文查询/关联公文-所有字段Hql
	protected static final String ListAllColumnHql = SummaryListHql + "," + SummaryColbListHql + "," + AffariListHql;
		
	//公文查询 含自定义公文元素
	protected static final String ListAllColumnHasSummaryExtendHql = SummaryListHql + "," + SummaryColbListHql + "," + AffariListHql+ "," + SummaryExtentListHql;
	//发文/签报登记簿-所有字段Hql
	private static final String RegisterAllColumnHql = RegSummaryHql + "," + RegClobSummaryHql+ "," + SummaryArchive_PhoneHql;
	private static final String RegisterAllColumnHqlSummaryExt = RegSummaryHql + "," + RegClobSummaryHql+ "," + SummaryArchive_PhoneHql+ "," + SummaryExtentListHql;
	//收文登记簿-所有字段Hql
	private static final String RecRegisterAllColumnHql = RegSummaryHql + "," + RegClobSummaryHql+ "," + SummaryArchive_PhoneHql + "," + RegRecHql;
	
	private static final String RecRegisterAllColumnHqlSummaryExt = RegSummaryHql + "," + RegClobSummaryHql+ "," + SummaryArchive_PhoneHql + "," + RegRecHql+ "," + SummaryExtentListHql;
	
	private static final String BackExchangeHql = "summary.id,summary.docMark,summary.completeTime,summary.subject,log.backOpinion,detail.sendAccountId,detail.sendUnit,detail.recOrgName";
	private static final String ReturndBackExchangeHql = "summary.id,summary.docMark,summary.subject,detail.sendUnit,detail.recOrgName,log.backOpinion,log.time";
	
	private static final String queryOldDataIdByNewDataId = "select oldDataId from UpgradeDataRelation where newDataId in (:newTids)";
	
	/**
	 * 公文列表数据展及查询
	 * 用于二级菜单-待办/已办/已发/待发 发文管理/收文管理/签报管理
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings("unchecked")
	public List<GovdocListVO> findList(FlipInfo flipInfo, Map<String, String> params) throws BusinessException {
		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		
		String listType = params.get("listType");
		String govdocType = params.get("govdocType");
		Long memberId = Long.parseLong(params.get("memberId"));
		boolean hasDeduplication = "true".equals(params.get("deduplication"));
		boolean hasAgent = "true".equals(params.get("agentFlag")) || "true".equals(params.get("agentToFlag"));
		
		Map<String, Object> paramMap = new HashMap<String, Object>();
		
		StringBuilder buffer = new StringBuilder();
		buffer.append("select " + ListAllColumnHql);
		buffer.append(" from EdocSummary summary");
		buffer.append(" , CtpAffair affair");
		buffer.append(" where summary.id = affair.objectId");

		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, memberId, hasAgent));
		//公文app及subApp
		buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true));
		//处理状态
		buffer.append(getHqlByAffairState(paramMap, listType));
		//按节点权限过滤
		buffer.append(getHqlByAffairPolicy(paramMap, params));
		//按节点权限(非)过滤
		buffer.append(getHqlByAffairNotInPolicy(paramMap, params));
		//按条件查询
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
			buffer.append(getHqlBySummaryTemplateIds(paramMap, params));//公文模板ID查询-来自业务生成器
			buffer.append(getAffairHqlByCondition(paramMap, params));//按affair表字段查询
		}
		
		if(hasDeduplication) {
			buffer.append(" and affair.id = (");
			buffer.append(" 	select max(affair2.id) from CtpAffair affair2");
			buffer.append("  	where affair2.objectId = summary.id");
			buffer.append("  	and affair2.subApp = affair.subApp");
			//公文处理人员过滤(包括代理人)
			buffer.append(getHqlByAffairMember(paramMap, memberId, hasAgent, "affair2"));
			//公文app及subApp
			buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true, "affair2"));
			//处理状态`
			buffer.append(getHqlByAffairState(paramMap, listType, "affair2"));
			//按节点权限过滤
			buffer.append(getHqlByAffairPolicy(paramMap, params, "affair2"));
			//按节点权限(非)过滤
			buffer.append(getHqlByAffairNotInPolicy(paramMap, params, "affair2"));
			//按条件查询
			if(params.containsKey("condition")) {
				buffer.append(getAffairHqlByCondition(paramMap, params, "affair2"));//按affair表字段查询
			}
			buffer.append(" )");
		}
		//添加分保插件
		buffer.append(getHqlByMemberSecret(paramMap,params,"summary"));
		if(listType != null){
			if(listType.startsWith("listDone")) {//已办
				buffer.append(" order by affair.completeTime desc");
			} else if(listType.startsWith("listPending")) {//待办
				buffer.append(" order by affair.receiveTime desc");
			} else if(listType.startsWith("listWaitSend")) {//待发
				buffer.append(" order by summary.createTime desc");
			} else {
				buffer.append(" order by summary.createTime desc");
			}
		}

		boolean dumpData = DumpDataVO.dataType.dumpData.getKey().equals(params.get("dumpData"));
		
		List<Object[]> result = null;
		if(flipInfo != null && !dumpData) {// 查询转储数据必须要用super的find相关方法才能查询分库
			result = DBAgent.find(buffer.toString(), paramMap,flipInfo);
		} else {
			result = super.find(buffer.toString(), paramMap);
		}


		if(Strings.isNotEmpty(result)) {
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				vo.toObject(object, hasDeduplication);
				voList.add(vo);
			}
		}
		
		return voList;
	}
	
	/**
	 * 公文列表数据总条数查询
	 * 用于二级菜单-待办/已办/已发/待发 发文管理/收文管理/签报管理
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	public int findListCount(Map<String, String> params) throws BusinessException {
		int resultCount = 0;
		String listType = params.get("listType");
		String govdocType = params.get("govdocType");
		Long memberId = Long.parseLong(params.get("memberId"));
		boolean hasDeduplication = "true".equals(params.get("deduplication"));
		boolean hasAgent = "true".equals(params.get("agentFlag")) || "true".equals(params.get("agentToFlag"));
		
		Map<String, Object> paramMap = new HashMap<String, Object>();
		
		StringBuilder buffer = new StringBuilder();
		buffer.append("select count(summary.id) from EdocSummary summary");
		buffer.append(" , CtpAffair affair");
		buffer.append(" where summary.id = affair.objectId");
		
		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, memberId, hasAgent));
		//公文app及subApp
		buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true));
		//处理状态
		buffer.append(getHqlByAffairState(paramMap, listType));
		//按节点权限过滤
		buffer.append(getHqlByAffairPolicy(paramMap, params));
		//按节点权限(非)过滤
		buffer.append(getHqlByAffairNotInPolicy(paramMap, params));
		//按条件查询
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
			buffer.append(getHqlBySummaryTemplateIds(paramMap, params));//公文模板ID查询-来自业务生成器
			buffer.append(getAffairHqlByCondition(paramMap, params));//按affair表字段查询
		}
		
		if(hasDeduplication) {
			buffer.append(" and affair.completeTime = (");
			buffer.append(" 	select max(affair2.completeTime) from CtpAffair affair2");
			buffer.append("  	where affair2.objectId = summary.id");
			//公文处理人员过滤(包括代理人)
			buffer.append(getHqlByAffairMember(paramMap, memberId, hasAgent, "affair2"));
			//公文app及subApp
			buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true, "affair2"));
			//处理状态
			buffer.append(getHqlByAffairState(paramMap, listType, "affair2"));
			//按节点权限过滤
			buffer.append(getHqlByAffairPolicy(paramMap, params, "affair2"));
			//按节点权限(非)过滤
			buffer.append(getHqlByAffairNotInPolicy(paramMap, params, "affair2"));
			//按条件查询
			if(params.containsKey("condition")) {
				buffer.append(getAffairHqlByCondition(paramMap, params, "affair2"));//按affair表字段查询
			}
			buffer.append(" )");
		}
		if(listType != null){
			if(listType.startsWith("listDone")) {//已办
				buffer.append(" order by affair.completeTime desc");
			} else if(listType.startsWith("listPending")) {//待办
				buffer.append(" order by affair.receiveTime desc");
			} else if(listType.startsWith("listWaitSend")) {//待发
				buffer.append(" order by summary.createTime desc");
			} else {
				buffer.append(" order by summary.createTime desc");
			}
		}
		
		resultCount = DBAgent.count(buffer.toString(), paramMap);
		return resultCount;
	}

	protected String  getHqlByMemberSecret(Map<String, Object> paramMap,Map<String, String> params,String sqlCloumnName) {
		//分保：查询条件有密级、待发列表不进行过滤
		if(SecretUtil.hasPluginSecret() && !(params.containsKey("condition")  
				&& (Strings.isNotBlank(params.get("secretLevel")) 
						|| (Strings.isNotBlank(params.get("listType")) && params.get("listType").contains("listWaitSend"))))){
			StringBuilder secretLevelHql = new StringBuilder();
			String secret = params.get("secretLevel");
        	Integer secretLevel = 100;
        	Long memberId = Long.parseLong(params.get("memberId"));
        	if(memberId != null) {
    			V3xOrgMember member = GovdocOrgHelper.getMemberById(memberId);
    			if(member != null && member.getSecretLevel()!=null) {
    				secretLevel = member.getSecretLevel();
    			}
    		}

			if (Strings.isNotBlank(secret)) {
				int secretLevelInt = Integer.parseInt(secret);
				//如果密级超过当前人员的密级，查询密级设置为-1，查不出数据
				if (secretLevel < secretLevelInt) {
					secretLevelInt = -1;
				}
				secretLevel = secretLevelInt;
			}
			if(secretLevel==null){
				secretLevel=0;
			}

        	secretLevelHql.append(" and (").append(sqlCloumnName).append(".secretLevel IS NULL");
        	secretLevelHql.append(" or ");
        	secretLevelHql.append(sqlCloumnName).append(".secretLevel <= :secretLevel");
			secretLevelHql.append(")");
			paramMap.put("secretLevel", String.valueOf(secretLevel));
			return secretLevelHql.toString();
        }else{
        	return "";
        }
	}
	
	/**
	 * 公文关联文档数据展示及查询
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings("unchecked")
	public List<GovdocListVO> find4QuoteList(FlipInfo flipInfo, Map<String, String> params) throws BusinessException {
		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		
		String listType = params.get("listType");
		String govdocType = params.get("govdocType");
		Long memberId = Long.parseLong(params.get("memberId"));
		boolean hasDeduplication = "true".equals(params.get("deduplication"));
		
		Map<String, Object> paramMap = new HashMap<String, Object>();
		
		StringBuilder buffer = new StringBuilder();
		buffer.append("select " + ListAllColumnHql);
		buffer.append(" from EdocSummary summary");
		buffer.append(" , CtpAffair affair");
		buffer.append(" where summary.id = affair.objectId");
		
		//分保-公文列表密级过滤
		if(SecretUtil.hasPluginSecret()) {
			buffer.append(getHqlBySummarySecret(paramMap,params, memberId));
		}
		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, memberId, false));
		//公文app及subApp
		buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, false));
		//处理状态
		buffer.append(getHqlByAffairState(paramMap, listType));
		//按节点权限过滤
		buffer.append(getHqlByAffairPolicy(paramMap, params));
		//按条件查询
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
			buffer.append(getAffairHqlByCondition(paramMap, params));//按affair表字段查询
		}
		
		if(hasDeduplication) {
			buffer.append(" and affair.completeTime = (");
			buffer.append(" 	select max(affair2.completeTime) from CtpAffair affair2");
			buffer.append("  	where affair2.objectId = summary.id");
			//公文处理人员过滤(包括代理人)
			buffer.append(getHqlByAffairMember(paramMap, memberId, false, "affair2"));
			//公文app及subApp
			buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, false, "affair2"));
			//处理状态
			buffer.append(getHqlByAffairState(paramMap, listType, "affair2"));
			//按节点权限过滤
			buffer.append(getHqlByAffairPolicy(paramMap, params, "affair2"));
			//按条件查询
			if(params.containsKey("condition")) {
				buffer.append(getAffairHqlByCondition(paramMap, params, "affair2"));//按affair表字段查询
			}
			buffer.append(" )");
		}
		if(listType != null){
			if(listType.startsWith("listDone")) {//已办
				buffer.append(" order by affair.completeTime desc");
			} else if(listType.startsWith("listPending")) {//待办
				buffer.append(" order by affair.receiveTime desc");
			} else if(listType.startsWith("listWaitSend")) {//待发
				buffer.append(" order by summary.createTime desc");
			} else {
				buffer.append(" order by summary.createTime desc");
			}
		}
		if(flipInfo != null) {
			flipInfo.setNeedTotal(true);
		}
		List<Object[]> result = DBAgent.find(buffer.toString(), paramMap, flipInfo);
		
		if(Strings.isNotEmpty(result)) {
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				vo.toObject(object, hasDeduplication);
				voList.add(vo);
			}
		}
		
		return voList;
	}

	/**
	 * 公文二级菜单-公文查询数据展示及查询
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings("unchecked")
	public List<GovdocListVO> findQueryResultList(FlipInfo flipInfo, Map<String, String> params) throws BusinessException {
		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		
		String listType = params.get("listType");
		String govdocType = params.get("govdocType");
		Long memberId = Long.parseLong(params.get("memberId"));
		boolean hasDeduplication = "true".equals(params.get("deduplication"));
		
		//公文查询-是否需要查询公文的扩展表
		boolean isNeedSearchSummaryExtend = false;
		if("true".equals(params.get("isNeedSearchSummaryExtend"))){
			isNeedSearchSummaryExtend = true;
		}
		Map<String, Object> paramMap = new HashMap<String, Object>();
		
		StringBuilder buffer = new StringBuilder();
		//公文查询含自定义公文元素
		if (isNeedSearchSummaryExtend) {
			buffer.append("select " + ListAllColumnHasSummaryExtendHql);
		}else{
			buffer.append("select " + ListAllColumnHql);
		}
		buffer.append(" from EdocSummary summary");
		buffer.append(" , CtpAffair affair");
		if (isNeedSearchSummaryExtend) {
			buffer.append(",EdocSummaryExtend summaryExt");
		}
		buffer.append(" where summary.id = affair.objectId");
		if (isNeedSearchSummaryExtend) {
			buffer.append(" and summary.id = summaryExt.summaryId ");
		}
		//分保-公文列表密级过滤
		if(SecretUtil.hasPluginSecret()) {
			buffer.append(getHqlBySummarySecret(paramMap,params, memberId));
		}
		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, memberId, false));
		//公文app及subApp
		buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, false));
		//处理状态
		buffer.append(getHqlByAffairState(paramMap, listType));
		//按节点权限过滤
		buffer.append(getHqlByAffairPolicy(paramMap, params));
		//按条件查询
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
			buffer.append(getAffairHqlByCondition(paramMap, params));//按affair表字段查询
			//公文查询含自定义公文元素
			if (isNeedSearchSummaryExtend) {
				//如果包含list自定义元素
				buffer.append(getEdocSummaryExtendHqlByCondition(paramMap, params));//按edocSummaryExtend表字段查询
			}
		}
		
		if(hasDeduplication) {
			buffer.append(" and affair.id = (");
			buffer.append(" 	select max(affair2.id) from CtpAffair affair2");
			buffer.append("  	where affair2.objectId = summary.id");
			//公文处理人员过滤(包括代理人)
			buffer.append(getHqlByAffairMember(paramMap, memberId, false, "affair2"));
			//公文app及subApp
			buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, false, "affair2"));
			//处理状态
			buffer.append(getHqlByAffairState(paramMap, listType, "affair2"));
			//按节点权限过滤
			buffer.append(getHqlByAffairPolicy(paramMap, params, "affair2"));
			//按条件查询
			if(params.containsKey("condition")) {
				buffer.append(getAffairHqlByCondition(paramMap, params, "affair2"));//按affair表字段查询
				//公文查询含自定义公文元素
				if (isNeedSearchSummaryExtend) {
					buffer.append(getEdocSummaryExtendHqlByCondition(paramMap, params));//按edocSummaryExtend表字段查询
				}
			}
			buffer.append(" )");
		}
		
		if(listType != null){
			if(listType.startsWith("listDone")) {//已办
				buffer.append(" order by affair.completeTime desc");
			} else if(listType.startsWith("listPending")) {//待办
				buffer.append(" order by affair.receiveTime desc");
			} else if(listType.startsWith("listWaitSend")) {//待发
				buffer.append(" order by summary.createTime desc");
			} else {
				buffer.append(" order by summary.createTime desc");
			}
		}
		
		if(flipInfo != null) {
			flipInfo.setNeedTotal(true);
		}
		List<Object[]> result = super.find(buffer.toString(), -1, -1, paramMap);
		
		if(Strings.isNotEmpty(result)) {
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				int n = vo.toObject(object, hasDeduplication);
				if (isNeedSearchSummaryExtend) {
					vo.toSummaryExtendObject(object, n);
				}
				voList.add(vo);
			}
		}
		
		return voList;
	}
	
	/**
	 * 发文/收文/签报登记簿数据展示及查询
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings("unchecked")
	public List<GovdocListVO> findRegisterList(FlipInfo flipInfo, Map<String, String> params) throws BusinessException {
		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		
		String listType = params.get("listType");
		Long orgAccountId = Long.parseLong(params.get("orgAccountId"));
		
		//公文统计自定义元素查询公文的扩展表
		boolean isNeedSearchSummaryExtend = false;
		if("true".equals(params.get("isNeedSearchSummaryExtend"))){
			isNeedSearchSummaryExtend = true;
		}
		
		Map<String, Object> paramMap = new HashMap<String, Object>();
		
		StringBuilder buffer = new StringBuilder();
		if("listRecRegister".equals(listType)) {
			//含自定义公文元素
			if (isNeedSearchSummaryExtend) {
				buffer.append("select " + RecRegisterAllColumnHqlSummaryExt);
				buffer.append(" from EdocSummary summary");
				buffer.append(" , GovdocRegister register");
				buffer.append(" , EdocSummaryExtend summaryExt");
			}else{
				buffer.append("select " + RecRegisterAllColumnHql);
				buffer.append(" from EdocSummary summary");
				buffer.append(" , GovdocRegister register");
			}
		} else {
             if (isNeedSearchSummaryExtend) {
                 buffer.append("select " + RegisterAllColumnHqlSummaryExt);
 				 buffer.append(" from EdocSummary summary");
 				 buffer.append(" , EdocSummaryExtend summaryExt");
			 }else{
				 buffer.append("select " + RegisterAllColumnHql);
				 buffer.append(" from EdocSummary summary");
			 }
		}
		buffer.append(" where summary.orgAccountId = :orgAccountId");
		if (isNeedSearchSummaryExtend) {
			buffer.append(" and summary.id = summaryExt.summaryId ");
		}
		paramMap.put("orgAccountId", orgAccountId);
		if("listRecRegister".equals(listType)) {
			buffer.append(" and summary.id = register.summaryId");
		}
		
		//分保-公文列表密级过滤
		if(SecretUtil.hasPluginSecret()) {
			buffer.append(getHqlBySummarySecret(paramMap, params,null));
		}
		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文GovdocType
		buffer.append(getHqlByGovdocType(paramMap, listType));
		//按条件查询
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
			//含自定义公文元素
			if (isNeedSearchSummaryExtend) {
				buffer.append(getEdocSummaryExtendHqlByCondition(paramMap, params));//按edocSummaryExtend表字段查询
			}
		}
		
		if(listType != null ){
			if(listType.startsWith("listDone")) {//已办
				buffer.append(" order by affair.completeTime desc");
			} else if(listType.startsWith("listPending")) {//待办
				buffer.append(" order by affair.receiveTime desc");
			} else if(listType.startsWith("listWaitSend")) {//待发
				buffer.append(" order by summary.createTime desc");
			} else {
				buffer.append(" order by summary.createTime desc");
			}
		}
		
		if(flipInfo != null) {
			//设置needTotal会导致m3端发文、收文登记簿数据重复加载的问题  liusy
		    //flipInfo.setNeedTotal(true);
			
		}
		
		List<Object[]> result = DBAgent.find(buffer.toString(), paramMap, flipInfo);
		
		if(Strings.isNotEmpty(result)) {
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				int n = vo.toRegister(object);
				int m = vo.toRegisterClob(object, n);
				//收文登记簿增加收文编号显示
				int dtx = m;
				if("listRecRegister".equals(listType)) {
					dtx =vo.toRecRegisterAdd(object,m);
				}
				if (isNeedSearchSummaryExtend) {
				   vo.toSummaryExtendObject(object, dtx);
				}
				voList.add(vo);
			}
		}
		
		return voList;
	}
	
	/**
	 * 管理员查询收发文的数量
	 * @param params
	 * @return
	 */
	public Integer getRegisterCount(Map<String, String> params){
		int count = 0;
		
		String listType = params.get("listType");
		Long orgAccountId = Long.parseLong(params.get("orgAccountId"));
		
		Map<String, Object> paramMap = new HashMap<String, Object>();
		
		StringBuilder buffer = new StringBuilder();
		if("listRecRegister".equals(listType)) {
			buffer.append("select " + RecRegisterAllColumnHql);
			buffer.append(" from EdocSummary summary");
			buffer.append(" , GovdocRegister register");
		} else {
			buffer.append("select " + RegisterAllColumnHql);
			buffer.append(" from EdocSummary summary");
		}
		buffer.append(" where summary.orgAccountId = :orgAccountId");
		paramMap.put("orgAccountId", orgAccountId);
		if("listRecRegister".equals(listType)) {
			buffer.append(" and summary.id = register.summaryId");
		}
		
		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文GovdocType
		buffer.append(getHqlByGovdocType(paramMap, listType));
		//按条件查询
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
		}
		
		count = DBAgent.count(buffer.toString(), paramMap);
		
		
		return count;
	}
	
	/**
	 * 公文交换-待发送列表数据数据展示及查询
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings("unchecked")
	public List<GovdocListVO> findSendPendingList(FlipInfo flipInfo, Map<String, String> params) throws BusinessException {
		Map<String, Object> paramMap = new HashMap<String, Object>();
		
		StringBuilder buffer = new StringBuilder();
		buffer.append("select " + ListAllColumnHql);
		buffer.append(" from EdocSummary summary, CtpAffair affair");
		buffer.append(" where summary.id =  affair.objectId ");
		
		//分保-公文列表密级过滤
		if(SecretUtil.hasPluginSecret()) {
			buffer.append(getHqlBySummarySecret(paramMap, params,null));
		}
		String listType = params.get("listType");
		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文app及subApp
		String govdocType = GovdocListTypeEnum.getGovdocTypeName(listType);
		buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true));
		//处理状态
		buffer.append(getHqlByAffairState(paramMap, listType));
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, Long.valueOf(params.get("memberId")), true));
		//按节点权限过滤
		buffer.append(getHqlByAffairPolicy(paramMap, params));
		//按条件查询
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
			buffer.append(getAffairHqlByCondition(paramMap, params));//按aff表字段查询
		}
		buffer.append(" order by affair.receiveTime desc");
		if(params.containsKey("onlyCount")){
			int cout = DBAgent.count(buffer.toString(), paramMap);
			flipInfo.setTotal(cout);
			return new ArrayList<GovdocListVO>();
		}
		List<Object[]> result = DBAgent.find(buffer.toString(), paramMap, flipInfo);

		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		if(Strings.isNotEmpty(result)) {
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				vo.toObject(object, false);
				voList.add(vo);
			}
		}
		
		return voList;
	}
	/**
	 * 公文交换-待发送列表数据总数查询
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	public int findSendPendingListCount(Map<String, String> params) throws BusinessException {
		int resultCount = 0;
		Map<String, Object> paramMap = new HashMap<String, Object>();
		
		StringBuilder buffer = new StringBuilder();
		buffer.append("select count(summary.id) from EdocSummary summary, CtpAffair affair");
		buffer.append(" where summary.id =  affair.objectId and affair.app = :app and affair.subApp in (:subApp) and affair.state = :affairState ");
		
		String listType = params.get("listType");
		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文app及subApp
		String govdocType = GovdocListTypeEnum.getGovdocTypeName(listType);
		buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true));
		//处理状态
		buffer.append(getHqlByAffairState(paramMap, listType));
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, Long.valueOf(params.get("memberId")), true));
		//按节点权限过滤
		buffer.append(getHqlByAffairPolicy(paramMap, params));
		//按条件查询
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
		}
		buffer.append(" order by affair.receiveTime desc");
		resultCount = DBAgent.count(buffer.toString(), paramMap);
		return resultCount;
	}
	
	
	
	/**
	 * 公文交换-已发送列表数据数据展示及查询
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings("unchecked")
	public List<GovdocListVO> findSendDoneList(FlipInfo flipInfo, Map<String, String> params) throws BusinessException {
		Map<String, Object> paramMap = new HashMap<String, Object>();
		
		StringBuilder buffer = new StringBuilder();
		buffer.append("select " + ListAllColumnHql);
		buffer.append(" from EdocSummary summary, CtpAffair affair");
		buffer.append(" where summary.exchangeSendAffairId = affair.id");
		
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, Long.valueOf(params.get("memberId")), true));
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
		}
		buffer.append(getHqlByMemberSecret(paramMap, params,"summary"));
		buffer.append(" order by affair.completeTime desc");
		List<Object[]> result = DBAgent.find(buffer.toString(), paramMap, flipInfo);

		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		if(Strings.isNotEmpty(result)) {
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				vo.toObject(object, false);
				voList.add(vo);
			}
		}
		
		return voList;
	}

	/**
	 * 公文交换-待签收列表数据数据展示及查询
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings({ "unchecked", "serial" })
	public List<GovdocListVO> findSignPendingList(FlipInfo flipInfo, Map<String, String> params) throws BusinessException {
		StringBuilder buffer = new StringBuilder();
		buffer.append("select " + ListAllColumnHql);
		buffer.append(" from EdocSummary summary");
		buffer.append(" , CtpAffair affair");
		buffer.append(" where summary.id =  affair.objectId");
		buffer.append(" and summary.exchangeType in (:exchangeType)");

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("exchangeType", new ArrayList<Integer>() {
			{
				add(GovdocExchangeTypeEnum.jiaohuan.ordinal());
				add(GovdocExchangeTypeEnum.zhuansw.ordinal());
			}
		});

		
		//分保-公文列表密级过滤
		if(SecretUtil.hasPluginSecret()) {
			buffer.append(getHqlBySummarySecret(paramMap, params,null));
		}
		String listType = params.get("listType");
		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文app及subApp
		String govdocType = GovdocListTypeEnum.getGovdocTypeName(listType);
		buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true));
		//处理状态
		buffer.append(getHqlByAffairState(paramMap, listType));
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, Long.valueOf(params.get("memberId")), true));
		//按节点权限过滤
		buffer.append(getHqlByAffairPolicy(paramMap, params));
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
			buffer.append(getAffairHqlByCondition(paramMap, params));//按aff表字段查询
		}
		buffer.append(" order by affair.receiveTime desc");
		List<Object[]> result = null;
		if(params.containsKey("onlyCount")){
			int cout = DBAgent.count(buffer.toString(), paramMap);
			flipInfo.setTotal(cout);
			return new ArrayList<GovdocListVO>();
		}
		if(flipInfo!=null) {
			result = DBAgent.find(buffer.toString(), paramMap, flipInfo);
		}else {
			result = DBAgent.find(buffer.toString(), paramMap);
		}


		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		if(Strings.isNotEmpty(result)) {
			List<Long> odsummaryIdList = new ArrayList<Long>();
			List<Long> odrecieveIdList = new ArrayList<Long>();
			List<Long> summaryIdList = new ArrayList<Long>();
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				vo.toObject(object, false);
				voList.add(vo);
				if(vo.getGovdocType() != null && vo.getGovdocType().intValue()!=0) {
					summaryIdList.add(vo.getSummaryId());
				} else {
					odsummaryIdList.add(vo.getSummaryId());
					odrecieveIdList.add(vo.getAffairSubObjectId());
				}
			}
			
			if(Strings.isNotEmpty(summaryIdList)) {
				result = findExchangeSignColumns(summaryIdList);
				if(Strings.isNotEmpty(result)) {
					for(Object[] object : result) {
						for(GovdocListVO vo : voList) {
							Long summaryId = ((Long)object[0]).longValue();
							if(vo.getSummaryId().longValue() == summaryId) {
								vo.toExObject(object);
							}
						}
					}
				}
			}
			
			if(Strings.isNotEmpty(odsummaryIdList)) {
				result = findOldExchangeSignColumns(odsummaryIdList, odrecieveIdList);
				if(Strings.isNotEmpty(result)) {
					for(Object[] object : result) {
						for(GovdocListVO vo : voList) {
							Long summaryId = ((Long)object[0]).longValue();
							if(vo.getSummaryId().longValue() == summaryId) {
								vo.toOldExObject(object);
							}
						}
					}
				}
			}
		}
		
		return voList;
	}
	/**
	 * 公文交换-待签收列表数据总条数
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings({ "serial" })
	public int findSignPendingListCount(Map<String, String> params) throws BusinessException {
		int resultCount = 0;
		StringBuilder buffer = new StringBuilder();
		buffer.append("select count(summary.id) from EdocSummary summary");
		buffer.append(" , CtpAffair affair");
		buffer.append(" where summary.id =  affair.objectId");
		buffer.append(" and summary.exchangeType in (:exchangeType)");
		
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("exchangeType", new ArrayList<Integer>() {
			{
				add(GovdocExchangeTypeEnum.jiaohuan.ordinal());
				add(GovdocExchangeTypeEnum.zhuansw.ordinal());
			}
		});
		
		String listType = params.get("listType");
		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文app及subApp
		String govdocType = GovdocListTypeEnum.getGovdocTypeName(listType);
		buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true));
		//处理状态
		buffer.append(getHqlByAffairState(paramMap, listType));
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, Long.valueOf(params.get("memberId")), true));
		//按节点权限过滤
		buffer.append(getHqlByAffairPolicy(paramMap, params));
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
		}
		buffer.append(" order by affair.receiveTime desc");
		
		resultCount = DBAgent.count(buffer.toString(), paramMap);
		return resultCount;
	}
	
	@SuppressWarnings("unchecked")
	public List<Object[]> findExchangeSignColumns(List<Long> summaryIdList) {
		String hql = "select detail.summaryId, detail.sendAccountId, detail.sendUnit, detail.recUserName, detail.recOrgName, detail.recTime, detail.createTime from GovdocExchangeDetail detail where detail.summaryId in (:summaryId)";
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("summaryId", summaryIdList);
		return DBAgent.find(hql, paramMap);
	}
	
	@SuppressWarnings("unchecked")
	public List<Object[]> findOldExchangeSignColumns(List<Long> summaryIdList, List<Long> odrecieveIdList) {
		String hql = "select detail.edocId, detail.sendUnit,detail.sendTo, detail.recUserId, detail.recTime from EdocRecieveRecord detail where detail.edocId in (:summaryId) and id in (:recieveId)";
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("summaryId", summaryIdList);
		paramMap.put("recieveId", odrecieveIdList);
		return DBAgent.find(hql, paramMap);
	}

	/**
	 * 公文交换-已签收列表数据数据展示及查询
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings({ "unchecked", "serial" })
	public List<GovdocListVO> findSignDoneList(FlipInfo flipInfo, Map<String, String> params) throws BusinessException {
		StringBuilder buffer = new StringBuilder();
		buffer.append("select " + ListAllColumnHql);
		buffer.append(" from EdocSummary summary");
		buffer.append(" , CtpAffair affair");
		buffer.append(" where summary.id =  affair.objectId");
		buffer.append(" and summary.exchangeType in (:exchangeType)");

		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("exchangeType", new ArrayList<Integer>() {
			{
				add(GovdocExchangeTypeEnum.jiaohuan.ordinal());
				add(GovdocExchangeTypeEnum.zhuansw.ordinal());
			}
		});
		
		//分保-公文列表密级过滤
		if(SecretUtil.hasPluginSecret()) {
			buffer.append(getHqlBySummarySecret(paramMap, params,null));
		}
		String listType = params.get("listType");
		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文app及subApp
		String govdocType = GovdocListTypeEnum.getGovdocTypeName(listType);
		buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true));
		//处理状态
		buffer.append(getHqlByAffairState(paramMap, listType));
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, Long.valueOf(params.get("memberId")), false));
		//按节点权限过滤
		buffer.append(getHqlByAffairPolicy(paramMap, params, "affair"));
		
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
		}
		buffer.append(getHqlByMemberSecret(paramMap, params,"summary"));
		buffer.append(" order by affair.completeTime desc");
		List<Object[]> result = DBAgent.find(buffer.toString(), paramMap, flipInfo);

		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		if(Strings.isNotEmpty(result)) {
			List<Long> odsummaryIdList = new ArrayList<Long>();
			List<Long> odrecieveIdList = new ArrayList<Long>();
			List<Long> summaryIdList = new ArrayList<Long>();
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				vo.toObject(object, false);
				voList.add(vo);
				if(vo.getGovdocType() != null && vo.getGovdocType().intValue()!=0) {
					summaryIdList.add(vo.getSummaryId());
				} else {
					odsummaryIdList.add(vo.getSummaryId());
					odrecieveIdList.add(vo.getAffairSubObjectId());
				}
			}
			
			if(Strings.isNotEmpty(summaryIdList)) {
				result = findExchangeSignColumns(summaryIdList);
				if(Strings.isNotEmpty(result)) {
					for(Object[] object : result) {
						for(GovdocListVO vo : voList) {
							Long summaryId = ((Long)object[0]).longValue();
							if(vo.getSummaryId().longValue() == summaryId) {
								vo.toExObject(object);
							}
						}
					}
				}
			}
			
			if(Strings.isNotEmpty(odsummaryIdList)) {
				result = findOldExchangeSignColumns(odsummaryIdList, odrecieveIdList);
				if(Strings.isNotEmpty(result)) {
					for(Object[] object : result) {
						for(GovdocListVO vo : voList) {
							Long summaryId = ((Long)object[0]).longValue();
							if(vo.getSummaryId().longValue() == summaryId) {
								vo.toOldExObject(object);
							}
						}
					}
				}
			}
		}
		
		return voList;
	}
	
	/**
	 * 公文交换-已回退列表数据数据展示及查询
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings("unchecked")
	public List<GovdocListVO> findFallbackList(FlipInfo flipInfo, Map<String, String> params) throws BusinessException {
		StringBuilder buffer = new StringBuilder();
		Map<String, Object> paramMap = new HashMap<String, Object>();
		buffer.append("select " + BackExchangeHql);
		buffer.append(" from EdocSummary summary,GovdocExchangeDetailLog log,GovdocExchangeMain main,GovdocExchangeDetail detail");
		buffer.append(" where summary.id = log.backSummaryId");
		buffer.append(" and log.detailId = detail.id ");
		buffer.append(" and detail.mainId = main.id");
		buffer.append(" and summary.transferStatus = :transferStatus");
		paramMap.put("transferStatus", TransferStatus.stepbackWaitSend.getKey());
		buffer.append(" and detail.recAccountId = :recAccountId ");
		buffer.append(" and main.type <> :type ");
		paramMap.put("type", 1);//不等于联合发文
		paramMap.put("recAccountId", Long.valueOf(params.get("orgAccountId")));
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
			if(Strings.isNotBlank(params.get("exchangeSendUnitId")) || Strings.isNotBlank(params.get("exchangeSendUnitName"))) {//按发文单位查询(无Affair关联)
				buffer.append(getExchangeHqlBySendUnitId(paramMap, params));
			}
		}
		buffer.append(getHqlByMemberSecret(paramMap, params,"summary"));
		buffer.append(" order by summary.completeTime desc");
		List<Object[]> result = DBAgent.find(buffer.toString(), paramMap, flipInfo);

		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		if(Strings.isNotEmpty(result)) {
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				vo.toExchangeFallback(object);
				voList.add(vo);
			}
		}
		
		return voList;
	}


	/**
	 * 公文交换-被回退列表数据数据展示及查询
	 * @param flipInfo
	 * @param params
	 * @return
	 * @throws BusinessException
	 */
	@SuppressWarnings("unchecked")
	public List<GovdocListVO> findReturnedFallbackList(FlipInfo flipInfo, Map<String, String> params) throws BusinessException {
		StringBuilder buffer = new StringBuilder();
		Map<String, Object> paramMap = new HashMap<String, Object>();
		buffer.append("select " + ReturndBackExchangeHql);
		buffer.append(" from GovdocExchangeDetailLog log,EdocSummary summary,GovdocExchangeDetail detail");
		buffer.append(" where log.backSummaryId = summary.id");
		buffer.append(" and log.detailId = detail.id");
		buffer.append(" and summary.transferStatus = :transferStatus");
		paramMap.put("transferStatus", TransferStatus.stepbackWaitSend.getKey());
		buffer.append(" and detail.sendAccountId = :sendAccountId");
		paramMap.put("sendAccountId", Long.valueOf(params.get("orgAccountId")));
		buffer.append(" and detail.status = :status");
		paramMap.put("status", GovdocEnum.ExchangeDetailStatus.hasBack.getKey());
		buffer.append(" and log.status = :logStatus");
		paramMap.put("logStatus", GovdocEnum.ExchangeDetailStatus.hasBack.getKey());
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
			if(Strings.isNotBlank(params.get("exchangeRecUnitName"))) {//按发文单位查询(无Affair关联)

				buffer.append(" and detail.recOrgName like :exchangeRecUnitName");
				paramMap.put("exchangeRecUnitName", "%" + params.get("exchangeRecUnitName") + "%");

			}
		}
		buffer.append(getHqlByMemberSecret(paramMap, params,"summary"));
		buffer.append(" order by summary.completeTime desc");
		List<Object[]> result = DBAgent.find(buffer.toString(), paramMap, flipInfo);

		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		if(Strings.isNotEmpty(result)) {
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				vo.toReturnedExchangeFallback(object);
				voList.add(vo);
			}
		}

		return voList;
	}
	public List<GovdocListVO> findCompeteDoneList(FlipInfo flipInfo, Map<String, String> params) {
		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		
		String listType = params.get("listType");
		String govdocType = params.get("govdocType");
		Long memberId = Long.parseLong(params.get("memberId"));
		boolean hasDeduplication = "true".equals(params.get("deduplication"));
		boolean hasAgent = "true".equals(params.get("agentFlag")) || "true".equals(params.get("agentToFlag"));
		
		Map<String, Object> paramMap = new HashMap<String, Object>();
		
		StringBuilder buffer = new StringBuilder();
		buffer.append("select " + ListAllColumnHql);
		buffer.append(" ,extend.competitiveMemberId ");
		buffer.append(" from EdocSummary summary");
		buffer.append(" , CtpAffair affair");
		buffer.append(" , CtpAffairExtend extend");
		buffer.append(" where summary.id = affair.objectId");
		buffer.append(" and affair.id = extend.id");
		buffer.append(" and extend.competitive = :competitive");
		paramMap.put("competitive", Boolean.TRUE);
		//公文流程状态
		buffer.append(getHqlBySummaryState(paramMap, listType));
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, memberId, hasAgent));
		//公文app及subApp
		buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true));
		//处理状态
		buffer.append(getHqlByAffairState(paramMap, listType));
		//按节点权限过滤
		buffer.append(getHqlByAffairPolicy(paramMap, params));
		//按节点权限(非)过滤
		buffer.append(getHqlByAffairNotInPolicy(paramMap, params));
		//按条件查询
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
			buffer.append(getHqlBySummaryTemplateIds(paramMap, params));//公文模板ID查询-来自业务生成器
			buffer.append(getAffairHqlByCondition(paramMap, params));//按affair表字段查询
		}
		
		if(hasDeduplication) {
			buffer.append(" and affair.id = (");
			buffer.append(" 	select max(affair2.id) from CtpAffair affair2, CtpAffairExtend extend2");
			buffer.append("  	where affair2.objectId = summary.id");
			buffer.append("  	and affair2.subApp = affair.subApp");
			buffer.append("     and affair2.id = extend2.id");
			buffer.append("     and extend2.competitive = :competitive");
			
			if(Strings.isNotBlank(params.get("competeTransactor"))) {//已办所属人
				if(paramMap.get("competitiveMemberId") instanceof ArrayList) {
					List<Long> competitiveMemberId = (List<Long>) paramMap.get("competitiveMemberId");
					if(competitiveMemberId.size() > 1) {
						buffer.append(" and  extend2.competitiveMemberId in (:competitiveMemberId)");
					} else {
						buffer.append(" and  extend2.competitiveMemberId = :competitiveMemberId ");
					}
				} else {
					buffer.append(" and  extend2.competitiveMemberId = -1 ");//not exit record;
				}
			}
			//公文处理人员过滤(包括代理人)
			buffer.append(getHqlByAffairMember(paramMap, memberId, hasAgent, "affair2"));
			//公文app及subApp
			buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true, "affair2"));
			//处理状态`
			buffer.append(getHqlByAffairState(paramMap, listType, "affair2"));
			//按节点权限过滤
			buffer.append(getHqlByAffairPolicy(paramMap, params, "affair2"));
			//按节点权限(非)过滤
			buffer.append(getHqlByAffairNotInPolicy(paramMap, params, "affair2"));
			//按条件查询
			if(params.containsKey("condition")) {
				buffer.append(getAffairHqlByCondition(paramMap, params, "affair2"));//按affair表字段查询
			}
			buffer.append(" )");
		}

		if("listCompeteDoneOther".equals(listType)){
			buffer.append("  and extend.competitiveMemberId <> :memberId");
		}

		buffer.append(" order by affair.updateDate desc");

		boolean dumpData = DumpDataVO.dataType.dumpData.getKey().equals(params.get("dumpData"));
		
		List<Object[]> result = null;
		if(flipInfo != null && !dumpData){// 查询转储数据必须要用super的find相关方法才能查询分库
			result = DBAgent.find(buffer.toString(), paramMap,flipInfo);
		}else{
			result = super.find(buffer.toString(), paramMap);
		}


		if(Strings.isNotEmpty(result)) {
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				vo.toObject(object, hasDeduplication);
				vo.setCompeteTransactorId(String.valueOf(object[object.length-1]));
				vo.setCompeteTransactor(vo.getCompeteTransactorId());
				voList.add(vo);
			}
		}

		return voList;
	}

	public List<GovdocListVO> findExchangeCompeteDoneList(FlipInfo flipInfo, Map<String, String> params) {
		Map<String, Object> paramMap = new HashMap<String, Object>();
		String listType = params.get("listType");


		StringBuilder buffer = new StringBuilder();
		buffer.append("select " + ListAllColumnHql);
		buffer.append(" ,extend.competitiveMemberId ");
		buffer.append(" from EdocSummary summary, CtpAffair affair");
		buffer.append(" , CtpAffairExtend extend");
		buffer.append(" where summary.id = affair.objectId");
		buffer.append(" and affair.id = extend.id");
		buffer.append(" and extend.competitive = :competitive");
		paramMap.put("competitive", Boolean.TRUE);
		buffer.append(" and extend.nodeType = :nodeType");
		if(listType.endsWith("Send")){
			paramMap.put("nodeType", CtpAffairExtend.NODETYPE_EXCHANGE_SEND);
		}else if(listType.endsWith("Sign")){
			paramMap.put("nodeType", CtpAffairExtend.NODETYPE_EXCHANGE_SIGN);
		}

		//公文app及subApp
		String govdocType = GovdocListTypeEnum.getGovdocTypeName(listType);
		buffer.append(getHqlByAffairApp(paramMap, govdocType, listType, true));
		//处理状态
		buffer.append(getHqlByAffairState(paramMap, listType));
		//公文处理人员过滤(包括代理人)
		buffer.append(getHqlByAffairMember(paramMap, Long.valueOf(params.get("memberId")), true));
		if(params.containsKey("condition")) {
			buffer.append(getHqlByCondition(paramMap, params));//按summary表字段查询
		}
		buffer.append(getHqlByMemberSecret(paramMap, params,"summary"));
		buffer.append(" order by affair.updateDate desc");
		List<Object[]> result = DBAgent.find(buffer.toString(), paramMap, flipInfo);

		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		if(Strings.isNotEmpty(result)) {
			for(Object[] object : result) {
				GovdocListVO vo = new GovdocListVO();
				vo.toObject(object, false);
				vo.setCompeteTransactorId(String.valueOf(object[object.length-1]));
				vo.setCompeteTransactor(vo.getCompeteTransactorId());
				vo.setReceiptDate(vo.getAffairUpdateDate());
				voList.add(vo);
			}
		}
		
		return voList;
	}
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public List<WorkflowData> getAdminWfDataList(FlipInfo flipInfo,Map<String,Object> conditionParam,long accountId,boolean isPage,FlipInfo fi) {
		Map<String,Object> paramMap = new HashMap<String, Object>();
		String operationTypeIdsStr =(String) conditionParam.get("operationTypeIds");
        String operationType = (String) conditionParam.get("operationType");
        int edocType = (Integer)conditionParam.get("edocType");
        String[] operationTypeIds = null;
        if(operationTypeIdsStr!=null&&!"".equals(operationTypeIdsStr.trim())){
            operationTypeIds = operationTypeIdsStr.split(",");
        }
        if(operationType!= null){
            if("".equals(operationType.trim())){
                operationType= null;
            }
        }
        int flowstate = 0;
        String flowstateStr = (String)conditionParam.get("flowstate");
        if(Strings.isNotBlank(flowstateStr)){
            flowstate = Integer.parseInt(flowstateStr);
        }
	    StringBuilder buffer = new StringBuilder();
	    buffer.append("select " + AdminManagerHql);
	    buffer.append(" from CtpAffair affair,EdocSummary summary,OrgUnit u");
	    if("true".equals(conditionParam.get("onlyObjectId"))){
	    	buffer.append(" where u.id = summary.orgDepartmentId and summary.id = affair.objectId and affair.objectId in (:objctIds)");
	    	paramMap.put("objctIds", conditionParam.get("objectIds"));
	    }else{
	    	boolean needTemplate = EdocWorkflowManageHandler.TEMPLETE_WORKFLOW.equals(operationType)&& (operationTypeIds == null|| operationTypeIds.length==0);
	    	if(needTemplate) {
	    		buffer.append(",CtpTemplate t ");
	    	}
	    	
	    	buffer.append(" where u.id = summary.orgDepartmentId and summary.id = affair.objectId ");
	    	//buffer.append(" and summary.edocType = :edocType");
	    	if(needTemplate) {
	    		buffer.append(" and t.id = summary.templeteId and t.system = :isSystem");
	    		paramMap.put("isSystem", true);
	    	}
	    	if(null != conditionParam.get("needOldEdoc") && "false".equals(conditionParam.get("needOldEdoc"))){
	    		buffer.append(" and summary.govdocType != 4 ");
	    	}


			buffer.append(" and summary.state = :summarystate");
			paramMap.put("summarystate", flowstate);
			boolean  selectQuickSendFlag = false;
			if((EdocWorkflowManageHandler.COMMON_WORKFLOW.equals(operationType)|| (Strings.isBlank(operationType) && AppContext.isGroupAdmin()))&& edocType == 0 && flowstate == 3)
			{
				selectQuickSendFlag = true;
			}

			if(!selectQuickSendFlag)
			{
				buffer.append(" and summary.isQuickSend = :isQuickSend");
				paramMap.put("isQuickSend", Boolean.FALSE);
			}
	    	
	    	buffer.append(" and affair.app = :app");
	    	paramMap.put("app", ApplicationCategoryEnum.edoc.key());
	    	//OA-178994 caseid为空的不用查询出来 和协同保持一致
	    	buffer.append(" and summary.caseId is not null");
	    	
	    	buffer.append(" and affair.subApp in (:subApp)");
	    	List<Integer> subAppList = new ArrayList<Integer>();
	    	if (edocType==0) {
	    		subAppList.add(ApplicationSubCategoryEnum.edoc_fawen.key());
	    		subAppList.add(ApplicationSubCategoryEnum.old_edocSend.key());
	    	} else if(edocType==2) {//签报
	    		subAppList.add(ApplicationSubCategoryEnum.edoc_qianbao.key());
	    		subAppList.add(ApplicationSubCategoryEnum.old_edocSign.key());
	    	}else {
	    		subAppList.add(ApplicationSubCategoryEnum.edoc_shouwen.key());
	    		subAppList.add(ApplicationSubCategoryEnum.edoc_jiaohuan.key());
	    		subAppList.add(ApplicationSubCategoryEnum.old_edocRec.key());
	    	}
	    	paramMap.put("subApp", subAppList);
	    	
	    	buffer.append(" and (affair.state = :sentstate or (affair.state = :waitstate and affair.subState = :waitsubstate))");
	    	paramMap.put("sentstate", StateEnum.col_sent.key());
	    	paramMap.put("waitstate", StateEnum.col_waitSend.key());
	    	paramMap.put("waitsubstate", SubStateEnum.col_pending_specialBacked.key());
	    	Map<String,List<Long>> senderMap = (Map<String,List<Long>>)conditionParam.get("senderMap");
	    	
	    	String senders = (String) conditionParam.get("senders");
	    	if(Strings.isNotBlank(senders)) {
	    		List<V3xOrgMember> memberList = GovdocOrgHelper.getMembersByTypeAndIds(senders);
				if (Strings.isNotEmpty(memberList)) {
					List<List<V3xOrgMember>> listGroup = Lists.partition(memberList, 1000);
					if (listGroup.size() > 1) {
						buffer.append(" and (");
						for (int i = 0; i < listGroup.size(); i++) {
							List<Long> memberIdList = transToMemberIds(listGroup.get(i));
							if (i > 0) {
								buffer.append(" or ");
							}
							buffer.append("summary.startUserId in (:sendMember" + i + ")");
							paramMap.put("sendMember" + i, memberIdList);
						}
						buffer.append(")");
					} else {
						List<Long> memberIdList = transToMemberIds(memberList);
						buffer.append(" and summary.startUserId in (:sendMember) ");
						paramMap.put("sendMember", memberIdList);
					}
				}
	    	}
	    	
	    	//单位类型， 查的自由流程或者 模板流程
	    	if(Strings.isNotBlank(operationType)){
	    		//如果发起对象没有选择，然而是查整个单位
	    		if(senderMap == null || senderMap.size() == 0) {
	    			buffer.append(" and summary.orgAccountId = :accountId ");
	    			paramMap.put("accountId",  accountId);
	    		}
	    		//自由流程
	    		if(EdocWorkflowManageHandler.COMMON_WORKFLOW.equals(operationType)){
	    			buffer.append(" and summary.templeteId is null ");
	    		} else { //模板流程
	    			buffer.append(" and summary.templeteId is not null ");
	    			if(operationTypeIds != null && operationTypeIds.length > 0){
	    				List<Long> templeteList = new ArrayList<Long>();
	    				for(String tid : operationTypeIds){
	    					templeteList.add(Long.parseLong(tid));
	    				}
	    				Map<String, Object> queryOldIdsParam = new HashMap<String, Object>();
	    				queryOldIdsParam.put("newTids", templeteList);
	    				List<Long> oldDataIds = DBAgent.find(queryOldDataIdByNewDataId, queryOldIdsParam);
	    				buffer.append(" and summary.templeteId in (:tid) ");
	    				templeteList.addAll(oldDataIds);
	    				paramMap.put("tid", templeteList);
	    			}
	    		}
	    	}
	    	String subject = (String) conditionParam.get("subject");
	    	if(Strings.isNotBlank(subject)) {
	    		buffer.append(" and summary.subject like :subject");
	    		paramMap.put("subject", "%" + SQLWildcardUtil.escape(subject.trim()) + "%");
	    	}
	    	Date beginDate = (Date)conditionParam.get("bDate");
	    	Date endDate = (Date)conditionParam.get("eDate");
	    	if(beginDate != null) {
	    		buffer.append(" and summary.createTime >= :beginTime");
	    		paramMap.put("beginTime", beginDate);
	    	}
	    	if(endDate != null) {
	    		buffer.append(" and summary.createTime <= :endTime");
	    		paramMap.put("endTime", endDate);
	    	}
	    	
	    	String deadlineBeginDateStr = (String) conditionParam.get("deadlineBeginDate");
	    	if(Strings.isNotEmpty(deadlineBeginDateStr)) {
	    		Date deadlineBeginDate = Datetimes.getTodayFirstTime(deadlineBeginDateStr);
	    		buffer.append(" and summary.deadlineDatetime >= :deadlineBeginDate");
	    		paramMap.put("deadlineBeginDate", deadlineBeginDate);
	    	}
	    	String deadlineEndDateStr = (String) conditionParam.get("deadlineEndDate");
	    	if(Strings.isNotEmpty(deadlineEndDateStr)) {
	    		Date deadlineEndDate = Datetimes.getTodayFirstTime(deadlineEndDateStr);
	    		buffer.append(" and summary.deadlineDatetime >= :deadlineEndDate");
	    		paramMap.put("deadlineEndDate", deadlineEndDate);
	    	}
	    	
	    	buffer.append(" order by summary.createTime desc ");
	    }
        
        List<WorkflowData> flowList = new ArrayList<WorkflowData>();

        List list = null;
        if(flipInfo != null){
        	list = DBAgent.find(buffer.toString(), paramMap, flipInfo);
        } else {
        	if(isPage) {
        		list = DBAgent.find(buffer.toString(), paramMap, fi);
	        } else {
	        	list = DBAgent.find(buffer.toString(), paramMap);
	        }
        }

        for(int i=0;i<list.size();i++) {
            Object[] obj = (Object[])list.get(i);
            WorkflowData flow = new WorkflowData();
            int j=0;
            int sub_app = -1;
            Object sub_app_obj = obj[j++];
            if(null != sub_app_obj){
            	sub_app =(Integer)sub_app_obj;
            }
            flow.setEdocType(-1);
            flow.setSummaryId(String.valueOf(obj[j++]));
            //应用类型
            int app = Integer.parseInt(String.valueOf(obj[j++]));
            flow.setAppEnum(app);
            String appStr = "";
            if(app == ApplicationCategoryEnum.edocSend.key()){ //TODO 增加一个判断新公文 和老公文的区别，同时区分收文和发文
                appStr = ResourceUtil.getString("edoc.docmark.inner.send");
            }else if(app==ApplicationCategoryEnum.edocRec.key()){
                appStr = ResourceUtil.getString("edoc.docmark.inner.receive");
            }else{
                appStr = ResourceUtil.getString("edoc.docmark.inner.signandreport");
            }

//            flow.setAppType(String.valueOf(obj[j++]));
            flow.setSubject(String.valueOf(obj[j++]));
            
            Long startUserId = (Long)(obj[j++]);
            if(startUserId != null) {
                V3xOrgMember member = null;
                try {
                    member = orgManager.getMemberById(startUserId);
                } catch (BusinessException e) {
                }
                if(member != null) {
                    flow.setInitiator(member.getName());
                }
            }
            flow.setDepName(String.valueOf(obj[j++]));
			flow.setQuickSend(Boolean.valueOf(String.valueOf(obj[j++])));
            flow.setSendTime(Timestamp.valueOf(String.valueOf(obj[j++])));
            flow.setProcessId(String.valueOf(obj[j++]));
            flow.setCaseId((Long)(obj[j++]));
            if(obj[j] !=null){
                Long deadline = (Long)(obj[j]);
                flow.setDeadLine(deadline);
            }
            j++;
            flow.setAdvanceRemind((Long)(obj[j++]));
            flow.setAccountId((Long)(obj[j++]));
//            long end = (Long)(obj[j++]);
//            flow.setEndFlag((int)end);
            j++;
            Long templeteId = (Long)(obj[j++]);
            if(templeteId!= null){
                flow.setIsFromTemplete(true) ;
                flow.setTempleteId(templeteId);
            }
            if(edocType == 1){
                flow.setAppEnumStr(ApplicationCategoryEnum.edocRec.name());
            }else if(edocType == 0){
                flow.setAppEnumStr(ApplicationCategoryEnum.edocSend.name());
            }else if(edocType == 2){
                flow.setAppEnumStr(ApplicationCategoryEnum.edocSign.name());
            }
            if(app == ApplicationCategoryEnum.edoc.key() && sub_app != -1){
            	if(sub_app == ApplicationSubCategoryEnum.edoc_fawen.key()){//發文
            		appStr = ResourceUtil.getString("edoc.docmark.inner.send");
            		flow.setAppEnumStr(ApplicationCategoryEnum.govdocSend.name());
            		flow.setEdocType(FormType.govDocSendForm.getKey());
            		flow.setNps(ApplicationCategoryEnum.govdocSend.name());
            	}else if(sub_app == ApplicationSubCategoryEnum.edoc_shouwen.key()){//收文
            		appStr = ResourceUtil.getString("edoc.docmark.inner.receive");
            		flow.setAppEnumStr(ApplicationCategoryEnum.govdocRec.name());
            		flow.setEdocType(FormType.govDocReceiveForm.getKey());
            		flow.setNps(ApplicationCategoryEnum.govdocRec.name());
            	}else if(sub_app == ApplicationSubCategoryEnum.edoc_qianbao.key()){//签报
            		appStr = ResourceUtil.getString("edoc.docmark.inner.signandreport");
            		flow.setAppEnumStr(ApplicationCategoryEnum.govdocSign.name());
            		flow.setEdocType(FormType.govDocSignForm.getKey());
            		flow.setNps(ApplicationCategoryEnum.govdocSign.name());
            	}else if(sub_app == ApplicationCategoryEnum.edocSend.getKey()){//發文
            		appStr = ResourceUtil.getString("edoc.docmark.inner.send");
            		flow.setAppEnumStr(ApplicationCategoryEnum.edocSend.name());
            		flow.setEdocType(FormType.govDocSendForm.getKey());
            		flow.setNps(ApplicationCategoryEnum.edocSend.name());
            	}else if(sub_app == ApplicationCategoryEnum.edocRec.getKey()){//收文
            		appStr = ResourceUtil.getString("edoc.docmark.inner.receive");
            		flow.setAppEnumStr(ApplicationCategoryEnum.edocRec.name());
            		flow.setEdocType(FormType.govDocReceiveForm.getKey());
            		flow.setNps(ApplicationCategoryEnum.edocRec.name());
            	}else if(sub_app == ApplicationCategoryEnum.edocSign.getKey()){//签报
            		appStr = ResourceUtil.getString("edoc.docmark.inner.signandreport");
            		flow.setAppEnumStr(ApplicationCategoryEnum.edocSign.name());
            		flow.setEdocType(FormType.govDocSignForm.getKey());
            		flow.setNps(ApplicationCategoryEnum.edocSign.name());
            	}else{
            		appStr = ResourceUtil.getString("edoc.docmark.inner.receive");
            		flow.setAppEnumStr(ApplicationCategoryEnum.govdocRec.name());
            		flow.setEdocType(FormType.govDocExchangeForm.getKey());
            		flow.setNps(ApplicationCategoryEnum.govdocSend.name());
            	}
            }
            flow.setAppType(appStr);
            flow.setAppEnumStr("edoc");
            EdocSummary summary = new EdocSummary();
            String completeTime = String.valueOf(obj[j++]);
            if(Strings.isNotBlank(completeTime)&& !"null".equals(completeTime)){
            	summary.setCompleteTime(Timestamp.valueOf(completeTime));
            }
            Date deadlineDatetime=(Date)obj[j++];
            //设置流程期限显示内容
            flow.setDeadlineDatetimeName(WFComponentUtil.getDeadLineName(deadlineDatetime));
            String currentNodesInfo = (String)(obj[j++]);
            if(flowstate == flowState.terminate.ordinal() || flowstate == flowState.finish.ordinal()){
    			currentNodesInfo = "";
    		}else if(Strings.isNotBlank(currentNodesInfo) && currentNodesInfo.indexOf(";;") != -1){
            	currentNodesInfo = currentNodesInfo.replaceAll(";;", ";");
            }
            summary.setCurrentNodesInfo(currentNodesInfo);
            flow.setCurrentNodesInfo(GovdocHelper.parseCurrentNodesInfo(summary));
            flow.setSecretLevel(String.valueOf((obj[j++])));
            
            Object formAppIdObj = obj[j++];
            flow.setFormAppId(formAppIdObj == null ? null : ((Number)formAppIdObj).longValue());
            
            Object formRecordIdObj = obj[j++];
            flow.setFormRecordId(formRecordIdObj == null ? null : ((Number)formRecordIdObj).longValue());
            if(flowstate == 1 || flowstate == 3){
               	flow.setEndFlag(0); //0 - 结束
            }
            flowList.add(flow);
        }
	    return flowList;
	}
	
	
	/************************ 以下为公文列表辅助Hql方法 start ***************************/
	//分保-公文列表密级过滤
	private String getHqlBySummarySecret(Map<String, Object> paramMap, Map<String, String> params,Long memberId) {
		StringBuilder buffer = new StringBuilder();
		Integer secretLevel = 100;
		String secret = params.get("secretLevel");
		if(memberId != null) {
			V3xOrgMember member = GovdocOrgHelper.getMemberById(memberId);
			if(member != null && member.getSecretLevel()!=null) {
				secretLevel = member.getSecretLevel();
			}
		} else {
			User user = AppContext.getCurrentUser();
			if(user != null) {
				secretLevel = SecretUtil.getSecretLevelOfCurrentUser();
			}
		}
		if(Strings.isNotBlank(secret)){
			int secretLevelInt=Integer.parseInt(secret);
			//如果密级超过当前人员的密级，查询密级设置为-1，查不出数据
			if(secretLevel< secretLevelInt){
				secretLevelInt=-1;
			}
			secretLevel=secretLevelInt;
		}
		if(secretLevel==null){
			secretLevel=0;
		}
		buffer.append(" and summary.edocSecretLevel <= :secretLevelList");
		paramMap.put("secretLevelList", Integer.valueOf(secretLevel));
		return buffer.toString();
	}
	private String getHqlBySummaryState(Map<String, Object> paramMap, String listType) {
		StringBuilder buffer = new StringBuilder();
		//公文流程状态
		if(listType != null) {
	        String flowState = GovdocListTypeEnum.getFlowStateName(listType);
			if(Strings.isNotBlank(flowState) && !"-1".equals(flowState)) {
				List<Integer> flowStateList = new ArrayList<Integer>();
				String[] flowStates = flowState.split(",");
				if(flowStates != null) {
					for(int i=0; i<flowStates.length; i++) {
						flowStateList.add(Integer.parseInt(flowStates[i]));
					}
				}
				if(flowStateList.size() == 1) {
					buffer.append(" and summary.state = :summaryState");
					paramMap.put("summaryState", flowStateList.get(0));
				} else {
					buffer.append(" and summary.state in (:summaryState)");
					paramMap.put("summaryState", flowStateList);
				}
			}
			//已发列表不查联合发文数据
			if(GovdocListTypeEnum.listSent.getKey().equals(listType)) {
				buffer.append(" and (summary.exchangeType is null or summary.exchangeType <> :exchangeType)");
				paramMap.put("exchangeType", GovdocExchangeMain.EXCHANGE_TYPE_LIANHE);
			}
		} else {
			List<Integer> flowStateList = new ArrayList<Integer>();
			flowStateList.add(EdocConstant.flowState.run.ordinal());
			flowStateList.add(EdocConstant.flowState.finish.ordinal());
			flowStateList.add(EdocConstant.flowState.terminate.ordinal());
			buffer.append(" and summary.state in (:summaryState)");
			paramMap.put("summaryState", flowStateList);
		}
		return buffer.toString();
	}	
	private String getHqlBySummaryTemplateIds(Map<String, Object> paramMap, Map<String, String> params) {
		StringBuilder buffer = new StringBuilder();
		//公文流程状态
        String templateIds = params.get("templateIds") == null ? null : (String)params.get("templateIds");
		if(Strings.isNotBlank(templateIds)) {
			List<Long> templateIdList = new ArrayList<Long>();
			String[] templateIdArr = templateIds != null ? templateIds.split(",") : null;
			if(templateIdArr != null) {
				for(int i=0; i<templateIdArr.length; i++) {
					templateIdList.add(Long.parseLong(templateIdArr[i]));
				}
				if(templateIdList.size() == 1) {
					buffer.append(" and summary.templeteId = :templeteId");
					paramMap.put("templeteId", templateIdList.get(0));
				} else {
					buffer.append(" and summary.templeteId in (:templeteId)");
					paramMap.put("templeteId", templateIdList);
				}
			}
		}
		return buffer.toString();
	}
	private String getHqlByGovdocType(Map<String, Object> paramMap, String listType) {
		StringBuilder buffer = new StringBuilder();
		if(!"listRecRegister".equals(listType)) {//发文/签报登记簿
			buffer.append(" and (");
			buffer.append(" 	summary.govdocType in (:govdocType) ");
			buffer.append(" 	or (summary.govdocType = :oldgovdocType and summary.edocType in (:edocType))");
			buffer.append(" )");
	        
			paramMap.put("oldgovdocType", ApplicationSubCategoryEnum.old_summary.getKey());
			paramMap.put("edocType", OldEdocTypeEnum.rec.ordinal());
	        //签报数据
			if("listSignRegister".equals(listType)) {
				paramMap.put("govdocType", ApplicationSubCategoryEnum.edoc_qianbao.getKey());
				paramMap.put("edocType", OldEdocTypeEnum.sign.ordinal());
			} else {
				paramMap.put("edocType", OldEdocTypeEnum.send.ordinal());
				paramMap.put("govdocType", ApplicationSubCategoryEnum.edoc_fawen.getKey());
			}
        }
		return buffer.toString();
	}
	
	private String getHqlByAffairApp(Map<String, Object> paramMap, String govdocType, String listType, boolean hasExData) {
		return getHqlByAffairApp(paramMap, govdocType, listType, hasExData, "affair");
	}
	private String getHqlByAffairApp(Map<String, Object> paramMap, String govdocType, String listType, boolean hasExData, String affairTable) {
		StringBuilder buffer = new StringBuilder();
		//公文处理状态
		List<Integer> subAppList = new ArrayList<Integer>();
		if(Strings.isNotBlank(govdocType)) {
			String[] govdocTypes = govdocType.split(",");
			for(String type : govdocTypes) {
				Integer subApp = Integer.parseInt(type);
				if(!subAppList.contains(subApp)) {
					subAppList.add(subApp);
				}
				if(subApp.intValue() == ApplicationSubCategoryEnum.edoc_fawen.key()) {//发文管理
					subAppList.add(ApplicationSubCategoryEnum.old_edocSend.key());
					if(hasExData) {
						subAppList.add(ApplicationSubCategoryEnum.old_exSend.key());
					}
				} else if(subApp.intValue() == ApplicationSubCategoryEnum.edoc_shouwen.key()) {//收文管理
					subAppList.add(ApplicationSubCategoryEnum.old_edocRec.key());
				} else if(subApp.intValue() == ApplicationSubCategoryEnum.edoc_qianbao.key()) {//签报管理
					subAppList.add(ApplicationSubCategoryEnum.old_edocSign.key());
				} else if(subApp.intValue() == ApplicationSubCategoryEnum.edoc_jiaohuan.key()) {//交换列表
					if(hasExData) {
						if(listType.startsWith("listSent") || listType.startsWith("listWaitSend")) {//收文已发没有交换数据，因为状态数据的state为4，数据到已办中去了
							continue;
						}
						subAppList.add(ApplicationSubCategoryEnum.old_exSign.key());
						subAppList.add(ApplicationSubCategoryEnum.old_edocRegister.key());
						subAppList.add(ApplicationSubCategoryEnum.old_edocRecDistribute.key());
					}
				}
			}
		} else {//公文待办/已办/待发/已发列表
			if(listType.startsWith("listSent") || listType.startsWith("listWaitSend")) {//公文待发/已发列表不展示交换已发/待发数据
				subAppList.add(ApplicationSubCategoryEnum.edoc_fawen.key());
				subAppList.add(ApplicationSubCategoryEnum.edoc_shouwen.key());
				subAppList.add(ApplicationSubCategoryEnum.edoc_qianbao.key());
				subAppList.add(ApplicationSubCategoryEnum.old_edocSend.key());
				subAppList.add(ApplicationSubCategoryEnum.old_edocRec.key());
				subAppList.add(ApplicationSubCategoryEnum.old_edocSign.key());
			}
		}
		
		//新老公文统一app为4
		buffer.append(" and "+affairTable+".app = :affariApp");
		paramMap.put("affariApp", ApplicationCategoryEnum.edoc.key());
		
		//新老公文区分subApp
		if(Strings.isNotEmpty(subAppList)) {
			if(subAppList.size() == 1) {
				buffer.append(" and "+affairTable+".subApp = :affairSubApp");
				paramMap.put("affairSubApp", subAppList.get(0));
			} else {
				buffer.append(" and "+affairTable+".subApp in (:affairSubApp)");
				paramMap.put("affairSubApp", subAppList);
			}
		}
		return buffer.toString();
	}
	
	protected String getHqlByAffairMember(Map<String, Object> paramMap, Long memberId, boolean hasAgent) {
		return getHqlByAffairMember(paramMap, memberId, hasAgent, "affair");
	}

	private String getHqlByAffairMember(Map<String, Object> paramMap, Long memberId, boolean hasAgent, String affairTable) {
		StringBuilder buffer = new StringBuilder();
		buffer.append(" and " + affairTable + ".delete = :isDelete");
		paramMap.put("isDelete", Boolean.FALSE);

		buffer.append(" and (" + affairTable + ".memberId=:memberId");
		paramMap.put("memberId", memberId);
		if (hasAgent) {
			// 以当前登录用户为判断标准，首先判断当前用户是否为代理人(即当前用户替别人干活)，如果当前登录用户为代理人，则用!affair.getMemberId().equals(userId)来判断设值
			// 如果当前登录用户不是代理人，则判断其是否是被代理人，如果是被代理人，则判断其设置的代理人是否有效(即：affair事项处于代理期限内)
			// 查询我所代理的人(我为代理人，我给别人干活)列表
			// 以前的方式有性能问题，查询方式和协同保持一致 OA-184924
			buffer.append(" or " + affairTable + ".proxyMemberId=:proxyMemberId");
			paramMap.put("proxyMemberId", memberId);
		}
		buffer.append(")");
		return buffer.toString();
	}
	
	private String getHqlByAffairState(Map<String, Object> paramMap, String listType) {
		return getHqlByAffairState(paramMap, listType, "affair");
	}
	@SuppressWarnings("unchecked")
	private String getHqlByAffairState(Map<String, Object> paramMap, String listType, String affairTable) {
		StringBuilder buffer = new StringBuilder();
		String affairState = GovdocListTypeEnum.getStateName(listType);
		if(Strings.isNotBlank(affairState) && !"-1".equals(affairState)) {
			List<Integer> stateList = new ArrayList<Integer>();//公文处理状态
			String[] states = affairState.split(",");
			if(states != null) {
				for(int i=0; i<states.length; i++) {
					stateList.add(Integer.parseInt(states[i]));
				}
			}
			
			List<Integer> affairSubAppList = new ArrayList<Integer>();
			Object affairSubApp = paramMap.get("affairSubApp");
			if(affairSubApp != null) {
				if(affairSubApp instanceof List) {
					affairSubAppList.addAll((List<Integer>)affairSubApp);	
				} else {
					affairSubAppList.add((Integer)affairSubApp);
				}
			}
			
			buffer.append(" and (");
			if(stateList.contains(2) && affairSubAppList.contains(4)) {//已发&交换
				Integer sent = StateEnum.col_sent.key();
				Integer exchange = ApplicationSubCategoryEnum.edoc_jiaohuan.key();
				stateList.remove(sent);
				affairSubAppList.remove(exchange);

				buffer.append("	    ("+affairTable+".state = :stateSent");
				paramMap.put("stateSent", sent);
				
				if(Strings.isNotEmpty(affairSubAppList)) {
					if(affairSubAppList.size() == 1) {
						buffer.append("  and "+affairTable+".subApp = :subAppSent");	
						paramMap.put("subAppSent", affairSubAppList.get(0));
					} else {
						buffer.append("  and "+affairTable+".subApp in (:subAppSent)");
						paramMap.put("subAppSent", affairSubAppList);
					}
				}
				buffer.append("	    )");//已发end
				
				if(Strings.isNotEmpty(stateList)) {
					buffer.append("	or");
				}
			}//已发&交换 end
		
			if(Strings.isNotEmpty(stateList)) {
				if(stateList.size() == 1) {
					buffer.append(" " +affairTable+".state = :affairState");
					paramMap.put("affairState", stateList.get(0));
				} else {
					buffer.append(" "+affairTable+".state in (:affairState)");
					paramMap.put("affairState", stateList);	
				}
			}
			buffer.append(" )");//and end
			
		}
		
		//公文处理子状态
		List<Integer> substateList = new ArrayList<Integer>();
		String substate = GovdocListTypeEnum.getSubStateName(listType);
		if(Strings.isNotBlank(substate) && !"-1".equals(substate)) {
			String[] substates = substate.split(",");
			if(substates != null) {
				for(int i=0; i<substates.length; i++) {
					substateList.add(Integer.parseInt(substates[i]));
				}
			}
			if(Strings.isNotEmpty(substateList)) {
				if(substateList.size() == 1) {
					buffer.append(" and "+affairTable+".subState = :affairSubState");
					paramMap.put("affairSubState", substateList.get(0));
				} else {
					buffer.append(" and "+affairTable+".subState in (:affairSubState)");
					paramMap.put("affairSubState", substateList);
				}
			}
		}
		//已指定回退数据不在待办列表中查询--同步协同查询
		if(listType.contains("listPending") || listType.contains("listReading")){
			buffer.append(" and "+affairTable+".subState not in (:specialBackSubState)");
			List<Integer> specialBackSubState = new ArrayList<Integer>();
			specialBackSubState.add(SubStateEnum.col_pending_specialBack.getKey());
			specialBackSubState.add(SubStateEnum.col_pending_specialBackCenter.getKey());
			paramMap.put("specialBackSubState",specialBackSubState);
		}
		
		return buffer.toString();
	}
	
	private String getHqlByAffairPolicy(Map<String, Object> paramMap, Map<String, String> params) {
		return getHqlByAffairPolicy(paramMap, params, "affair");
	}
	private String getHqlByAffairPolicy(Map<String, Object> paramMap, Map<String, String> params, String affairTable) {
		StringBuilder buffer = new StringBuilder();
		if(Strings.isNotBlank(params.get("nodePolicy"))) {
			List<String> nodePolicyList = new ArrayList<String>();
			for(String nodePolicy : params.get("nodePolicy").split(",")) {
				nodePolicyList.add(nodePolicy);
			}
			if (nodePolicyList.size() == 1) {
				buffer.append(" and "+affairTable+".nodePolicy = :nodePolicy");
				paramMap.put("nodePolicy", nodePolicyList.get(0));
			}else{
				buffer.append(" and "+affairTable+".nodePolicy in (:nodePolicy)");
				paramMap.put("nodePolicy", nodePolicyList);
			}
		}
		return buffer.toString();
	}
	
	private String getHqlByAffairNotInPolicy(Map<String, Object> paramMap, Map<String, String> params) {
		return getHqlByAffairNotInPolicy(paramMap, params, "affair");
	}
	private String getHqlByAffairNotInPolicy(Map<String, Object> paramMap, Map<String, String> params, String affairTable) {
		StringBuilder buffer = new StringBuilder();
		if(Strings.isNotBlank(params.get("notInNodePolicy"))) {
			buffer.append(" and "+affairTable+".nodePolicy not in (:notInNodePolicy)");
			List<String> notInNodePolicyList = new ArrayList<String>();
			for(String notInNodePolicy : params.get("notInNodePolicy").split(",")) {
				notInNodePolicyList.add(notInNodePolicy);
			}
			paramMap.put("notInNodePolicy", notInNodePolicyList);
		}
		return buffer.toString();
	}

	/**
	 * 按summary表字段查询
	 * @param paramMap  hql的参数
	 * @param params  传递进来的条件
	 * @return
	 */
	protected String getHqlByCondition(Map<String, Object> paramMap, Map<String, String> params) {
		StringBuilder buffer = new StringBuilder();
		if(params.containsKey("condition")) {
			//处理领导批示编号查询
        	if (Strings.isNotBlank(params.get("pishiName")) || Strings.isNotBlank(params.get("pishiYear")) || Strings.isNotBlank(params.get("pishiNo"))) {
        		List<Long> summaryIdList = getSummaryIdByPishi(params);
        		if(Strings.isNotEmpty(summaryIdList)) {
        			if(summaryIdList.size() < 1000) {
        				buffer.append(" and summary.id in (:summaryIds)");
        				paramMap.put("summaryIds", summaryIdList);
        			} else {//分页，oracle不支持in中超过1000

        			}
        		}else{
        			buffer.append(" and 1=0");
        		}
        	}
        	if(Strings.isNotBlank(params.get("subject"))) {//按标题查询
        		//BUG_紧急_V5_G6 V7.1sp1__南水北调中线干线工程建设管理局_公文管理-收文管理的已办中搜索配置的标题搜不到_20191101096701_2019-11-01

				buffer.append(" and summary.subject like :subject");

				paramMap.put("subject", "%" + GovdocUtil.convertSpecialChat(params.get("subject")) + "%");
			}
			if(Strings.isNotBlank(params.get("startUserId"))) {//按发起人查询(无Affair关联)
				buffer.append(getHqlByStartMemberId(paramMap, params));
			}
			if(Strings.isNotBlank(params.get("docMark"))) {//按公文文号查询
				buffer.append(" and summary.docMark like :docMark");
				paramMap.put("docMark", "%" + GovdocUtil.convertSpecialChat(params.get("docMark")) + "%");
			}
			if(Strings.isNotBlank(params.get("serialNo"))) {//按内部文号查询
				buffer.append(" and summary.serialNo like :serialNo");
				paramMap.put("serialNo", "%" + GovdocUtil.convertSpecialChat(params.get("serialNo")) + "%");
			}
			if(Strings.isNotBlank(params.get("hasArchive"))){//按是否归档查询
				buffer.append(" and summary.hasArchive = :hasArchive");
				String hasArchive = params.get("hasArchive").toString();
				if("1".equals(hasArchive)){
					paramMap.put("hasArchive", true);
					//GOVB-4267  公文查询，按照是否归档查询，查询出的数据不正确，显示了归档，但是无归档路径，点击进入其实又未归档 
					if(Strings.isNotBlank(params.get("searchCustomType")) &&"edocSearch".equals(params.get("searchCustomType"))){
						buffer.append(" and summary.archiveId IS NOT NULL");
					} 
				}else if("0".equals(hasArchive)){
					paramMap.put("hasArchive", false);
				}
			}
			if(Strings.isNotBlank(params.get("signMark"))) {//按内部文号查询
				buffer.append(" and summary.docMark2 like :signMark");
				paramMap.put("signMark", "%" + GovdocUtil.convertSpecialChat(params.get("signMark")) + "%");
			}
			if(Strings.isNotBlank(params.get("phone"))) {//按联系电话查询
				buffer.append(" and summary.phone like :phone");
				paramMap.put("phone", "%" + GovdocUtil.convertSpecialChat(params.get("phone")) + "%");
			}
			if(Strings.isNotBlank(params.get("sendUnit"))) {//按发文单位查询
				buffer.append(" and summary.sendUnit like :sendUnit");
				paramMap.put("sendUnit", "%" + GovdocUtil.convertSpecialChat(params.get("sendUnit")) + "%");
			}
			if(Strings.isNotBlank(params.get("sendDepartment"))){//按发文部门查询
				buffer.append(" and summary.sendDepartment like :sendDepartment");
				paramMap.put("sendDepartment", "%" + GovdocUtil.convertSpecialChat(params.get("sendDepartment")) + "%");
			}
			if(Strings.isNotBlank(params.get("sendTo"))) {//按主送单位查询
				buffer.append(" and summary.sendTo like :sendTo");
				paramMap.put("sendTo", "%" + GovdocUtil.convertSpecialChat(params.get("sendTo")) + "%");
			}
			if(Strings.isNotBlank(params.get("createPerson"))) {//按拟稿人查询
				buffer.append(" and summary.createPerson like :createPerson");
				paramMap.put("createPerson", "%" + GovdocUtil.convertSpecialChat(params.get("createPerson")) + "%");
			}
			if(Strings.isNotBlank(params.get("auditor"))) {//按审核人查询
				buffer.append(" and summary.auditor like :auditor");
				paramMap.put("auditor", "%" + GovdocUtil.convertSpecialChat(params.get("auditor")) + "%");
			}
			if(Strings.isNotBlank(params.get("review"))) {//按复核人查询
				buffer.append(" and summary.review like :review");
				paramMap.put("review", "%" + GovdocUtil.convertSpecialChat(params.get("review")) + "%");
			}
			if(Strings.isNotBlank(params.get("undertaker"))) {//按承办人查询
				buffer.append(" and summary.undertaker like :undertaker");
				paramMap.put("undertaker", "%" + GovdocUtil.convertSpecialChat(params.get("undertaker")) + "%");
			}
			if(Strings.isNotBlank(params.get("undertakenoffice"))) {//按承办机构查询
				buffer.append(" and summary.undertakenoffice like :undertakenoffice");
				paramMap.put("undertakenoffice", "%" + GovdocUtil.convertSpecialChat(params.get("undertakenoffice")) + "%");
			}
			if(Strings.isNotBlank(params.get("issuer"))) {//按签发人查询
				buffer.append(" and summary.issuer like :issuer");
				paramMap.put("issuer", "%" + GovdocUtil.convertSpecialChat(params.get("issuer")) + "%");
			}
			if(Strings.isNotBlank(params.get("printer"))) {//按打印人查询
				buffer.append(" and summary.printer like :printer");
				paramMap.put("printer", "%" + GovdocUtil.convertSpecialChat(params.get("printer")) + "%");
			}
			if(Strings.isNotBlank(params.get("printUnit"))) {//按印发单位查询
				buffer.append(" and summary.printUnit like :printUnit");
				paramMap.put("printUnit", "%" + GovdocUtil.convertSpecialChat(params.get("printUnit")) + "%");
			}
			if(Strings.isNotBlank(params.get("sendType"))) {//按行文类型查询
				buffer.append(" and summary.sendType like :sendType");
				paramMap.put("sendType", "%" + GovdocUtil.convertSpecialChat(params.get("sendType")) + "%");
			}
			if(Strings.isNotBlank(params.get("docType"))) {//按公文种类查询
				buffer.append(" and summary.docType = :docType");
				paramMap.put("docType",  GovdocUtil.convertSpecialChat(params.get("docType")));
			}
			if(Strings.isNotBlank(params.get("secretLevel"))) {//按密级查询
				/*buffer.append(" and summary.secretLevel like :secretLevel");
				paramMap.put("secretLevel", "%" + GovdocUtil.convertSpecialChat(params.get("secretLevel")) + "%");*/

				//2019-12-19:按产品意思修改：无，则查出选择了无和没选的
				String secVal=GovdocUtil.convertSpecialChat(params.get("secretLevel"));
				if("1".equals(secVal)){
					List<String> secretList=new ArrayList<String>(2);
					secretList.add("0");
					secretList.add("1");
					buffer.append(" and summary.secretLevel in (:secretLevel)");
					paramMap.put("secretLevel", secretList);
				}else{
					buffer.append(" and summary.secretLevel = :secretLevel");
					paramMap.put("secretLevel", secVal);
				}
			}
			if(Strings.isNotBlank(params.get("urgentLevel"))) {//按紧急程序查询
				buffer.append(" and summary.urgentLevel = :urgentLevel");
				paramMap.put("urgentLevel", GovdocUtil.convertSpecialChat(params.get("urgentLevel")));
			}
			if(Strings.isNotBlank(params.get("unitLevel"))) {//按公文级别查询
				buffer.append(" and summary.unitLevel = :unitLevel");
				paramMap.put("unitLevel", GovdocUtil.convertSpecialChat(params.get("unitLevel")));
			}
			if(Strings.isNotBlank(params.get("keepPeriod"))) {//按保密期限查询
				buffer.append(" and summary.keepPeriod = :keepPeriod");
				paramMap.put("keepPeriod", Integer.valueOf(params.get("keepPeriod")));
			}
			if(Strings.isNotBlank(params.get("flowState"))) {//按处理状态查询
				buffer.append(" and summary.state = :flowState");
				paramMap.put("flowState", Integer.valueOf(params.get("flowState")));
			}
			if(Strings.isNotBlank(params.get("createTime"))) {//按创建时间查询(待发列表)
				String time = params.get("createTime").toString();
				String[] times = time.split("#");
				if(times.length>0 && Strings.isNotBlank(times[0])) {
					buffer.append(" and summary.createTime >= :createTimeS");
					paramMap.put("createTimeS", Datetimes.getTodayFirstTime(times[0]));
				}
				if(times.length>1 && Strings.isNotBlank(times[1])) {
					buffer.append(" and summary.createTime <= :createTimeE");
					paramMap.put("createTimeE", Datetimes.getTodayLastTime(times[1]));
				}
			}
			if(Strings.isNotBlank(params.get("startTime"))&&!",".equals(params.get("startTime"))) {//按发起时间查询
				String time = params.get("startTime").toString();
				//BUG_普通_V5_G6 V7.1_一星卡_四川省生态环境厅_收文登记簿统计出数据后导出后显示的是所有的，而不是按照统计的时间导出的。_20190919092662
				String[] times;
				boolean status = time.contains("#");
				if (status == false) {
					String to_time = params.get("to_startTime").toString();
					String toge_time = time + "#" + to_time;
					times = toge_time.split("#");
				}else {
					times = time.split("#");
				}
                if (times.length > 0 && Strings.isNotBlank(times[0]) && (!",".equals(time))) {
					buffer.append(" and summary.createTime >= :startTimeS");
					paramMap.put("startTimeS", Datetimes.getTodayFirstTime(times[0]));
				}
				if(times.length>1 && Strings.isNotBlank(times[1])) {
					buffer.append(" and summary.createTime <= :startTimeE");
					paramMap.put("startTimeE", Datetimes.getTodayLastTime(times[1]));
				}
			}
			if(Strings.isNotBlank(params.get("summaryDeadLine"))) {//按处理时限查询
				String time = params.get("summaryDeadLine").toString();
				String[] times = time.split("#");
                if (times.length > 0 && Strings.isNotBlank(times[0]) && (!",".equals(time))) {
					buffer.append(" and summary.deadlineDatetime >= :startTimeS");
					paramMap.put("startTimeS", Datetimes.getTodayFirstTime(times[0]));
				}
				if(times.length>1 && Strings.isNotBlank(times[1])) {
					buffer.append(" and summary.deadlineDatetime <= :startTimeE");
					paramMap.put("startTimeE", Datetimes.getTodayLastTime(times[1]));
				}
			}
			if(Strings.isNotBlank(params.get("packTime"))) {//按分送日期查询
				String time = params.get("packTime").toString();
				String[] times = time.split("#");
				if(times.length>0 && Strings.isNotBlank(times[0])) {
					buffer.append(" and summary.packTime >= :packTimeS");
					paramMap.put("packTimeS", Datetimes.getTodayFirstTime(times[0]));
				}
				if(times.length>1 && Strings.isNotBlank(times[1])) {
					buffer.append(" and summary.packTime <= :packTimeE");
					paramMap.put("packTimeE", Datetimes.getTodayLastTime(times[1]));
				}
			}
			if(Strings.isNotBlank(params.get("signingDate"))&&!",".equals(params.get("signingDate"))) {//按签发日期查询
				String time = params.get("signingDate").toString();
				String[] times = time.split("#");
				//客开 项目名称： 作者：fzc 修改日期：2018-4-3 [修改功能：]start
                if (times.length > 0 && Strings.isNotBlank(times[0]) && (!",".equals(time))) {
                    //客开 项目名称： 作者：fzc 修改日期：2018-4-3 [修改功能：]end
					buffer.append(" and summary.signingDate >= :signingDateS");
					paramMap.put("signingDateS", Datetimes.getTodayFirstTime(times[0]));
				}
				if(times.length>1 && Strings.isNotBlank(times[1])) {
					buffer.append(" and summary.signingDate <= :signingDateE");
					paramMap.put("signingDateE", Datetimes.getTodayLastTime(times[1]));
				}
			}
			if(Strings.isNotBlank(params.get("recieveDate"))) {//按签收日期查询
				String time = params.get("recieveDate").toString();
				String[] times = time.split("#");
				//客开 项目名称： 作者：fzc 修改日期：2018-4-3 [修改功能：]start
                if (times.length > 0 && Strings.isNotBlank(times[0]) && (!",".equals(time))) {
                    //客开 项目名称： 作者：fzc 修改日期：2018-4-3 [修改功能：]end
					buffer.append(" and summary.receiptDate >= :receiptDateS");
					paramMap.put("receiptDateS", Datetimes.getTodayFirstTime(times[0]));
				}
				if(times.length>1 && Strings.isNotBlank(times[1])) {
					buffer.append(" and summary.receiptDate <= :receiptDateE");
					paramMap.put("receiptDateE", Datetimes.getTodayLastTime(times[1]));
				}
			}
			if(Strings.isNotBlank(params.get("registerDate"))) {//按登记日期查询
				String time = params.get("registerDate").toString();
				String[] times = time.split("#");
				//客开 项目名称： 作者：fzc 修改日期：2018-4-3 [修改功能：]start
                if (times.length > 0 && Strings.isNotBlank(times[0]) && (!",".equals(time))) {
                    //客开 项目名称： 作者：fzc 修改日期：2018-4-3 [修改功能：]end
					buffer.append(" and summary.registrationDate >= :registrationDateS");
					paramMap.put("registrationDateS", Datetimes.getTodayFirstTime(times[0]));
				}
				if(times.length>1 && Strings.isNotBlank(times[1])) {
					buffer.append(" and summary.registrationDate <= :registrationDateE");
					paramMap.put("registrationDateE", Datetimes.getTodayLastTime(times[1]));
				}
			}
			if(Strings.isNotBlank(params.get("completeTime"))) {//按处理时间查询(交换列表)
				String time = params.get("completeTime").toString();
				String[] times = time.split("#");
				if(times.length>0 && Strings.isNotBlank(times[0])) {
					buffer.append(" and affair.completeTime >= :completeTimeS");
					paramMap.put("completeTimeS", Datetimes.getTodayFirstTime(times[0]));
				}
				if(times.length>1 && Strings.isNotBlank(times[1])) {
					buffer.append(" and affair.completeTime <= :completeTimeE");
					paramMap.put("completeTimeE", Datetimes.getTodayLastTime(times[1]));
				}
			}
			if(Strings.isNotBlank(params.get("edocTypeSearch"))) {//按公文类型查询
				buffer.append(" and summary.edocType = :edocTypeSearch");
				paramMap.put("edocTypeSearch", Integer.parseInt(params.get("edocTypeSearch")));
			}
			if(Strings.isNotBlank(params.get("summaryState"))) {//按流程状态查询
				buffer.append(" and summary.state = :summaryStateSearch");
				paramMap.put("summaryStateSearch", Integer.parseInt(params.get("summaryState")));
			}
			if(Strings.isNotBlank(params.get("exchangeSendAffairId"))) {//是否分送查询
				if("1".equals(params.get("exchangeSendAffairId"))){
					buffer.append(" and summary.exchangeSendAffairId IS NOT NULL ");
				}else{
					buffer.append(" and summary.exchangeSendAffairId IS NULL ");
				}
			}
			if(Strings.isNotBlank(params.get("signPerson"))) {//按标题查询
				buffer.append(" and summary.signPerson like :signPerson");
				paramMap.put("signPerson", "%" + GovdocUtil.convertSpecialChat(params.get("signPerson")) + "%");
			}
			if(Strings.isNotBlank(params.get("competeTransactor"))) {//已办所属人
				List<Long> competitiveMemberId = new ArrayList<Long>();
				List<V3xOrgMember> memberList = null;
				try {
					memberList = orgManager.getMemberByIndistinctName(params.get("competeTransactor"));
				} catch (BusinessException e) {
					e.printStackTrace();
				}
				int i=0;
				for (V3xOrgMember member : memberList) {
					if (i < 999) {
						competitiveMemberId.add(member.getId());
					}
					i++;
				}
				if (competitiveMemberId.isEmpty() ) {
					buffer.append(" and  extend.competitiveMemberId = -1 ");//not exit record;
				} else if( i > 1) {
					buffer.append(" and  extend.competitiveMemberId in (:competitiveMemberId)");
					paramMap.put("competitiveMemberId", competitiveMemberId);
				} else {
					buffer.append(" and  extend.competitiveMemberId = :competitiveMemberId ");
					paramMap.put("competitiveMemberId", competitiveMemberId);
				}
			}
			//公文统计 按印发份数查询
			if(Strings.isNotBlank(params.get("copies"))) {//印发份数
				buffer.append(" and summary.copies = :copies");
				paramMap.put("copies", Integer.parseInt(params.get("copies")));
			}
			
			if(Strings.isNotBlank(params.get("currentNodesInfo"))) {//当前待办人
				buffer.append(" and (summary.currentNodesInfo like :currentNodesInfo and summary.completeTime is null) ");
				paramMap.put("currentNodesInfo", "%" + GovdocUtil.convertSpecialChat(params.get("currentNodesInfo")) + "%");
			}
			if(Strings.isNotBlank(params.get("transferStatus"))) {//流转状态

				String transferStatus = params.get("transferStatus");
				if(transferStatus.indexOf(",") != -1)
				{
					buffer.append(" and summary.transferStatus in(:transferStatus)");
					String[] transferStatusArray = transferStatus.split(",");
					List<Integer>  transferStatusList = new ArrayList<Integer>();
					for(String tempTransferStatus: transferStatusArray)
					{
						transferStatusList.add(Integer.parseInt(tempTransferStatus));
					}
					paramMap.put("transferStatus", transferStatusList);
				}
				else
				{
					buffer.append(" and summary.transferStatus =:transferStatus");
					paramMap.put("transferStatus", Integer.parseInt(transferStatus));
				}

			}
		}
		return buffer.toString();
	}
	
	/**
	 * 按summaryExtend扩展表字段查询
	 * @param paramMap  hql的参数
	 * @param params  传递进来的条件
	 * @return
	 */
	protected String getEdocSummaryExtendHqlByCondition(Map<String, Object> paramMap, Map<String, String> params) {
		StringBuilder buffer = new StringBuilder();
		if(params.containsKey("condition")) {
			if(Strings.isNotBlank(params.get("string1"))) {//string
				buffer.append(" and summaryExt.varchar1 like :string1");
				paramMap.put("string1", "%" + GovdocUtil.convertSpecialChat(params.get("string1")) + "%");
			}
			if(Strings.isNotBlank(params.get("string2"))) {
				buffer.append(" and summaryExt.varchar2 like :string2");
				paramMap.put("string2", "%" + GovdocUtil.convertSpecialChat(params.get("string2")) + "%");
			}
			if(Strings.isNotBlank(params.get("string3"))) {
				buffer.append(" and summaryExt.varchar3 like :string3");
				paramMap.put("string3", "%" + GovdocUtil.convertSpecialChat(params.get("string3")) + "%");
			}
			if(Strings.isNotBlank(params.get("string4"))) {
				buffer.append(" and summaryExt.varchar4 like :string4");
				paramMap.put("string4", "%" + GovdocUtil.convertSpecialChat(params.get("string4")) + "%");
			}
			if(Strings.isNotBlank(params.get("string5"))) {
				buffer.append(" and summaryExt.varchar5 like :string5");
				paramMap.put("string5", "%" + GovdocUtil.convertSpecialChat(params.get("string5")) + "%");
			}
			if(Strings.isNotBlank(params.get("string6"))) {
				buffer.append(" and summaryExt.varchar6 like :string6");
				paramMap.put("string6", "%" + GovdocUtil.convertSpecialChat(params.get("string6")) + "%");
			}
			if(Strings.isNotBlank(params.get("string7"))) {
				buffer.append(" and summaryExt.varchar7 like :string7");
				paramMap.put("string7", "%" + GovdocUtil.convertSpecialChat(params.get("string7")) + "%");
			}
			if(Strings.isNotBlank(params.get("string8"))) {
				buffer.append(" and summaryExt.varchar8 like :string8");
				paramMap.put("string8", "%" + GovdocUtil.convertSpecialChat(params.get("string8")) + "%");
			}
			if(Strings.isNotBlank(params.get("string9"))) {
				buffer.append(" and summaryExt.varchar9 like :string9");
				paramMap.put("string9", "%" + GovdocUtil.convertSpecialChat(params.get("string9")) + "%");
			}
			if(Strings.isNotBlank(params.get("string10"))) {
				buffer.append(" and summaryExt.varchar10 like :string10");
				paramMap.put("string10", "%" + GovdocUtil.convertSpecialChat(params.get("string10")) + "%");
			}
			if(Strings.isNotBlank(params.get("string11"))) {
				buffer.append(" and summaryExt.varchar11 like :string11");
				paramMap.put("string11", "%" + GovdocUtil.convertSpecialChat(params.get("string11")) + "%");
			}
			if(Strings.isNotBlank(params.get("string12"))) {
				buffer.append(" and summaryExt.varchar12 like :string12");
				paramMap.put("string12", "%" + GovdocUtil.convertSpecialChat(params.get("string12")) + "%");
			}
			if(Strings.isNotBlank(params.get("string13"))) {
				buffer.append(" and summaryExt.varchar13 like :string13");
				paramMap.put("string13", "%" + GovdocUtil.convertSpecialChat(params.get("string13")) + "%");
			}
			if(Strings.isNotBlank(params.get("string14"))) {
				buffer.append(" and summaryExt.varchar14 like :string14");
				paramMap.put("string14", "%" + GovdocUtil.convertSpecialChat(params.get("string14")) + "%");
			}
			if(Strings.isNotBlank(params.get("string15"))) {
				buffer.append(" and summaryExt.varchar15 like :string15");
				paramMap.put("string15", "%" + GovdocUtil.convertSpecialChat(params.get("string15")) + "%");
			}
			if(Strings.isNotBlank(params.get("string16"))) {
				buffer.append(" and summaryExt.varchar16 like :string16");
				paramMap.put("string16", "%" + GovdocUtil.convertSpecialChat(params.get("string16")) + "%");
			}
			if(Strings.isNotBlank(params.get("string17"))) {
				buffer.append(" and summaryExt.varchar1 like :string17");
				paramMap.put("string17", "%" + GovdocUtil.convertSpecialChat(params.get("string17")) + "%");
			}
			if(Strings.isNotBlank(params.get("string18"))) {
				buffer.append(" and summaryExt.varchar18 like :string18");
				paramMap.put("string18", "%" + GovdocUtil.convertSpecialChat(params.get("string18")) + "%");
			}
			if(Strings.isNotBlank(params.get("string19"))) {
				buffer.append(" and summaryExt.varchar19 like :string19");
				paramMap.put("string19", "%" + GovdocUtil.convertSpecialChat(params.get("string19")) + "%");
			}
			if(Strings.isNotBlank(params.get("string20"))) {
				buffer.append(" and summaryExt.varchar20 like :string20");
				paramMap.put("string20", "%" + GovdocUtil.convertSpecialChat(params.get("string20")) + "%");
			}
			if(Strings.isNotBlank(params.get("string21"))) {
				buffer.append(" and summaryExt.varchar21 like :string21");
				paramMap.put("string21", "%" + GovdocUtil.convertSpecialChat(params.get("string21")) + "%");
			}
			if(Strings.isNotBlank(params.get("string22"))) {
				buffer.append(" and summaryExt.varchar22 like :string22");
				paramMap.put("string22", "%" + GovdocUtil.convertSpecialChat(params.get("string22")) + "%");
			}
			if(Strings.isNotBlank(params.get("string23"))) {
				buffer.append(" and summaryExt.varchar23 like :string23");
				paramMap.put("string23", "%" + GovdocUtil.convertSpecialChat(params.get("string23")) + "%");
			}
			if(Strings.isNotBlank(params.get("string24"))) {
				buffer.append(" and summaryExt.varchar24 like :string24");
				paramMap.put("string24", "%" + GovdocUtil.convertSpecialChat(params.get("string24")) + "%");
			}
			if(Strings.isNotBlank(params.get("string25"))) {
				buffer.append(" and summaryExt.varchar25 like :string25");
				paramMap.put("string25", "%" + GovdocUtil.convertSpecialChat(params.get("string25")) + "%");
			}
			if(Strings.isNotBlank(params.get("string26"))) {
				buffer.append(" and summaryExt.varchar26 like :string26");
				paramMap.put("string26", "%" + GovdocUtil.convertSpecialChat(params.get("string26")) + "%");
			}
			if(Strings.isNotBlank(params.get("string27"))) {
				buffer.append(" and summaryExt.varchar27 like :string27");
				paramMap.put("string27", "%" + GovdocUtil.convertSpecialChat(params.get("string27")) + "%");
			}
			if(Strings.isNotBlank(params.get("string28"))) {
				buffer.append(" and summaryExt.varchar28 like :string28");
				paramMap.put("string28", "%" + GovdocUtil.convertSpecialChat(params.get("string28")) + "%");
			}
			if(Strings.isNotBlank(params.get("string29"))) {
				buffer.append(" and summaryExt.varchar29 like :string29");
				paramMap.put("string29", "%" + GovdocUtil.convertSpecialChat(params.get("string29")) + "%");
			}
			if(Strings.isNotBlank(params.get("string30"))) {
				buffer.append(" and summaryExt.varchar30 like :string30");
				paramMap.put("string30", "%" + GovdocUtil.convertSpecialChat(params.get("string30")) + "%");
			}
			if(Strings.isNotBlank(params.get("integer1"))) {
				buffer.append(" and summaryExt.integer1 = :integer1"); //integer
				paramMap.put("integer1",  GovdocUtil.convertSpecialChat(params.get("integer1")));
			}
			if(Strings.isNotBlank(params.get("integer2"))) {
				buffer.append(" and summaryExt.integer2 = :integer2");
				paramMap.put("integer2",  GovdocUtil.convertSpecialChat(params.get("integer2")));
			}
			if(Strings.isNotBlank(params.get("integer3"))) {
				buffer.append(" and summaryExt.integer3 = :integer3");
				paramMap.put("integer3",  GovdocUtil.convertSpecialChat(params.get("integer3")));
			}
			if(Strings.isNotBlank(params.get("integer4"))) {
				buffer.append(" and summaryExt.integer4 = :integer4");
				paramMap.put("integer4",  GovdocUtil.convertSpecialChat(params.get("integer4")));
			}
			if(Strings.isNotBlank(params.get("integer5"))) {
				buffer.append(" and summaryExt.integer5 = :integer5");
				paramMap.put("integer5",  GovdocUtil.convertSpecialChat(params.get("integer5")));
			}
			if(Strings.isNotBlank(params.get("integer6"))) {
				buffer.append(" and summaryExt.integer6 = :integer6");
				paramMap.put("integer6",  GovdocUtil.convertSpecialChat(params.get("integer6")));
			}
			if(Strings.isNotBlank(params.get("integer7"))) {
				buffer.append(" and summaryExt.integer7 = :integer7");
				paramMap.put("integer7",  GovdocUtil.convertSpecialChat(params.get("integer7")));
			}
			if(Strings.isNotBlank(params.get("integer8"))) {
				buffer.append(" and summaryExt.integer8 = :integer8");
				paramMap.put("integer8",  GovdocUtil.convertSpecialChat(params.get("integer8")));
			}
			if(Strings.isNotBlank(params.get("integer9"))) {
				buffer.append(" and summaryExt.integer9 = :integer9");
				paramMap.put("integer9",  GovdocUtil.convertSpecialChat(params.get("integer9")));
			}
			if(Strings.isNotBlank(params.get("integer10"))) {
				buffer.append(" and summaryExt.integer10 = :integer10");
				paramMap.put("integer10",  GovdocUtil.convertSpecialChat(params.get("integer10")));
			}
			if(Strings.isNotBlank(params.get("integer11"))) {
				buffer.append(" and summaryExt.integer11 = :integer11");
				paramMap.put("integer11",  GovdocUtil.convertSpecialChat(params.get("integer11")));
			}
			if(Strings.isNotBlank(params.get("integer12"))) {
				buffer.append(" and summaryExt.integer12 = :integer12");
				paramMap.put("integer12",  GovdocUtil.convertSpecialChat(params.get("integer12")));
			}
			if(Strings.isNotBlank(params.get("integer13"))) {
				buffer.append(" and summaryExt.integer13 = :integer13");
				paramMap.put("integer13",  GovdocUtil.convertSpecialChat(params.get("integer13")));
			}
			if(Strings.isNotBlank(params.get("integer14"))) {
				buffer.append(" and summaryExt.integer14 = :integer14");
				paramMap.put("integer14",  GovdocUtil.convertSpecialChat(params.get("integer14")));
			}
			if(Strings.isNotBlank(params.get("integer15"))) {
				buffer.append(" and summaryExt.integer15 = :integer15");
				paramMap.put("integer15",  GovdocUtil.convertSpecialChat(params.get("integer15")));
			}
			if(Strings.isNotBlank(params.get("integer16"))) {
				buffer.append(" and summaryExt.integer16 = :integer16");
				paramMap.put("integer16",  GovdocUtil.convertSpecialChat(params.get("integer16")));
			}
			if(Strings.isNotBlank(params.get("integer17"))) {
				buffer.append(" and summaryExt.integer17 = :integer17");
				paramMap.put("integer17",  GovdocUtil.convertSpecialChat(params.get("integer17")));
			}
			if(Strings.isNotBlank(params.get("integer18"))) {
				buffer.append(" and summaryExt.integer18 = :integer18");
				paramMap.put("integer18",  GovdocUtil.convertSpecialChat(params.get("integer18")));
			}
			if(Strings.isNotBlank(params.get("integer19"))) {
				buffer.append(" and summaryExt.integer19 = :integer19");
				paramMap.put("integer19",  GovdocUtil.convertSpecialChat(params.get("integer19")));
			}
			if(Strings.isNotBlank(params.get("integer20"))) {
				buffer.append(" and summaryExt.integer20 = :integer20");
				paramMap.put("integer20",  GovdocUtil.convertSpecialChat(params.get("integer20")));
			}
			
			if(Strings.isNotBlank(params.get("decimal1"))) {
				buffer.append(" and summaryExt.decimal1 = :decimal1");
				paramMap.put("decimal1",  GovdocUtil.convertSpecialChat(params.get("decimal1")));
			}
			if(Strings.isNotBlank(params.get("decimal2"))) {
				buffer.append(" and summaryExt.decimal2 = :decimal2");
				paramMap.put("decimal2",  GovdocUtil.convertSpecialChat(params.get("decimal2")));
			}
			if(Strings.isNotBlank(params.get("decimal3"))) {
				buffer.append(" and summaryExt.decimal3 = :decimal3");
				paramMap.put("decimal3",  GovdocUtil.convertSpecialChat(params.get("decimal3")));
			}
			if(Strings.isNotBlank(params.get("decimal4"))) {
				buffer.append(" and summaryExt.decimal4 = :decimal4");
				paramMap.put("decimal4",  GovdocUtil.convertSpecialChat(params.get("decimal4")));
			}
			if(Strings.isNotBlank(params.get("decimal5"))) {
				buffer.append(" and summaryExt.decimal5 = :decimal5");
				paramMap.put("decimal5",  GovdocUtil.convertSpecialChat(params.get("decimal5")));
			}
			if(Strings.isNotBlank(params.get("decimal6"))) {
				buffer.append(" and summaryExt.decimal6 = :decimal6");
				paramMap.put("decimal6",  GovdocUtil.convertSpecialChat(params.get("decimal6")));
			}
			if(Strings.isNotBlank(params.get("decimal7"))) {
				buffer.append(" and summaryExt.decimal7 = :decimal7");
				paramMap.put("decimal7",  GovdocUtil.convertSpecialChat(params.get("decimal7")));
			}
			if(Strings.isNotBlank(params.get("decimal8"))) {
				buffer.append(" and summaryExt.decimal8 = :decimal8");
				paramMap.put("decimal8",  GovdocUtil.convertSpecialChat(params.get("decimal8")));
			}
			if(Strings.isNotBlank(params.get("decimal9"))) {
				buffer.append(" and summaryExt.decimal9 = :decimal9");
				paramMap.put("decimal9",  GovdocUtil.convertSpecialChat(params.get("decimal9")));
			}
			if(Strings.isNotBlank(params.get("decimal10"))) {
				buffer.append(" and summaryExt.decimal10 = :decimal10");
				paramMap.put("decimal10",  GovdocUtil.convertSpecialChat(params.get("decimal10")));
			}
			if(Strings.isNotBlank(params.get("decimal11"))) {
				buffer.append(" and summaryExt.decimal11 = :decimal11");
				paramMap.put("decimal11",  GovdocUtil.convertSpecialChat(params.get("decimal11")));
			}
			if(Strings.isNotBlank(params.get("decimal12"))) {
				buffer.append(" and summaryExt.decimal12 = :decimal12");
				paramMap.put("decimal12",  GovdocUtil.convertSpecialChat(params.get("decimal12")));
			}
			if(Strings.isNotBlank(params.get("decimal13"))) {
				buffer.append(" and summaryExt.decimal13 = :decimal13");
				paramMap.put("decimal13",  GovdocUtil.convertSpecialChat(params.get("decimal13")));
			}
			if(Strings.isNotBlank(params.get("decimal14"))) {
				buffer.append(" and summaryExt.decimal14 = :decimal14");
				paramMap.put("decimal14",  GovdocUtil.convertSpecialChat(params.get("decimal14")));
			}
			if(Strings.isNotBlank(params.get("decimal15"))) {
				buffer.append(" and summaryExt.decimal15 = :decimal15");
				paramMap.put("decimal15",  GovdocUtil.convertSpecialChat(params.get("decimal15")));
			}
			if(Strings.isNotBlank(params.get("decimal16"))) {
				buffer.append(" and summaryExt.decimal16 = :decimal16");
				paramMap.put("decimal16",  GovdocUtil.convertSpecialChat(params.get("decimal16")));
			}
			if(Strings.isNotBlank(params.get("decimal17"))) {
				buffer.append(" and summaryExt.decimal17 = :decimal17");
				paramMap.put("decimal17",  GovdocUtil.convertSpecialChat(params.get("decimal17")));
			}
			if(Strings.isNotBlank(params.get("decimal18"))) {
				buffer.append(" and summaryExt.decimal18 = :decimal18");
				paramMap.put("decimal18",  GovdocUtil.convertSpecialChat(params.get("decimal18")));
			}
			if(Strings.isNotBlank(params.get("decimal19"))) {
				buffer.append(" and summaryExt.decimal19 = :decimal19");
				paramMap.put("decimal19",  GovdocUtil.convertSpecialChat(params.get("decimal19")));
			}
			if(Strings.isNotBlank(params.get("decimal20"))) {
				buffer.append(" and summaryExt.decimal20 = :decimal20");
				paramMap.put("decimal20",  GovdocUtil.convertSpecialChat(params.get("decimal20")));
			}
			if(Strings.isNotBlank(params.get("list1"))) {
			    buffer.append(" and summaryExt.list1 like :list1");
				paramMap.put("list1", "%" + GovdocUtil.convertSpecialChat(params.get("list1")) + "%");
			}
			if(Strings.isNotBlank(params.get("list2"))) {
				buffer.append(" and summaryExt.list2 like :list2");
				paramMap.put("list2", "%" + GovdocUtil.convertSpecialChat(params.get("list2")) + "%");
			}
			if(Strings.isNotBlank(params.get("list3"))) {
				buffer.append(" and summaryExt.list3 like :list3");
				paramMap.put("list3", "%" + GovdocUtil.convertSpecialChat(params.get("list3")) + "%");
			}
			if(Strings.isNotBlank(params.get("list4"))) {
				buffer.append(" and summaryExt.list4 like :list4");
				paramMap.put("list4", "%" + GovdocUtil.convertSpecialChat(params.get("list4")) + "%");
			}
			if(Strings.isNotBlank(params.get("list5"))) {
				buffer.append(" and summaryExt.list5 like :list5");
				paramMap.put("list5", "%" + GovdocUtil.convertSpecialChat(params.get("list5")) + "%");
			}
			if(Strings.isNotBlank(params.get("list6"))) {
				buffer.append(" and summaryExt.list6 like :list6");
				paramMap.put("list6", "%" + GovdocUtil.convertSpecialChat(params.get("list6")) + "%");
			}
			if(Strings.isNotBlank(params.get("list7"))) {
				buffer.append(" and summaryExt.list7 like :list7");
				paramMap.put("list7", "%" + GovdocUtil.convertSpecialChat(params.get("list7")) + "%");
			}
			if(Strings.isNotBlank(params.get("list8"))) {
				buffer.append(" and summaryExt.list8 like :list8");
				paramMap.put("list8", "%" + GovdocUtil.convertSpecialChat(params.get("list8")) + "%");
			}
			if(Strings.isNotBlank(params.get("list9"))) {
				buffer.append(" and summaryExt.list9 like :list9");
				paramMap.put("list9", "%" + GovdocUtil.convertSpecialChat(params.get("list9")) + "%");
			}
			if(Strings.isNotBlank(params.get("list10"))) {
				buffer.append(" and summaryExt.list10 like :list10");
				paramMap.put("list10", "%" + GovdocUtil.convertSpecialChat(params.get("list10")) + "%");
			}
			if(Strings.isNotBlank(params.get("list11"))) {
				buffer.append(" and summaryExt.list11 like :list11");
				paramMap.put("list11", "%" + GovdocUtil.convertSpecialChat(params.get("list11")) + "%");
			}
			if(Strings.isNotBlank(params.get("list12"))) {
				buffer.append(" and summaryExt.list12 like :list12");
				paramMap.put("list12", "%" + GovdocUtil.convertSpecialChat(params.get("list12")) + "%");
			}
			if(Strings.isNotBlank(params.get("list13"))) {
				buffer.append(" and summaryExt.list13 like :list13");
				paramMap.put("list13", "%" + GovdocUtil.convertSpecialChat(params.get("list13")) + "%");
			}
			if(Strings.isNotBlank(params.get("list14"))) {
				buffer.append(" and summaryExt.list14 like :list14");
				paramMap.put("list14", "%" + GovdocUtil.convertSpecialChat(params.get("list14")) + "%");
			}
			if(Strings.isNotBlank(params.get("list15"))) {
				buffer.append(" and summaryExt.list15 like :list15");
				paramMap.put("list15", "%" + GovdocUtil.convertSpecialChat(params.get("list15")) + "%");
			}
			if(Strings.isNotBlank(params.get("list16"))) {
				buffer.append(" and summaryExt.list16 like :list16");
				paramMap.put("list16", "%" + GovdocUtil.convertSpecialChat(params.get("list16")) + "%");
			}
			if(Strings.isNotBlank(params.get("list17"))) {
				buffer.append(" and summaryExt.list17 like :list17");
				paramMap.put("list17", "%" + GovdocUtil.convertSpecialChat(params.get("list17")) + "%");
			}
			if(Strings.isNotBlank(params.get("list18"))) {
				buffer.append(" and summaryExt.list18 like :list18");
				paramMap.put("list18", "%" + GovdocUtil.convertSpecialChat(params.get("list18")) + "%");
			}
			if(Strings.isNotBlank(params.get("list19"))) {
				buffer.append(" and summaryExt.list19 like :list19");
				paramMap.put("list19", "%" + GovdocUtil.convertSpecialChat(params.get("list19")) + "%");
			}
			if(Strings.isNotBlank(params.get("list20"))) {
				buffer.append(" and summaryExt.list20 like :list20");
				paramMap.put("list20", "%" + GovdocUtil.convertSpecialChat(params.get("list20")) + "%");
			}
			if(Strings.isNotBlank(params.get("text1"))) {
				buffer.append(" and summaryExt.text1 like :text1");
				paramMap.put("text1", "%" + GovdocUtil.convertSpecialChat(params.get("text1")) + "%");
			}
			if(Strings.isNotBlank(params.get("text2"))) {
				buffer.append(" and summaryExt.text2 like :text2");
				paramMap.put("text2", "%" + GovdocUtil.convertSpecialChat(params.get("text2")) + "%");
			}
			if(Strings.isNotBlank(params.get("text3"))) {
				buffer.append(" and summaryExt.text3 like :text3");
				paramMap.put("text3", "%" + GovdocUtil.convertSpecialChat(params.get("text3")) + "%");
			}
			if(Strings.isNotBlank(params.get("text4"))) {
				buffer.append(" and summaryExt.text4 like :text4");
				paramMap.put("text4", "%" + GovdocUtil.convertSpecialChat(params.get("text4")) + "%");
			}
			if(Strings.isNotBlank(params.get("text5"))) {
				buffer.append(" and summaryExt.text5 like :text5");
				paramMap.put("text5", "%" + GovdocUtil.convertSpecialChat(params.get("text5")) + "%");
			}
			if(Strings.isNotBlank(params.get("text6"))) {
				buffer.append(" and summaryExt.text6 like :text6");
				paramMap.put("text6", "%" + GovdocUtil.convertSpecialChat(params.get("text6")) + "%");
			}
			if(Strings.isNotBlank(params.get("text7"))) {
				buffer.append(" and summaryExt.text7 like :text7");
				paramMap.put("text7", "%" + GovdocUtil.convertSpecialChat(params.get("text7")) + "%");
			}
			if(Strings.isNotBlank(params.get("text8"))) {
				buffer.append(" and summaryExt.text8 like :text8");
				paramMap.put("text8", "%" + GovdocUtil.convertSpecialChat(params.get("text8")) + "%");
			}
			if(Strings.isNotBlank(params.get("text9"))) {
				buffer.append(" and summaryExt.text9 like :text9");
				paramMap.put("text9", "%" + GovdocUtil.convertSpecialChat(params.get("text9")) + "%");
			}
			if(Strings.isNotBlank(params.get("text10"))) {
				buffer.append(" and summaryExt.text10 like :text10");
				paramMap.put("text10", "%" + GovdocUtil.convertSpecialChat(params.get("text10")) + "%");
			}
			if(Strings.isNotBlank(params.get("text11"))) {
				buffer.append(" and summaryExt.text11 like :text11");
				paramMap.put("text11", "%" + GovdocUtil.convertSpecialChat(params.get("text11")) + "%");
			}
			if(Strings.isNotBlank(params.get("text12"))) {
				buffer.append(" and summaryExt.text12 like :text12");
				paramMap.put("text12", "%" + GovdocUtil.convertSpecialChat(params.get("text12")) + "%");
			}
			if(Strings.isNotBlank(params.get("text13"))) {
				buffer.append(" and summaryExt.text13 like :text13");
				paramMap.put("text13", "%" + GovdocUtil.convertSpecialChat(params.get("text13")) + "%");
			}
			if(Strings.isNotBlank(params.get("text14"))) {
				buffer.append(" and summaryExt.text14 like :text14");
				paramMap.put("text14", "%" + GovdocUtil.convertSpecialChat(params.get("text14")) + "%");
			}
			if(Strings.isNotBlank(params.get("text15"))) {
				buffer.append(" and summaryExt.text15 like :text15");
				paramMap.put("text15", "%" + GovdocUtil.convertSpecialChat(params.get("text15")) + "%");
			}
		}
		return buffer.toString();
	}
	
	private String getHqlByStartMemberId(Map<String, Object> paramMap, Map<String, String> params) {
		StringBuilder buffer = new StringBuilder();
		List<Long> userIdList = GovdocUtil.getIdList(params.get("startUserId"));
		if (userIdList.size() == 1) {
			buffer.append(" and summary.startUserId = :startUserId");
			paramMap.put("startUserId", userIdList);
		} else if (userIdList.size() <= 300) {
			buffer.append(" and summary.startUserId in (:startUserId)");
			paramMap.put("startUserId", userIdList);
		} else {// 300个一组用or连接
			Map<String, List<Long>> senderIdMap = new HashMap<String, List<Long>>();
			List<Long> startUserIdList = new ArrayList<Long>();
			for (int i = 0; i < userIdList.size(); i++) {
				startUserIdList.add(userIdList.get(i));
				if (i != 0 && i % 300 == 0) {
					senderIdMap.put("startUserId" + i, startUserIdList);
					startUserIdList = new ArrayList<Long>();
				}
				if (i == userIdList.size() - 1) {
					senderIdMap.put("startUserId" + i, startUserIdList);
				}
			}
			buffer.append(" and (");
			int i = 0;
			for (String key : senderIdMap.keySet()) {
				if (i != 0) {
					buffer.append(" or ");
				}
				i++;
				List<Long> ids = senderIdMap.get(key);
				buffer.append(" summary.startUserId in (:" + key + ")");
				paramMap.put(key, ids);
			}
			buffer.append(" )");
		}
		return buffer.toString();
	}
	
	private String getExchangeHqlBySendUnitId(Map<String, Object> paramMap, Map<String, String> params) {
		StringBuilder buffer = new StringBuilder();
//		List<Long> sendUnitIdList = GovdocUtil.getIdList(params.get("exchangeSendUnitId"));
//		if (sendUnitIdList.size() == 1) {
//			buffer.append(" and detail.sendAccountId = :exchangeSendUnitId");
//			paramMap.put("exchangeSendUnitId", sendUnitIdList.get(0));
//		} else if (sendUnitIdList.size() <= 300) {
//			buffer.append(" and detail.sendAccountId in (:exchangeSendUnitId)");
//			paramMap.put("exchangeSendUnitId", sendUnitIdList);
//		} else {
//			// 300个一组用or连接
//			Map<String, List<Long>> sendAccountIdMap = new HashMap<String, List<Long>>();
//			List<Long> unitIdList = new ArrayList<Long>();
//			for (int i = 0; i < sendUnitIdList.size(); i++) {
//				unitIdList.add(sendUnitIdList.get(i));
//				if (i != 0 && i % 300 == 0) {
//					sendAccountIdMap.put("exchangeSendUnitId" + i, unitIdList);
//					unitIdList = new ArrayList<Long>();
//				}
//				if (i == unitIdList.size() - 1) {
//					sendAccountIdMap.put("exchangeSendUnitId" + i, unitIdList);
//				}
//			}
//			buffer.append(" and (");
//			int i = 0;
//			for (String key : sendAccountIdMap.keySet()) {
//				if (i != 0) {
//					buffer.append(" or ");
//				}
//				i++;
//				List<Long> ids = sendAccountIdMap.get(key);
//				buffer.append(" detail.sendAccountId in (:" + key + ")");
//				paramMap.put(key, ids);
//			}
//			buffer.append(" )");
//		}
		buffer.append(" and detail.sendUnit like :exchangeSendUnitName");
		paramMap.put("exchangeSendUnitName", "%" + params.get("exchangeSendUnitName") + "%");
		return buffer.toString();
	}
	
	/**
	 * 扩展字段的查询
	 * @param paramMap
	 * @param params
	 * @return
	 */
	protected String getExtendHqlByCondition(Map<String, Object> paramMap, Map<String, String> params) {
		StringBuilder buffer = new StringBuilder();
		if(params.containsKey("condition")) {
			for(int i=1;i<21;i++){
				String key = "list"+i;
				if(Strings.isNotBlank(params.get(key))) {//按重要程序查询
					buffer.append(" and extend.");
					buffer.append(key).append(" = :").append(key).append(" ");
					paramMap.put(key, params.get(key));
					break;
				}
			}
		}
		return buffer.toString();
	}
	
	/**
	 * 按affair表字段查询
	 * @param paramMap
	 * @param params
	 * @return
	 */
	protected String getAffairHqlByCondition(Map<String, Object> paramMap, Map<String, String> params) {
		return getAffairHqlByCondition(paramMap, params, "affair");
	}
	protected String getAffairHqlByCondition(Map<String, Object> paramMap, Map<String, String> params, String affairTable) {
		StringBuilder buffer = new StringBuilder();
		//客开 项目名称： [修改功能：] 作者：fzc 修改日期：2018-5-10 start
        List<Integer> subStateIds1 = new ArrayList<Integer>();
        subStateIds1.add(SubStateEnum.col_pending_specialBack.getKey());
        subStateIds1.add(SubStateEnum.col_pending_specialBackCenter.getKey());
        subStateIds1.add(SubStateEnum.col_pending_specialBackToSenderCancel.getKey());
        subStateIds1.add(SubStateEnum.col_pending_specialBackToSenderReGo.getKey());
        //客开 项目名称： [修改功能：] 作者：fzc 修改日期：2018-5-10 end
		if(params.containsKey("condition")) {
			if(Strings.isNotBlank(params.get("importantLevel"))) {//按重要程序查询
				buffer.append(" and affair.importantLevel like :importantLevel");
				paramMap.put("importantLevel", "%" + GovdocUtil.convertSpecialChat(params.get("importantLevel")) + "%");
			}
			if(Strings.isNotBlank(params.get("draftState"))) {//按状态查询
				String subState_str = params.get("draftState");
				if(subState_str.contains(",")){
					String[] subStates = subState_str.split(",");
					List<Integer> subStateList = new ArrayList<Integer>();
					for(String ss:subStates){
						subStateList.add(Integer.parseInt(ss));
					}
					buffer.append(" and affair.subState in (:draftState)");
					paramMap.put("draftState", subStateList);
				}else{
					int subState = Integer.valueOf(subState_str);
					if(subState == SubStateEnum.col_waitSend_stepBack.key()) {
						List<Integer> subStateList = new ArrayList<Integer>();
						subStateList.add(subState);
						subStateList.add(SubStateEnum.col_pending_specialBacked.key());
						subStateList.add(SubStateEnum.col_waitSend_sendBack.key());
						subStateList.add(SubStateEnum.col_pending_specialBackToSenderCancel.key());
						buffer.append(" and affair.subState in (:draftState)");
						paramMap.put("draftState", subStateList);
					} else {
						buffer.append(" and affair.subState = :draftState");
						paramMap.put("draftState", subState);	
					}
				}
			}
			if(Strings.isNotBlank(params.get("dealState"))) {//按处理状态查询
                //客开 项目名称： [修改功能：] 作者：fzc 修改日期：2018-5-10 start
                Integer subSate = Integer.valueOf(params.get("dealState"));
                if (String.valueOf(SubStateEnum.col_waitSend_stepBack.getKey()).equals(subSate+"")) {
                    //指定回退不允许暂存待办   普通回退经过暂存待办的  查询被回退的时候也应该过滤。
        	        subStateIds1.add(SubStateEnum.col_pending_ZCDB.getKey());
                    buffer.append(" and (affair.subState not in (:subStateIds) and affair.backFromId is not null )");
                    paramMap.put("subStateIds", subStateIds1);
                } else if (String.valueOf(SubStateEnum.col_pending_specialBack.getKey()).equals(subSate+"")) {
                    //指定回退
                    buffer.append(" and (affair.subState in (:subStateIds))");
                    paramMap.put("subStateIds", subStateIds1);
                } else if (String.valueOf(SubStateEnum.col_pending_read.getKey()).equals(subSate+"")) {
                    //已读
                    List<Integer> subStateIds2 = new ArrayList<Integer>();
                    subStateIds2.add(SubStateEnum.col_pending_read.getKey());//已读
                    subStateIds2.add(SubStateEnum.col_pending_ZCDB.getKey());//暂存待办
                    buffer.append(" and (affair.subState in (:subStateIds))");
                    paramMap.put("subStateIds", subStateIds2);
                } else {
                    buffer.append(" and affair.subState = :dealState");
                    paramMap.put("dealState", Integer.valueOf(params.get("dealState")));
                }
                //客开 项目名称： [修改功能：] 作者：fzc 修改日期：2018-5-10 end
            }
			if(Strings.isNotBlank(params.get("receiveTime"))) {//按接收时间查询
				String time = params.get("receiveTime").toString();
				String[] times = time.split("#");
				if(times.length>0 && Strings.isNotBlank(times[0])) {
					buffer.append(" and affair.receiveTime >= :receiveTimeS");
					paramMap.put("receiveTimeS", Datetimes.getTodayFirstTime(times[0]));
				}
				if(times.length>1 && Strings.isNotBlank(times[1])) {
					buffer.append(" and affair.receiveTime <= :receiveTimeE");
					paramMap.put("receiveTimeE", Datetimes.getTodayLastTime(times[1]));
				}
			}
			if(Strings.isNotBlank(params.get("completeTime"))) {//按处理时间查询
				String time = params.get("completeTime").toString();
				String[] times = time.split("#");
				if(times.length>0 && Strings.isNotBlank(times[0])) {
					buffer.append(" and affair.completeTime >= :completeTimeS");
					paramMap.put("completeTimeS", Datetimes.getTodayFirstTime(times[0]));
				}
				if(times.length>1 && Strings.isNotBlank(times[1])) {
					buffer.append(" and affair.completeTime <= :completeTimeE");
					paramMap.put("completeTimeE", Datetimes.getTodayLastTime(times[1]));
				}
			}
			if(Strings.isNotBlank(params.get("affairExpectedProcessTime"))) {//按处理期限查询
				String time = params.get("affairExpectedProcessTime").toString();
				String[] times = time.split("#");
				if(times.length>0 && Strings.isNotBlank(times[0])) {
					buffer.append(" and affair.expectedProcessTime is not null and affair.expectedProcessTime >= :affairExpectedProcessTimeS");
					paramMap.put("affairExpectedProcessTimeS", Datetimes.getTodayFirstTime(times[0]));
				}
				if(times.length>1 && Strings.isNotBlank(times[1])) {
					buffer.append(" and affair.expectedProcessTime is not null and affair.expectedProcessTime <= :affairExpectedProcessTimeE");
					paramMap.put("affairExpectedProcessTimeE", Datetimes.getTodayLastTime(times[1]));
				}
			}
			if(Strings.isNotBlank(params.get("affairSubject"))) {//按流程标题查询
				buffer.append(" and affair.subject like :affairSubject");
				paramMap.put("affairSubject", "%" + GovdocUtil.convertSpecialChat(params.get("affairSubject")) + "%");
			}
			if(Strings.isNotBlank(params.get("affairSubApp"))){
				buffer.append(" and affair.subApp in (:affairSubApp) ");
				String[] sas = params.get("affairSubApp").split(",");
				List<Integer> subApps = new ArrayList<Integer>();
				for(String s:sas){
					if(Strings.isNotBlank(s)){
						subApps.add(Integer.parseInt(s));
					}
				}
				paramMap.put("affairSubApp", subApps);
			}
			if(Strings.isNotBlank(params.get("backFromId"))) {//处理人查询
				String backFromIds_str = params.get("backFromId");
				String[] backFromIds = backFromIds_str.split(",");
				List<Long> backFromIdList = new ArrayList<Long>();
				for(String ss:backFromIds){
					backFromIdList.add(Long.parseLong(ss));
				}
				if (backFromIdList.size() <= 300) {
					buffer.append(" and affair.backFromId in (:backFromId)");
					paramMap.put("backFromId", backFromIdList);
				} else {// 300个一组用or连接
					Map<String, List<Long>> idMap = new HashMap<String, List<Long>>();
					List<Long> idList = new ArrayList<Long>();
					for (int i = 0; i < backFromIdList.size(); i++) {
						idList.add(backFromIdList.get(i));
						if (i != 0 && i % 300 == 0) {
							idMap.put("backFromId" + i, idList);
							idList = new ArrayList<Long>();
						}
						if (i == backFromIdList.size() - 1) {
							idMap.put("backFromId" + i, idList);
						}
					}
					buffer.append(" and (");
					int i = 0;
					for (String key : idMap.keySet()) {
						if (i != 0) {
							buffer.append(" or ");
						}
						i++;
						List<Long> ids = idMap.get(key);
						buffer.append(" affair.backFromId in (:" + key + ")");
						paramMap.put(key, ids);
					}
					buffer.append(" )");
				}
			}
			if(Strings.isNotBlank(params.get("memberIdSearch"))) {//按处理人查询
				String memberIds_str = params.get("memberIdSearch");
				String[] memberIds = memberIds_str.split(",");
				List<Long> memberIdList = new ArrayList<Long>();
				for(String ss:memberIds){
					memberIdList.add(Long.parseLong(ss));
				}
				if (memberIdList.size() <= 300) {
					buffer.append(" and affair.memberId in (:memberIdSearch)");
					paramMap.put("memberIdSearch", memberIdList);
				} else {// 300个一组用or连接
					Map<String, List<Long>> idMap = new HashMap<String, List<Long>>();
					List<Long> idList = new ArrayList<Long>();
					for (int i = 0; i < memberIdList.size(); i++) {
						idList.add(memberIdList.get(i));
						if (i != 0 && i % 300 == 0) {
							idMap.put("memberIdSearch" + i, idList);
							idList = new ArrayList<Long>();
						}
						if (i == memberIdList.size() - 1) {
							idMap.put("memberIdSearch" + i, idList);
						}
					}
					buffer.append(" and (");
					int i = 0;
					for (String key : idMap.keySet()) {
						if (i != 0) {
							buffer.append(" or ");
						}
						i++;
						List<Long> ids = idMap.get(key);
						buffer.append(" affair.memberId in (:" + key + ")");
						paramMap.put(key, ids);
					}
					buffer.append(" )");
				}
			}
			if(Strings.isNotBlank(params.get("turnRecUserId"))) {//按转办人查询
				String memberIds_str = params.get("turnRecUserId");
				if("1".equals(memberIds_str)){
					buffer.append(" and affair.turnRecUserId IS NOT NULL ");
				}else if("0".equals(memberIds_str)){
					buffer.append(" and affair.turnRecUserId IS NULL ");
				}else{
					String[] memberIds = memberIds_str.split(",");
					List<Long> memberIdList = new ArrayList<Long>();
					for(String ss:memberIds){
						memberIdList.add(Long.parseLong(ss));
					}
					if (memberIdList.size() <= 300) {
						buffer.append(" and affair.turnRecUserId in (:turnRecUserId)");
						paramMap.put("turnRecUserId", memberIdList);
					} else {// 300个一组用or连接
						Map<String, List<Long>> idMap = new HashMap<String, List<Long>>();
						List<Long> idList = new ArrayList<Long>();
						for (int i = 0; i < memberIdList.size(); i++) {
							idList.add(memberIdList.get(i));
							if (i != 0 && i % 300 == 0) {
								idMap.put("turnRecUserId" + i, idList);
								idList = new ArrayList<Long>();
							}
							if (i == memberIdList.size() - 1) {
								idMap.put("turnRecUserId" + i, idList);
							}
						}
						buffer.append(" and (");
						int i = 0;
						for (String key : idMap.keySet()) {
							if (i != 0) {
								buffer.append(" or ");
							}
							i++;
							List<Long> ids = idMap.get(key);
							buffer.append(" affair.turnRecUserId in (:" + key + ")");
							paramMap.put(key, ids);
						}
						buffer.append(" )");
					}
				}
			}
		}
		return buffer.toString();
	}
	
    @SuppressWarnings("unchecked")
	public List<Long> getSummaryIdByPishi(Map<String, String> params) {
 		 if(!(Boolean)SysFlag.sys_isG6S.getFlag()){
			if (Strings.isNotBlank(params.get("pishiName")) || Strings.isNotBlank(params.get("pishiYear")) || Strings.isNotBlank(params.get("pishiNo"))) {
				Map<String, Object> queryParams = new HashMap<String, Object>();// 查询条件
				StringBuilder hql = new StringBuilder();
				hql.append(" select summaryId from EdocLeaderPishiNo pishi");
				hql.append(" where 1=1 ");
				if (Strings.isNotBlank(params.get("pishiName"))) {
					hql.append(" and pishi.pishiName = :psName ");
					queryParams.put("psName", SQLWildcardUtil.escape(params.get("pishiName")));
				}
				if (Strings.isNotBlank(params.get("pishiYear"))) {
					String pishiYear = params.get("pishiYear");
					hql.append(" and pishi.pishiYear like :psYear ");
					queryParams.put("psYear", "%"+SQLWildcardUtil.escape(pishiYear)+"%");
				}
				if (Strings.isNotBlank(params.get("pishiNo"))) {
					String pishiNo = params.get("pishiNo");
					hql.append(" and pishi.pishiNo = :psNo ");
					try {
						queryParams.put("psNo", Integer.valueOf(SQLWildcardUtil.escape(pishiNo)));
					} catch (Exception e) {
						queryParams.put("psNo", -1);
					}
				}
				hql.append(" group by pishi.summaryId");
				return (List<Long>)DBAgent.find(hql.toString(), queryParams);
			}
		}
		 return null;
	 }
    
	private List<Long> transToMemberIds(List<V3xOrgMember> members) {
		List<Long> list = new ArrayList<Long>();
		for (V3xOrgMember member : members) {
			list.add(member.getId());
		}
		return list;
	}
	/************************ 以下为公文列表辅助Hql方法   end ***************************/
}
