package com.seeyon.apps.govdoc.listener;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.aspectj.lang.ProceedingJoinPoint;

import com.seeyon.apps.secret.util.SecretUtil;
import com.seeyon.ctp.common.ctpenumnew.manager.EnumManager;
import com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem;
import com.seeyon.ctp.form.bean.FormBean;
import com.seeyon.ctp.form.bean.FormDataMasterBean;
import com.seeyon.ctp.form.bean.FormFieldBean;
import com.seeyon.ctp.organization.bo.MemberPost;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.workflow.event.EventDataContext;
import com.seeyon.ctp.workflow.wapi.WorkflowNodeUsersMatchResult;

import net.joinwork.bpm.engine.wapi.WorkflowBpmContext;

/**
 * 流程中人员根据密级过滤aop
 * 
 * @author Administrator
 *
 */
public class GovDocWorkFlowMatchUserAspect {
	private EnumManager enumManagerNew;

	public void setEnumManagerNew(EnumManager enumManagerNew) {
		this.enumManagerNew = enumManagerNew;
	}

	public Object afterGetUserList(ProceedingJoinPoint JoinPoint) {
		Object returnValue=null;
		try {
			Object[] args = JoinPoint.getArgs();
			returnValue = JoinPoint.proceed(args);
			if (!SecretUtil.hasPluginSecret()) {
				return returnValue;
			}
			Map map=(Map) args[1];
			Map formData = (Map) map.get(EventDataContext.CTP_FORM_DATA);
			if(null==formData){
				return returnValue;
			}
			FormDataMasterBean formDataMasterBean = (FormDataMasterBean) formData.get("formDataBean");
			if(null==formDataMasterBean){
				return returnValue;
			}
			
			// 获取表单密级
			Integer secretLevel = null;
			List<FormFieldBean> formFieldBeans = formDataMasterBean.getFormTable().getFields();
			for(FormFieldBean fieldBean:formFieldBeans){
					if (null != fieldBean && "secret_level".equals(fieldBean.getMappingField())) {
						Object value = formDataMasterBean.getFieldValue(fieldBean.getName());
						if (value == null || Strings.isBlank(value.toString())) {
							secretLevel = 0;
							break;
						}
						CtpEnumItem item = enumManagerNew.getCacheEnumItem(Long.valueOf(value.toString()));
						if (item != null) {
							String secretLevelStr = item.getEnumvalue();
							secretLevel = Integer.valueOf(secretLevelStr);
						} else {
							secretLevel = 0;
						}
						break;
					}
			}
			// 过滤掉返回人员中不符合密级的人员
			if (returnValue != null && secretLevel != null) {
				WorkflowNodeUsersMatchResult matchResult=(WorkflowNodeUsersMatchResult)returnValue;
				List<V3xOrgMember> members = matchResult.getMembers();
				Iterator<V3xOrgMember> iterator = members.iterator();
				while (iterator.hasNext()) {
					V3xOrgMember member = iterator.next();
					int memberSecretLevel = member.getSecretLevel()==null?1:member.getSecretLevel();
					if (memberSecretLevel < secretLevel) {
						iterator.remove();
						Iterator<MemberPost> memberPostIterator = matchResult.getMemberPosts().iterator();
						while(memberPostIterator.hasNext()){
							if(memberPostIterator.next().getMemberId().longValue()==member.getId()){
								memberPostIterator.remove();
								break;
							}
						}
					}
				}
			}
			return returnValue;
		} catch (Throwable e) {
		}
		return returnValue;
	}
}
