package com.seeyon.apps.govdoc.listener;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.seeyon.apps.govdoc.manager.GovdocGenerateManager;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.form.api.FormApi4Cap3;
import com.seeyon.ctp.organization.event.AddAccountEvent;
import com.seeyon.ctp.organization.event.AddAdminMemberEvent;
import com.seeyon.ctp.organization.event.DeleteDepartmentEvent;
import com.seeyon.ctp.util.annotation.ListenEvent;

public class GovdocOrganizationEventListener {
	
	private GovdocGenerateManager govdocGenerateManager;
	private FormApi4Cap3 formApi4Cap3;
	
	private static final Log log = LogFactory.getLog(GovdocOrganizationEventListener.class);

	/**
	 *   新建单位时，表单公文触发的事件
	 * @param evt
	 * @throws Exception
	 */
	@ListenEvent(event=AddAccountEvent.class)
	public void onAddAccount(AddAccountEvent evt) throws Exception {
		if(!AppContext.hasPlugin("edoc")) {
    		return;
    	}
		govdocGenerateManager.generate(evt.getAccount());
	}
	
	/**
	 *   新建单位管理员时，表单公文触发的事件
	 * @param evt
	 * @throws Exception
	 */
	@ListenEvent(event=AddAdminMemberEvent.class,async = true)
	public void onAddAdminMember(AddAdminMemberEvent evt) throws Exception {
		if(!AppContext.hasPlugin("edoc")) {
    		return;
    	}
		try {
			formApi4Cap3.savePresetGovdocform(evt.getMember());
		} catch (Exception e) {
			log.error("新建单位管理员时，表单公文触发的事件失败！",e);
		}
		//govdocGenerateManager.generateGovform(evt.getMember());
	}

	/**
	 * 监听删除部门事件，从机构组中删除
	 * @param evt
	 * @throws Exception
	 */
	@ListenEvent(event=DeleteDepartmentEvent.class)
	public void onDeleteDepartment(DeleteDepartmentEvent evt) throws Exception {
		if(!AppContext.hasPlugin("edoc")) {
    		return;
    	}
		govdocGenerateManager.deleteEdocObjTeam(evt.getDept());
	}

	public void setGovdocGenerateManager(GovdocGenerateManager govdocGenerateManager) { this.govdocGenerateManager = govdocGenerateManager; }
	public void setFormApi4Cap3(FormApi4Cap3 formApi4Cap3) { this.formApi4Cap3 = formApi4Cap3; }
	
}
