/**
 * 
 */
package com.seeyon.apps.govdoc.listener;

import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;

import com.seeyon.apps.govdoc.manager.GovdocElementManager;
import com.seeyon.apps.govdoc.manager.GovdocGenerateManager;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.common.plugin.PluginAddEvent;
import com.seeyon.ctp.organization.bo.V3xOrgAccount;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.annotation.ListenEvent;

/**
 * 新增edoc插件，需补全公文基础数据
 * @author : 唐桂林
 * @Date : 2019年3月16日
 */
public class GovdocPluginChangeListener {
	
	private static Log LOGGER = CtpLogFactory.getLog(GovdocPluginChangeListener.class);
	
	private GovdocGenerateManager govdocGenerateManager;
	private GovdocElementManager govdocElementManager;
	private OrgManager orgManager;

	@ListenEvent(event = PluginAddEvent.class)
	public void onPluginAdd(PluginAddEvent evt) {
		Set<String> plugins = evt.getPlugins();
		if (Strings.isNotEmpty(plugins) && plugins.contains("edoc")) {
			LOGGER.info("edoc插件补偿");
			try {
				// 遍历各单位，确定是否未初始化；如未初始化，则进行补偿。
				List<V3xOrgAccount> allAccounts = orgManager.getAllAccounts();
				if (Strings.isNotEmpty(allAccounts)) {
					//直接检查补偿公文元素
					govdocElementManager.fixElements(null);
					//boolean isCreate = isNeedCreate(allAccounts);

					//if (isCreate) {
					for (V3xOrgAccount account : allAccounts) {
						if (!account.isGroup()) {
							LOGGER.info("复制公文分类数据开始...");
							govdocGenerateManager.generateTemplateCategory(account);
							LOGGER.info("复制公文分类数据结束...");
						}
					}
					//}
				}
			} catch (Exception e) {
				LOGGER.error("edoc插件补偿：", e);
			}
		}
	}

	private boolean isNeedCreate(List<V3xOrgAccount> allAccounts) throws BusinessException {
		boolean isCreate = false;
		if (Strings.isNotEmpty(allAccounts)) {
			long accountId = 0;
			for (V3xOrgAccount account : allAccounts) {
				if (!account.isGroup()) {
					accountId = account.getId();
					/**
					 * 1、判断是否有单位没有公文元素
					 */
					int count = govdocElementManager.getAllEdocElementCount(accountId);
					if (count == 0) {
						return true;
					}
				}
			}
		}
		return isCreate;
	}
	

	public void setOrgManager(OrgManager orgManager) { this.orgManager = orgManager; }
	public void setGovdocElementManager(GovdocElementManager govdocElementManager) { this.govdocElementManager = govdocElementManager; }
	public void setGovdocGenerateManager(GovdocGenerateManager govdocGenerateManager) { this.govdocGenerateManager = govdocGenerateManager; }
	
}
