package com.seeyon.apps.govdoc.resourceManager;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;

import com.seeyon.apps.edoc.constants.EdocConstant.flowState;
import com.seeyon.apps.govdoc.constant.GovdocEnum;
import com.seeyon.apps.govdoc.dao.GovdocListDao;
import com.seeyon.apps.govdoc.helper.GovdocHelper;
import com.seeyon.apps.govdoc.helper.GovdocListHelper;
import com.seeyon.apps.govdoc.manager.GovdocElementManager;
import com.seeyon.apps.govdoc.manager.GovdocListManager;
import com.seeyon.apps.govdoc.manager.GovdocPishiManager;
import com.seeyon.apps.govdoc.resourceManager.GovdocResourceEnum.ResourceMemberTypeEnums;
import com.seeyon.apps.govdoc.resourceManager.vo.GovdocResourceListVO;
import com.seeyon.apps.govdoc.util.GovdocAuthorityUtil;
import com.seeyon.apps.govdoc.util.GovdocUtil;
import com.seeyon.apps.govdoc.vo.GovdocListVO;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.affair.util.WFComponentUtil;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.constants.ApplicationSubCategoryEnum;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.ctpenumnew.EnumNameEnum;
import com.seeyon.ctp.common.ctpenumnew.manager.EnumManager;
import com.seeyon.ctp.common.dump.vo.DumpDataVO;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.permission.manager.PermissionManager;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.po.ctpenumnew.CtpEnumBean;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.resourceEngine.bo.ResourceItem;
import com.seeyon.ctp.resourceEngine.bo.ResourceItemValueBO;
import com.seeyon.ctp.resourceEngine.bo.ResourceValueBO;
import com.seeyon.ctp.resourceEngine.manager.PrivilegeConfigManager;
import com.seeyon.ctp.resourceEngine.manager.ResourceItemHandler;
import com.seeyon.ctp.resourceEngine.util.ResourceMenuContact;
import com.seeyon.ctp.util.DBAgent;
import com.seeyon.ctp.util.FlipInfo;
import com.seeyon.ctp.util.IOUtility;
import com.seeyon.ctp.util.IdentifierUtil;
import com.seeyon.ctp.util.JDBCAgent;
import com.seeyon.ctp.util.StringUtil;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.XXEUtil;
import com.seeyon.v3x.edoc.domain.EdocSummary;

public class GovdocResourceItemHandler extends GovdocListDao implements ResourceItemHandler {
	private final static Log logger = LogFactory.getLog(GovdocResourceItemHandler.class);
	private PrivilegeConfigManager privilegeConfigManager;
	private OrgManager orgManager;
    private EnumManager enumManagerNew;
    private PermissionManager permissionManager;
	private AffairManager affairManager;
	@Override
	public String getPlugin() {
		return "govdoc";
	}

	@Override
	public String getCode() {
		return "govdocResource";
	}

	@Override
	public String getName() {
		return "公文资源";
	}

	@Override
	public String getBeanId() {
		return "govdocResourceItemHandler";
	}

	@Override
	public List<ResourceItem> addItems() {
		List<ResourceItem> allItem = new ArrayList<ResourceItem>();
		String xmlPath = AppContext.getCfgHome().getAbsolutePath() + File.separator + "plugin" + File.separator + "govdoc" + File.separator + "resourceItems.xml";
		InputStream in = null;
		try{
			in = new FileInputStream(new File(xmlPath));
			
			String content = IOUtility.toString(new InputStreamReader(in,"UTF-8"));
			Document doc;
			try {
				doc = XXEUtil.safeParseText(content);
				Element rootElm = doc.getRootElement();
				if (rootElm == null) {
					throw new BusinessException("根节点不存在 ");
				}

				List<Element> elements = rootElm.elements("itemList");
				if(elements!=null && elements.size()>0){
					elements = elements.get(0).elements();
				}
				for(Element aelement:elements){
					if (aelement != null) {
						ResourceItemGovdoc item = new ResourceItemGovdoc();
						/*<item i18n="org.dealUser" code="affair.member" desc="查询事项处理范围" sort="0" 
						dataType="string" selectType="select" enumId="" itemType="dealMembers" className="" 
								canShow="0" dataSource="1" searchSource="0" isButton="0"></item>*/
						Attribute fattrib = aelement.attribute("code");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setCode(fattrib.getValue());
							}
						}
						fattrib = aelement.attribute("elementCode");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setElementCode(fattrib.getValue());
							}
						}
						fattrib = aelement.attribute("isButton");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setButton("1".equals(fattrib.getValue())?true:false);
							}
						}
						fattrib = aelement.attribute("searchSource");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setSearchSource("1".equals(fattrib.getValue())?true:false);
							}
						}
						fattrib = aelement.attribute("dataSource");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setDataSource("1".equals(fattrib.getValue())?true:false);
							}
						}
						fattrib = aelement.attribute("canShow");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setCanShow("1".equals(fattrib.getValue())?true:false);
							}
						}
						fattrib = aelement.attribute("hbmAttr");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setHbmAttr(fattrib.getValue());
							}
						}
						fattrib = aelement.attribute("className");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setClassName(fattrib.getValue());
							}
						}
						fattrib = aelement.attribute("itemType");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setItemType(fattrib.getValue());
							}
						}
						fattrib = aelement.attribute("enumId");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setEnumId(Long.parseLong(fattrib.getValue()));
							}
						}
						fattrib = aelement.attribute("selectType");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setSelectType(fattrib.getValue());
							}
						}
						fattrib = aelement.attribute("dataType");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setDataType(fattrib.getValue());
							}
						}
						fattrib = aelement.attribute("i18n");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setI18n(fattrib.getValue());
							}
						}
						fattrib = aelement.attribute("desc");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setDesc(fattrib.getValue());
							}
						}
						fattrib = aelement.attribute("sort");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								item.setSort(Integer.parseInt(fattrib.getValue()));
							}
						}
						allItem.add(item);
					}
				}
			} catch (DocumentException arg10) {
				throw new BusinessException("XML格式不对", arg10);
			}
		}catch(Exception e){
			logger.error(e);
		}
		return allItem;
	}

	@Override
	public FlipInfo getList(FlipInfo flipInfo, Map<String, Object> params1) throws BusinessException {
		Object reId_str = params1.get("resourceId");
		Map<String, String> params = new HashMap<String,String>();
		if(reId_str != null && String.valueOf(ResourceMenuContact.RESOURCE_ID_LEADERPISHI_PENDDING).equals(reId_str.toString()) && params1.containsKey("selectCount")){
			GovdocPishiManager manager = (GovdocPishiManager)AppContext.getBean("govdocPishiManager");
			params.put("selectCount", "true");
			params.put("leaderPishiType", "");
			try {
				manager.getlistPishiList(flipInfo,params);
			} catch (ParseException e) {
				logger.error(e);
			}
			return flipInfo;
		}else if(reId_str != null && String.valueOf(ResourceMenuContact.RESOURCE_ID_ExchangeSend_PENDDING).equals(reId_str.toString())){
			GovdocListManager govdocListManager = (GovdocListManager)AppContext.getBean("govdocListManager");
			params.put("onlyCount", "true");
			params.put("listType", "listExchangeSendPending");
			return govdocListManager.findPendingList(flipInfo, params);
		}else if(reId_str != null && String.valueOf(ResourceMenuContact.RESOURCE_ID_ExchangeSign_PENDDING).equals(reId_str.toString())){
			GovdocListManager govdocListManager = (GovdocListManager)AppContext.getBean("govdocListManager");
			params.put("listType", "listExchangeSignPending");
			params.put("onlyCount", "true");
			return govdocListManager.findPendingList(flipInfo, params);
		}
		long d1 = System.currentTimeMillis();
		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		boolean hasDeduplication = false;
		boolean selectCountOnly = false;
		Set<String> bigFields = new HashSet<String>();
		bigFields.add("nibanyijian");//大字段的code
		bigFields.add("currentNodesInfo");
		bigFields.add("sendTo");
		bigFields.add("copyTo");
		bigFields.add("reportTo");
		bigFields.add("printUnit");
		bigFields.add("sendUnit");
		bigFields.add("sendDepartment");
		List<String> extendField4Show = new ArrayList<String>();//都显示哪些扩展字段
		List<String> bigField4Show = new ArrayList<String>();//都显示大字段
		boolean hasShowExtendField = false;//顯示是否有扩展字段
		boolean hasWhereExtendField = false;//查询是否有扩展字段
		boolean isFormM = com.seeyon.ctp.common.constants.Constants.login_useragent_from.pc == AppContext.getCurrentUser().getUserAgentFromEnum()?false:true;
		String conditionKey="condition";
		for(String key:params1.keySet()){
			//if("condition".equals(key)){
			if(key.startsWith(conditionKey)){
				String conditionSuffix=key.substring(key.indexOf(conditionKey)+conditionKey.length());
				Object value = params1.get("value"+conditionSuffix);
				if("datemulti".equals(params1.get("type"))){
					List<Object> vList = (List<Object>)value;
					value = String.valueOf(vList.get(0))+"#" +String.valueOf(vList.get(1));
				}
				String paramKey = String.valueOf(params1.get(key));
				if(paramKey.startsWith("list")){
					hasWhereExtendField = true;
				}
				if("affairReceiveTime".equals(paramKey)){//receiveTime
					paramKey = "receiveTime";
				}else if("startUserName".equals(paramKey)){//发起者
					if(Strings.isEmpty(value.toString().trim())){
						continue;
					}
					List<V3xOrgMember> memberList = null;
					try {
						memberList = orgManager.getMemberByIndistinctName(value.toString().trim());
					} catch (BusinessException e) {
						e.printStackTrace();
					}
					if(Strings.isEmpty(memberList)){
						return flipInfo;
					}
					List<Long> members = new ArrayList<Long>();
					for (V3xOrgMember member : memberList) {
						members.add(member.getId());
					}
					value = Strings.join(members, ",");
					paramKey = "startUserId";
				}else if("affairDeadLineName".equals(paramKey)){
					paramKey = "affairExpectedProcessTime";
				}else if("subState".equals(paramKey)){
					paramKey = "draftState";
				}else if("backFromName".equals(paramKey)){
					if(Strings.isEmpty(value.toString().trim())){
						continue;
					}
					List<V3xOrgMember> memberList = null;
					try {
						memberList = orgManager.getMemberByIndistinctName(value.toString().trim());
					} catch (BusinessException e) {
						e.printStackTrace();
					}
					if(Strings.isEmpty(memberList)){
						return flipInfo;
					}
					List<Long> members = new ArrayList<Long>();
					for (V3xOrgMember member : memberList) {
						members.add(member.getId());
					}
					value = Strings.join(members, ",");
					paramKey = "backFromId";
				//20200313:GOVB-2612 在办列表处理人为空查询，应该查询出所有数据
				}else if("memberName".equals(paramKey)&&Strings.isNotEmpty(value.toString().trim())){
					List<V3xOrgMember> memberList = null;
					try {
						memberList = orgManager.getMemberByIndistinctName(value.toString().trim());
					} catch (BusinessException e) {
						e.printStackTrace();
					}
					if(Strings.isEmpty(memberList)){
						return flipInfo;
					}
					List<Long> members = new ArrayList<Long>();
					for (V3xOrgMember member : memberList) {
						members.add(member.getId());
					}
					value = Strings.join(members, ",");
					paramKey = "memberIdSearch";
				}else if("affairCompleteTime".equals(paramKey)){
					paramKey = "completeTime";
				}else if("receiptDate".equals(paramKey)){
					paramKey = "recieveDate";
				}else if("govdocType".equals(paramKey)){
					//paramKey = "edocTypeSearch";
					paramKey = "affairSubApp";
				}
				params.put(paramKey, String.valueOf(value));
			}
			if("selectCount".equals(key)){
				selectCountOnly = true;
				continue;
			}
			if("deduplication".equals(key) && "true".equals(params1.get("deduplication"))){
				hasDeduplication = true;
			}
			params.put(key, String.valueOf(params1.get(key)));
		}
		Object resourceId_str = params.get("resourceId");
		ResourceValueBO rvb = null;
		if(resourceId_str!=null){
			Long resourceId = Long.parseLong(resourceId_str.toString());
			rvb = privilegeConfigManager.findByResourceId(resourceId);
			if(rvb != null){
				for(ResourceItemValueBO value : rvb.getDataShowColumns()){
					if(value.getCode().startsWith("list")){
						hasShowExtendField = true;
						extendField4Show.add("extend."+value.getCode());
					}else if(bigFields.contains(value.getCode())){
						bigField4Show.add(value.getResourceItem().getHbmAttr(null));
					}
				}
				for(ResourceItemValueBO value : rvb.getDataSources()){
					if(value.getCode().startsWith("list")){
						hasWhereExtendField = true;
						break;
					}
				}
				Map<String, Object> paramMap = new LinkedHashMap<String, Object>();
				params.put("memberId", String.valueOf(AppContext.currentUserId()));
				StringBuilder bufferSelect = new StringBuilder();
				//后续需要啥子查啥子。。。。//TODO
				bufferSelect.append("select " + (isFormM ? GovdocResourceListVO.NEEDED_FIELDS_4M : GovdocResourceListVO.NEEDED_FIELDS_4PC) + "");
				if(Strings.isNotEmpty(bigField4Show)){
					bufferSelect.append(",").append(Strings.join(bigField4Show, ","));
				}
				if(Strings.isNotEmpty(extendField4Show) && hasWhereExtendField){
					bufferSelect.append(",").append(Strings.join(extendField4Show, ","));
				}
				StringBuilder fromBuffer = new StringBuilder();
				StringBuilder whereBuffer = new StringBuilder();
				fromBuffer.append(" from EdocSummary summary");
				fromBuffer.append(" , CtpAffair affair");
				if(hasWhereExtendField){
					fromBuffer.append(", EdocSummaryExtend extend");
				}
				StringBuilder whereNeededBuffer = new StringBuilder();
				whereNeededBuffer.append(" where summary.id = affair.objectId");
				if(hasWhereExtendField){
					whereNeededBuffer.append(" and summary.id=extend.summaryId");
				}
				whereBuffer.append(" and affair.app=:app");
				paramMap.put("app", ApplicationCategoryEnum.edoc.getKey());
				whereBuffer.append(" and affair.delete = :isDelete");
				paramMap.put("isDelete", Boolean.FALSE);
				//2020-3-4 ch 修改：解决为空造成多一个and而sql语句出错的问题
				String otherCondi=rvb.toSql(paramMap);
				if (Strings.isNotEmpty(otherCondi)) {
					whereBuffer.append(" and ").append(otherCondi); 
				}
				whereBuffer.append(this.getAffairHqlByCondition(paramMap, params));
				whereBuffer.append(this.getHqlByCondition(paramMap, params));
				whereBuffer.append(this.getExtendHqlByCondition(paramMap, params));
				//添加分保插件
				whereBuffer.append(getHqlByMemberSecret(paramMap,params,"summary"));
				
				boolean hasPending = false;
				boolean hasDone = false;
				for(ResourceItemValueBO itemValue:rvb.getDataSources()){
					if("affairState".equals(itemValue.getCode())){
						if(itemValue.getDbValue().toString().contains(""+StateEnum.col_pending.getKey())){
							hasPending = true;
						}else if(itemValue.getDbValue().toString().contains(""+StateEnum.col_done.getKey())){
							hasDone = true;
						}
					}
				}
				
				String orderField = "";
				if (hasPending) {// 待办
					orderField = " affair.receiveTime ";
				} else if (hasDone) {// 已办
					orderField = " affair.completeTime ";
				} else {
					orderField = " summary.createTime ";
				}
				String sqlOrder = " order by" + orderField +"desc ";
				

				//同一流程只显示一条
				StringBuffer runSql = new StringBuffer();
				if(hasDeduplication){//转jdbc
					StringBuffer sonSql = new StringBuffer();
					sonSql.append(" select MAX(affair.id) as aId ");
					sonSql.append(" from Edoc_Summary summary");
					sonSql.append(" , Ctp_Affair affair");
					if(hasWhereExtendField){
						sonSql.append(", Edoc_Summary_Extend extend");
					}
					sonSql.append(" where summary.id = affair.object_Id");
					if(hasWhereExtendField){
						sonSql.append(" and summary.id=extend.summary_Id");
					}
					sonSql.append(whereBuffer);
					sonSql.append("").append(" group by affair.object_Id ");
					//String hSelect ="summary.id,summary.subject,summary.identifier,summary.govdocType,summary.state,summary.isQuickSend,summary.templeteId,summary.archiveId,summary.processId,summary.caseId,summary.startUserId,summary.startTime,summary.createTime,summary.completeTime,summary.deadline,summary.deadlineDatetime,summary.coverTime,summary.docMark,summary.serialNo,summary.docMark2,summary.docType,summary.sendType,summary.secretLevel,summary.urgentLevel,summary.unitLevel,summary.keepPeriod,summary.copies,summary.issuer,summary.signingDate,summary.createPerson,summary.printer,summary.phone,summary.orgAccountId,summary.exchangeSendAffairId,summary.undertaker,summary.undertakenoffice,summary.bodyType,summary.hasArchive,summary.signPerson,summary.receiptDate,summary.registrationDate,summary.fromType,summary.newflowType,summary.currentNodesInfo, summary.sendTo, summary.copyTo, summary.reportTo, summary.printUnit, summary.sendUnit, summary.sendDepartment,affair.id,affair.identifier,affair.subject,affair.app,affair.subApp,affair.state,affair.subState,affair.finish,affair.objectId,affair.subObjectId,affair.activityId,affair.nodePolicy,affair.memberId,affair.transactorId,affair.hastenTimes,affair.coverTime,affair.remindDate,affair.deadlineDate,affair.receiveTime,affair.completeTime,affair.createDate,affair.updateDate,affair.expectedProcessTime,affair.importantLevel,affair.archiveId,affair.track,affair.backFromId,affair.preApprover,affair.proxyMemberId,affair.transactorId,affair.senderId ,affair.turnRecUserId";
					//String select = "summary.id,summary.SUBJECT,summary.identifier,summary.govdoc_type,summary.state,summary.is_quick_send,summary.templete_id,summary.archive_id,summary.process_id,summary.case_id,summary.start_user_id,summary.start_time,summary.create_time,summary.complete_time,summary.deadline,summary.deadline_datetime,summary.IS_COVER_TIME,summary.doc_mark,summary.serial_no,summary.doc_mark2,summary.doc_type,summary.send_type,summary.secret_level,summary.urgent_level,summary.unit_level,summary.keep_period,summary.copies,summary.ISSUER,summary.signing_date,summary.create_person,summary.printer,summary.phone,summary.org_account_id,summary.exchange_send_affairId,summary.undertaker,summary.undertakenoffice,summary.BODY_TYPE,summary.has_archive,summary.sign_person,summary.receipt_date,summary.registration_date,summary.from_type,summary.newflow_type,summary.current_nodes_info,summary.send_to,summary.copy_to,summary.report_to,summary.print_unit,summary.send_unit,summary.send_department,affair.ID,affair.IDENTIFIER,affair.SUBJECT,affair.APP,affair.SUB_APP,affair.STATE,affair.SUB_STATE,affair.IS_FINISH,affair.OBJECT_ID,affair.SUB_OBJECT_ID,affair.ACTIVITY_ID,affair.NODE_POLICY,affair.MEMBER_ID ,affair.TRANSACTOR_ID,affair.HASTEN_TIMES,affair.IS_COVER_TIME,affair.REMIND_DATE,affair.DEADLINE_DATE,affair.RECEIVE_TIME,affair.COMPLETE_TIME,affair.CREATE_DATE,affair.UPDATE_DATE,affair.EXPECTED_PROCESS_TIME,affair.IMPORTANT_LEVEL,affair.ARCHIVE_ID,affair.TRACK,affair.back_from_id,affair.PRE_APPROVER,affair.PROXY_MEMBER_ID,affair.TRANSACTOR_ID,affair.SENDER_ID,affair.TURN_REC_USER_ID";
					Map<String, String> hqlToSqlMap = new LinkedHashMap<String, String>();
					hqlToSqlMap.put("summary.govdocType", "summary.govdoc_type");
					hqlToSqlMap.put("summary.isQuickSend", "summary.is_quick_send");
					hqlToSqlMap.put("summary.templeteId", "summary.templete_id");
					hqlToSqlMap.put("summary.archiveId", "summary.archive_id");
					hqlToSqlMap.put("summary.processId", "summary.process_id");
					hqlToSqlMap.put("summary.caseId", "summary.case_id");
					hqlToSqlMap.put("summary.startUserId", "summary.start_user_id");
					hqlToSqlMap.put("summary.startTime", "summary.start_time");
					hqlToSqlMap.put("summary.createTime", "summary.create_time");
					hqlToSqlMap.put("summary.completeTime", "summary.complete_time");
					hqlToSqlMap.put("summary.deadlineDatetime", "summary.deadline_datetime");
					hqlToSqlMap.put("summary.coverTime", "summary.IS_COVER_TIME");
					hqlToSqlMap.put("summary.docMark", "summary.doc_mark");
					hqlToSqlMap.put("summary.serialNo", "summary.serial_no");
					hqlToSqlMap.put("summary.docMark2", "summary.doc_mark2");
					hqlToSqlMap.put("summary.docType", "summary.doc_type");
					hqlToSqlMap.put("summary.sendType", "summary.send_type");
					hqlToSqlMap.put("summary.secretLevel", "summary.secret_level");
					hqlToSqlMap.put("summary.urgentLevel", "summary.urgent_level");
					hqlToSqlMap.put("summary.unitLevel", "summary.unit_level");
					hqlToSqlMap.put("summary.keepPeriod", "summary.keep_period");
					hqlToSqlMap.put("summary.signingDate", "summary.signing_date");
					hqlToSqlMap.put("summary.createPerson", "summary.create_person");
					hqlToSqlMap.put("summary.orgAccountId", "summary.org_account_id");
					hqlToSqlMap.put("summary.exchangeSendAffairId", "summary.exchange_send_affairId");
					hqlToSqlMap.put("summary.bodyType", "summary.BODY_TYPE");
					hqlToSqlMap.put("summary.hasArchive", "summary.has_archive");
					hqlToSqlMap.put("summary.signPerson", "summary.sign_person");
					hqlToSqlMap.put("summary.receiptDate", "summary.receipt_date");
					hqlToSqlMap.put("summary.registrationDate", "summary.registration_date");
					hqlToSqlMap.put("summary.fromType", "summary.from_type");
					hqlToSqlMap.put("summary.newflowType", "summary.newflow_type");
					hqlToSqlMap.put("summary.currentNodesInfo", "summary.current_nodes_info");
					hqlToSqlMap.put("summary.sendTo", "summary.send_to");
					hqlToSqlMap.put("summary.copyTo", "summary.copy_to");
					hqlToSqlMap.put("summary.reportTo", "summary.report_to");
					hqlToSqlMap.put("summary.printUnit", "summary.print_unit");
					hqlToSqlMap.put("summary.sendUnit", "summary.send_unit");
					hqlToSqlMap.put("summary.sendDepartment", "summary.send_department");
					hqlToSqlMap.put("affair.subApp", "affair.SUB_APP");
					hqlToSqlMap.put("affair.subState", "affair.SUB_STATE");
					hqlToSqlMap.put("affair.finish", "affair.IS_FINISH");
					hqlToSqlMap.put("affair.objectId", "affair.OBJECT_ID");
					hqlToSqlMap.put("affair.subObjectId", "affair.SUB_OBJECT_ID");
					hqlToSqlMap.put("affair.activityId", "affair.ACTIVITY_ID");
					hqlToSqlMap.put("affair.nodePolicy", "affair.NODE_POLICY");
					hqlToSqlMap.put("affair.memberId", "affair.MEMBER_ID");
					hqlToSqlMap.put("affair.transactorId", "affair.TRANSACTOR_ID");
					hqlToSqlMap.put("affair.hastenTimes", "affair.HASTEN_TIMES");
					hqlToSqlMap.put("affair.coverTime", "affair.IS_COVER_TIME");
					hqlToSqlMap.put("affair.remindDate", "affair.REMIND_DATE");
					hqlToSqlMap.put("affair.deadlineDate", "affair.DEADLINE_DATE");
					hqlToSqlMap.put("affair.receiveTime", "affair.RECEIVE_TIME");
					hqlToSqlMap.put("affair.completeTime", "affair.COMPLETE_TIME");
					hqlToSqlMap.put("affair.createDate", "affair.CREATE_DATE");
					hqlToSqlMap.put("affair.updateDate", "affair.UPDATE_DATE");
					hqlToSqlMap.put("affair.expectedProcessTime", "affair.EXPECTED_PROCESS_TIME");
					hqlToSqlMap.put("affair.importantLevel", "affair.IMPORTANT_LEVEL");
					hqlToSqlMap.put("affair.archiveId", "affair.ARCHIVE_ID");
					hqlToSqlMap.put("affair.backFromId", "affair.back_from_id");
					hqlToSqlMap.put("affair.preApprover", "affair.PRE_APPROVER");
					hqlToSqlMap.put("affair.proxyMemberId", "affair.PROXY_MEMBER_ID");
					hqlToSqlMap.put("affair.transactorId", "affair.TRANSACTOR_ID");
					hqlToSqlMap.put("affair.senderId", "affair.SENDER_ID");
					hqlToSqlMap.put("affair.turnRecUserId", "affair.TURN_REC_USER_ID");
					hqlToSqlMap.put("EdocSummaryExtend", "Edoc_Summary_Extend");
					hqlToSqlMap.put("extend.summaryId", "extend.summary_Id");
					hqlToSqlMap.put("EdocSummary", "Edoc_Summary");
					hqlToSqlMap.put("CtpAffair", "Ctp_affair");
					hqlToSqlMap.put("affair.delete", "affair.is_delete");
					hqlToSqlMap.put("summary.edocType", "summary.edoc_type");
					hqlToSqlMap.put("affair.matchDepartmentId", "affair.match_Department_Id");
					hqlToSqlMap.put("summary.packTime", "summary.pack_date");
					hqlToSqlMap.put("summary.dealSuggestion", "summary.deal_Suggestion");

					Map<String, String> selectMap = new LinkedHashMap<String, String>();
					selectMap.put("summary.id,", "summary.id as summaryId,");
					selectMap.put("summary.identifier,", "summary.identifier as summaryIdentifier,");
					selectMap.put("summary.subject,", "summary.subject as sSubject,");
					selectMap.put("summary.state,", "summary.state as summaryState,");
					selectMap.put("summary.IS_COVER_TIME,", "summary.IS_COVER_TIME as sCoverTime,");
					selectMap.put("summary.complete_time,", "summary.complete_time as sCompleteTime,");
					selectMap.put("summary.archive_id,", "summary.archive_id as sArchiveId,");
					selectMap.put("affair.TRANSACTOR_ID,", "affair.TRANSACTOR_ID as aTRANSACTORID,");
					runSql.append(bufferSelect);
					runSql.append(fromBuffer);
					runSql.append(" , (").append(sonSql).append(") sonAffair");
					runSql.append(whereNeededBuffer);
					runSql.append(" and sonAffair.aId=affair.id ");
					runSql.append(sqlOrder);
					String tem = runSql.toString();
					for(String key:hqlToSqlMap.keySet()){
						tem = tem.replaceAll(key, hqlToSqlMap.get(key));
					}
					for(String key:selectMap.keySet()){
						tem = tem.replaceFirst(key, selectMap.get(key));
					}
					runSql = new StringBuffer(tem);
				}else{
					runSql.append(bufferSelect).append(fromBuffer).append(whereNeededBuffer).append(whereBuffer).append(sqlOrder);
				}
				
				boolean dumpData = DumpDataVO.dataType.dumpData.getKey().equals(params.get("dumpData"));
				logger.info("--------------------getList     组装SQL：" + (System.currentTimeMillis() - d1) + "MS");
				List<Object[]> result = null;
				if(flipInfo != null && !dumpData) {// 查询转储数据必须要用super的find相关方法才能查询分库
					if(hasDeduplication){
						JDBCAgent jdbc = new JDBCAgent(true, false);
				        try {
				        	if(selectCountOnly){
				        		int count  = jdbc.count(runSql.toString(), paramMap);
				        		flipInfo.setTotal(count);
				        		return flipInfo;
				        	}else{
				        		flipInfo  = jdbc.findNameByPaging(runSql.toString(), paramMap,flipInfo);
				        	}
				        	List<String> booleanList = new ArrayList<String>();
				        	List<Map> l = flipInfo.getData();
				        	result = new ArrayList<Object[]>();
				        	booleanList.add("is_quick_send");
				        	booleanList.add("scovertime");
				        	booleanList.add("has_archive");
				        	booleanList.add("is_finish");
				        	booleanList.add("is_cover_time");
				            for (Map<String,Object> map : l) {
				            	Object[] obj = new Object[map.size()];
				            	Iterator ite = map.keySet().iterator();
				            	int j = 0;
				            	while (ite.hasNext()) {
				                	String key = (String) ite.next();
				                	Object a =map.get(key);
				                	if(a != null && booleanList.contains(key) && Strings.isNotBlank(String.valueOf(a))) {
				                		a = Integer.valueOf(String.valueOf(a)) == 0 ? false : true;
				                	}
				                	obj[j] = a;
				                	j++;
				                }
				            	result.add(obj);
				            }
						} catch (Exception e) {
							logger.error("合并同一流程查询错误！",e);
						} finally {
							jdbc.close();
						}
					}else{
						if(selectCountOnly){
			        		int count  = DBAgent.count(runSql.toString(), paramMap);
			        		flipInfo.setTotal(count);
			        		return flipInfo;
			        	}else{
			        		result = DBAgent.find(runSql.toString(), paramMap,flipInfo);
			        	}
					}
				} else {
					if(selectCountOnly){
		        		int count  = super.count(runSql.toString(), paramMap);
		        		flipInfo.setTotal(count);
		        		return flipInfo;
		        	}else{
		        		result = super.find(runSql.toString(), paramMap);
		        	}
				}
				logger.info("--------------------getList     执行SQL的paramMap参数：" + paramMap.toString());
				logger.info("--------------------getList     执行SQL：" + (System.currentTimeMillis() - d1) + "MS");
				if(Strings.isNotEmpty(result)) {
					EnumManager enumManagerNew = (EnumManager)AppContext.getBean("enumManagerNew");
					GovdocElementManager elementManager = (GovdocElementManager)AppContext.getBean("govdocElementManager");
					bigField4Show.addAll(extendField4Show);
					for(Object[] object : result) {
						GovdocResourceListVO vo = new GovdocResourceListVO();
						if(isFormM){
							vo.toObject4M(object, false);
						}else{
							if(Strings.isNotEmpty(extendField4Show) && !hasWhereExtendField){//需要單獨查詢extnd
								String sql = "select " + Strings.join(extendField4Show, ",") +" from EdocSummaryExtend extend where extend.summaryId=:summaryId";
								Map<String,Object> m = new HashMap<String,Object>();
								m.put("summaryId", object[0]);
								List<Object[]> extend = DBAgent.find(sql, m);
								int dataLength = object.length;
								object = Arrays.copyOf(object,dataLength + extendField4Show.size());
								if(Strings.isNotEmpty(extend)){
									int count = 0;
									if(extendField4Show.size() == 1){
										object[dataLength + (count++)] = extend.get(0);
									}else{
										for(Object ex:extend.get(0)){
											object[dataLength + (count++)] = ex;
										}
									}
								}
							}
							vo.toObject(object, false,bigField4Show, enumManagerNew, elementManager);
						}
						voList.add(vo);
					}
				}
			}
		}
		voList = setListVO(voList, params,rvb);
		logger.info("--------------------getList     setListVO：" + (System.currentTimeMillis() - d1) + "MS"); 
		if(flipInfo == null) {
			flipInfo = new FlipInfo();
		}
		flipInfo.setData(voList);
		return flipInfo;
	}
	
	private List<GovdocListVO> setListVO(List<GovdocListVO> voList, Map<String, String> condition,ResourceValueBO rvb) throws BusinessException {
        Long memberId = Long.parseLong(condition.get("memberId"));
        //boolean agentFlag = GovdocParamUtil.getBoolean(condition, "agentFlag") || GovdocParamUtil.getBoolean(condition, "agentToFlag");
        CtpEnumBean docType = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_doc_type.name());//公文种类
        CtpEnumBean sendType = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_send_type.name());//行文类型
        CtpEnumBean secretLevel = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_secret_level.name());//密级
        CtpEnumBean urgentLevel = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_urgent_level.name());//紧急
        CtpEnumBean unitLevel = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_unit_level.name());//公文类别
        CtpEnumBean keepPeriod = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_keep_period.name());//保存期限
        //性能提升，预归档路径缓存
        Map<String, String> docPathMap = new HashMap<String, String>();
        //性能提升，领导批示
        //Map<Long, String> leaderPishiMap = new HashMap<Long, String>();
        List<Long> summaryIds = new ArrayList<Long>();
        for (GovdocListVO listVo : voList) {
            summaryIds.add(listVo.getSummaryId());
        }
        boolean hasNodePolicy = false;
        boolean hasCurrentNodesInfo = false;
        String listType = "";//
        boolean isCurrentUserData = false;//是否是当前登录人数据；
        if(rvb != null){
        	for(ResourceItemValueBO bo : rvb.getDataShowColumns()){
            	if(bo.getCode().equals("affairNodePolicy")){
            		hasNodePolicy = true;
            	}
            	if("currentNodesInfo".equals(bo.getCode())){
            		hasCurrentNodesInfo = true;
            	}
            }
        	for(ResourceItemValueBO itemValue:rvb.getDataSources()){
				if("affairState".equals(itemValue.getCode())){
					if(itemValue.getDbValue().toString().contains(""+StateEnum.col_pending.getKey())){
						listType = "Pending";
					}else if(itemValue.getDbValue().toString().contains(""+StateEnum.col_done.getKey())){
						listType = "Done";
					}
				}else if("affairMemberId".equals(itemValue.getCode())){
					if(ResourceMemberTypeEnums.currentMember.name().equals(itemValue.getDbValue().toString())){
						isCurrentUserData = true;
					}
				}
			}
        }
        for (GovdocListVO listVo : voList) {

			if(listVo.getProcessId() != null && listVo.getExchangeSendAffairId() != null && listVo.getExchangeSendAffairId().longValue() != -1){

				if (listVo.getExchangeSendAffairId().equals(listVo.getAffairId())){

					listVo.setCompeteDoneType(null);
				}
				else
				{
					CtpAffair sendAffair = affairManager.get(listVo.getExchangeSendAffairId());
					if(sendAffair!= null && sendAffair.getMemberId().equals(listVo.getAffairMemberId().longValue()) && "true".equals(condition.get("deduplication")))
					{
						listVo.setCompeteDoneType(null);
					}
					else
					{
						listVo.setCompeteDoneType("no");
					}

				}
			}
            //附件区分
            listVo.setHasAtt(IdentifierUtil.lookupInner(listVo.getIdentifier(), EdocSummary.INENTIFIER_INDEX.HAS_ATTACHMENTS.ordinal(), '1'));
            //发起人/拟稿人
            V3xOrgMember startUser = orgManager.getMemberById(listVo.getStartUserId());
            listVo.setStartUserName(startUser == null ? "" : GovdocUtil.getMemberName(startUser.getId()));
            //已办所属人
            //防护，防止登记簿过来时空指针异常,解决 GOVB-85 公文统计-收发文登记薄查询都报错
            if(listVo.getCompeteTransactor()!=null) {
                listVo.setCompeteTransactor(GovdocUtil.getMemberName(Long.valueOf(listVo.getCompeteTransactor())));
            }
            //设置处理人和回退人
            listVo.setMemberName(GovdocUtil.getMemberName(listVo.getAffairMemberId()));
            if(listVo.getAffairBackFromId() != null){
            	listVo.setBackFromName(GovdocUtil.getMemberName(listVo.getAffairBackFromId()));
            }

            //当前待办人
            if(hasCurrentNodesInfo){
            	if (listVo.getSummaryState() != null && listVo.getSummaryState() == flowState.terminate.ordinal()) {
                    listVo.setCurrentNodesInfo(ResourceUtil.getString("collaboration.list.finished.label"));
                } else {
                    listVo.setCurrentNodesInfo(GovdocHelper.parseCurrentNodesInfo(listVo.getCompleteTime(), listVo.getCurrentNodesInfo(), Collections.EMPTY_MAP));
                } 
            }
            //流程期限
            listVo.setSummaryDeadLineName(GovdocHelper.getDeadLineName(listVo.getSummaryDeadlineDatetime()));
            //节点期限
            if (listVo.getAffairExpectedProcessTime() != null) {
                listVo.setAffairDeadLineName(WFComponentUtil.getDeadLineName(listVo.getAffairExpectedProcessTime()));
            } else {
                listVo.setAffairDeadLineName(WFComponentUtil.getDeadLineName(listVo.getAffairDeadlineDate()));
            }
            //设置交换状态(未结束/已结束/已分送)
            if (listVo.getSummaryTransferStatus() != null && listVo.getSummaryTransferStatus().intValue() >= 2) {
                listVo.setSummaryTransferStatusName(ResourceUtil.getString(GovdocEnum.TransferStatus.getByKey(listVo.getSummaryTransferStatus()).getLabel()));
            } else if (listVo.getSummaryState() != null) {
                if (listVo.getSummaryState() != null) {
                    if (listVo.getSummaryState().intValue() == 0) {
                        listVo.setSummaryTransferStatusName(ResourceUtil.getString("govdoc.Notfinished"));
                    } else if (listVo.getSummaryState().intValue() == 1 || listVo.getSummaryState().intValue() == 3) {
                        if (listVo.getExchangeSendAffairId() == null || listVo.getExchangeSendAffairId() == -1) {
                            listVo.setSummaryTransferStatusName(ResourceUtil.getString("govdoc.Hasended"));
                        } else {
                            listVo.setSummaryTransferStatusName(ResourceUtil.getString("govdoc.sent"));
                        }
                    }
                }
            }
            //设置督办时间
            listVo.setSurplusTime(GovdocListHelper.calculateSurplusTime(listVo.getAffairReceiveTime(), listVo.getAffairExpectedProcessTime()));
            //公文枚举值设置
            GovdocListHelper.fillListEnumLabel(listVo, docType, sendType, keepPeriod, secretLevel, urgentLevel, unitLevel);
            //是否归档和归档路径
            if ("listQuery".equals(listType)) {
                GovdocListHelper.fillListArchiveData(listVo, docPathMap);
            }
            //列表展示动态标题
            /*if (Strings.isNotBlank(listVo.getAffairSubject())) {
                listVo.setSubject(listVo.getAffairSubject());
            }*/
            //自动发起标题
            if (listVo.getAutoRun()) {
                listVo.setSubject(ResourceUtil.getString("collaboration.newflow.fire.subject", listVo.getSubject()));
                listVo.setAffairSubject(ResourceUtil.getString("collaboration.newflow.fire.subject", listVo.getAffairSubject()));
            }
            if (listVo.getAffairMemberId() != null && memberId != null
                   && !"niwen".equals(listVo.getAffairNodePolicy()) && isCurrentUserData) {
                GovdocListHelper.fillListAgent(listVo, true, memberId, listType);//回填代理人
            }
			if (listVo.getAffairPreApprover() != null) {
				listVo.setPreUserName(GovdocUtil.getMemberName(listVo.getAffairPreApprover()));
			}
            // 暂时不需要 领导批示编号
			//listVo.setLeaderCommondNo(GovdocListHelper.getAllLeaderPishi(leaderPishiMap, summaryIds, listVo.getSummaryId()));
            if (listVo.getAffairTrack() == null) {
                listVo.setAffairTrack(0);
            }
            GovdocAuthorityUtil.addSummaryForView(listVo.getSummaryId(), listVo.getAffairId());
            if(listVo.getAffairNodePolicy()!=null && hasNodePolicy){//配置节点权限名称
            	String category =""; 
                if(listVo.getAffairSubApp() == ApplicationSubCategoryEnum.edoc_fawen.getKey()){  //发文
              	  category =EnumNameEnum.edoc_new_send_permission_policy.name();
                }else if(listVo.getAffairSubApp() == ApplicationSubCategoryEnum.edoc_shouwen.getKey()){  //收文
              	  category =EnumNameEnum.edoc_new_rec_permission_policy.name();
                }else if(listVo.getAffairSubApp() == ApplicationSubCategoryEnum.edoc_qianbao.getKey()){  //签报
              	  category =EnumNameEnum.edoc_new_qianbao_permission_policy.name();
                }else if(listVo.getAffairSubApp() == ApplicationSubCategoryEnum.edoc_jiaohuan.getKey()){  //交换
              	  category=EnumNameEnum.edoc_new_change_permission_policy.name();
        		}else if(listVo.getAffairSubApp() == ApplicationSubCategoryEnum.old_edocSend.getKey()) {
        			category = EnumNameEnum.edoc_send_permission_policy.name();
				} else if(listVo.getAffairSubApp() == ApplicationSubCategoryEnum.old_edocRec.getKey()) {
					category = EnumNameEnum.edoc_rec_permission_policy.name();
				} else if(listVo.getAffairSubApp() == ApplicationSubCategoryEnum.old_edocSign.getKey()) {
					category = EnumNameEnum.edoc_qianbao_permission_policy.name();
				}
                listVo.setAffairNodePolicy(permissionManager.getPermissionName(category, listVo.getAffairNodePolicy(), AppContext.currentAccountId()));
            }
        }
        return voList;
    }

	@Override
	public Map<String,List<ResourceItemValueBO>> getItemInitValues() throws BusinessException {
		return null;
		/*Map<String,List<ResourceItemValueBO>> allItemValue = new HashMap<String,List<ResourceItemValueBO>>();
		String xmlPath = AppContext.getCfgHome().getAbsolutePath() + File.separator + "plugin" + File.separator + "govdoc" + File.separator + "resourceItems.xml";
		InputStream in = null;
		try{
			in = new FileInputStream(new File(xmlPath));
			String content = IOUtility.toString(in);
			Document doc;
			try {
				doc = XXEUtil.safeParseText(content);
				Element rootElm = doc.getRootElement();
				if (rootElm == null) {
					throw new BusinessException("根节点不存在 ");
				}

				List<Element> elements = rootElm.elements("itemInitValues");
				if(elements!=null && elements.size()>0){
					elements = elements.get(0).elements();
				}
				List<ResourceItemValueBO> temList = null;
				for(Element aelement:elements){
					if (aelement != null) {
						temList = new ArrayList<ResourceItemValueBO>();
						String code = "";
						Attribute fattrib = aelement.attribute("code");
						if (fattrib != null) {
							if (!StringUtil.checkNull(fattrib.getValue())) {
								code = fattrib.getValue();
							}
						}
						allItemValue.put(code, temList);
						List<Element> firstElemens = aelement.elements("itemValue");//第一组初始值
						if(elements!=null && elements.size()>0){
							ResourceItemValueBO itemValue = null;
							for(Element e:firstElemens){
								itemValue = new ResourceItemValueBO();
								itemValue.setCode(code);
								temList.add(itemValue);
								fattrib = aelement.attribute("value");
								if (fattrib != null) {
									if (!StringUtil.checkNull(fattrib.getValue())) {
										itemValue.setDbValue(fattrib.getValue());
									}
								}
								fattrib = aelement.attribute("showName");
								if (fattrib != null) {
									if (!StringUtil.checkNull(fattrib.getValue())) {
										itemValue.setItemShowName(fattrib.getValue());
									}
								}
								
								
							}
						}
					}
				}
			} catch (DocumentException arg10) {
				throw new BusinessException("XML格式不对", arg10);
			}
		}catch(Exception e){
			logger.error(e);
		}
		return allItemValue;*/
	}

	public PrivilegeConfigManager getPrivilegeConfigManager() {
		return privilegeConfigManager;
	}

	public void setPrivilegeConfigManager(PrivilegeConfigManager privilegeConfigManager) {
		this.privilegeConfigManager = privilegeConfigManager;
	}

	public OrgManager getOrgManager() {
		return orgManager;
	}

	public void setOrgManager(OrgManager orgManager) {
		this.orgManager = orgManager;
	}

	public EnumManager getEnumManagerNew() {
		return enumManagerNew;
	}

	public void setEnumManagerNew(EnumManager enumManagerNew) {
		this.enumManagerNew = enumManagerNew;
	}

	@Override
	public List<String> getFrontJSFiles() throws BusinessException {
		List<String> jsFiles = new ArrayList<String>();
		jsFiles.add("apps_res/govdoc/js/list/list_common.js");
		jsFiles.add("apps_res/govdoc/js/list/list.js");
		jsFiles.add("apps_res/collaboration/js/batch.js");
		jsFiles.add("apps_res/govdoc/js/list/govdocResource.js");
		return jsFiles;
	}

	public PermissionManager getPermissionManager() {
		return permissionManager;
	}

	public void setPermissionManager(PermissionManager permissionManager) {
		this.permissionManager = permissionManager;
	}
	public void setAffairManager(AffairManager affairManager) {
		this.affairManager = affairManager;
	}

}
