package com.seeyon.apps.govdoc.resourceManager;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.seeyon.apps.govdoc.manager.GovdocOpenManager;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.ctpenumnew.EnumNameEnum;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.permission.manager.PermissionManager;
import com.seeyon.ctp.form.util.permission.factory.PermissionFatory;
import com.seeyon.ctp.resourceEngine.bo.ResourceItemValueBO;
import com.seeyon.ctp.resourceEngine.bo.ResourceValueBO;
import com.seeyon.ctp.resourceEngine.manager.PrivilegeConfigManager;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.annotation.AjaxAccess;

import edu.emory.mathcs.backport.java.util.Arrays;

public class GovdocResourceManagerImpl implements GovdocResourceManager {
	
	private PrivilegeConfigManager privilegeConfigManager;
	private PermissionManager permissionManager;

	@Override
	public Map<String, Object> getResourceAttr(Map<String,String> param) throws BusinessException {
		Map<String,Object> m = new HashMap<String,Object>();
		GovdocOpenManager govdocOpenManager = (GovdocOpenManager)AppContext.getBean("govdocOpenManager");
		String zhuanFaWen=govdocOpenManager.checkEdocZhuanfawen();//转发文默认设置
		m.put("zhuanfawen", zhuanFaWen);
		
		String apps = param.get("apps");
		List<String> appsSet = Arrays.asList(apps.split(""));
		boolean actionCancel = false;
		boolean actionArchive = false;
		List<String> baseAction = new ArrayList<String>();
		for (String govdocType : appsSet) {
			String permissionEnumName = PermissionFatory.getPermBySubApp(Long.valueOf(govdocType)).getCategorty();
			String configItem = null;
			if (permissionEnumName.equals(EnumNameEnum.edoc_new_send_permission_policy.name())
					|| permissionEnumName.equals(EnumNameEnum.edoc_new_qianbao_permission_policy.name())) {
				configItem = "niwen";
			} else if (permissionEnumName.equals(EnumNameEnum.edoc_new_rec_permission_policy.name())) {
				configItem = "dengji";
			}
			if (configItem == null) {
				continue;
			}
			baseAction = permissionManager.getBasicActionList(permissionEnumName, configItem,AppContext.currentAccountId());
			for (String action : baseAction) {
				// 撤销
				if ("cancel".toLowerCase().equals(action.trim().toLowerCase())) {
					actionCancel = true;
				}
				//归档
				if ("archive".toLowerCase().equals(action.trim().toLowerCase())) {
					actionArchive = true;
				}
			}
		}
		m.put("actionCancel", actionCancel);
		m.put("actionArchive", actionArchive);
		return m;
	}

	@Override
	@AjaxAccess
	public String getGovDocTypeByResourceId(String resourceId) throws BusinessException {
		String govdocType = "";
		ResourceValueBO pvb = privilegeConfigManager.findByResourceId(Long.parseLong(resourceId));
		if(pvb!=null){
			for(ResourceItemValueBO itemValue : pvb.getDataSources()){
				if("affairSubApp".equals(itemValue.getCode())){
					String subApp = itemValue.getDbValue().toString();
					if(Strings.isNotBlank(govdocType)){
						govdocType = govdocType + "," + subApp;
					}else{
						govdocType = subApp;
					}
				}
			}
		}
		return govdocType;
	}

	public PrivilegeConfigManager getPrivilegeConfigManager() {
		return privilegeConfigManager;
	}

	public void setPrivilegeConfigManager(PrivilegeConfigManager privilegeConfigManager) {
		this.privilegeConfigManager = privilegeConfigManager;
	}

	public PermissionManager getPermissionManager() {
		return permissionManager;
	}

	public void setPermissionManager(PermissionManager permissionManager) {
		this.permissionManager = permissionManager;
	}

}
