package com.seeyon.apps.govdoc.resourceManager;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;

import com.seeyon.apps.govdoc.constant.GovdocEnum.govdocTypeEnum;
import com.seeyon.apps.govdoc.manager.GovdocElementManager;
import com.seeyon.apps.govdoc.resourceManager.GovdocResourceEnum.GovdocAffairSubAppIncludeOldExchangeEnums;
import com.seeyon.apps.govdoc.resourceManager.GovdocResourceEnum.ResourceMemberTypeEnums;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.ctpenumnew.manager.EnumManager;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.common.po.ctpenumnew.CtpEnumBean;
import com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.dao.OrgHelper;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.resourceEngine.bo.ResourceItem;
import com.seeyon.ctp.resourceEngine.bo.ResourceItemValueBO;
import com.seeyon.ctp.resourceEngine.bo.ResourceValueBO;
import com.seeyon.ctp.resourceEngine.enums.ResourceConfigEnums.ResourceDataTypeEnums;
import com.seeyon.ctp.resourceEngine.enums.ResourceConfigEnums.ResourceExpressionEnums;
import com.seeyon.ctp.resourceEngine.enums.ResourceConfigEnums.ResourceItemTypeEnums;
import com.seeyon.ctp.resourceEngine.enums.ResourceConfigEnums.ResourceSelectTypeEnums;
import com.seeyon.ctp.util.Strings;
import com.seeyon.v3x.edoc.domain.EdocElement;

public class ResourceItemGovdoc extends ResourceItem {
	private final static Log     logger       = CtpLogFactory.getLog(ResourceItemGovdoc.class);
	
	private String elementCode;

	/**
	 * 
	 */
	private static final long serialVersionUID = -2975111412014462109L;

	@Override
	public Long getEnumId() {
		if(ResourceItemTypeEnums.govdocElements.getKey().equals(this.getItemType()) && ResourceSelectTypeEnums.select.getKey().equals(this.getSelectType())){
			GovdocElementManager gem = (GovdocElementManager)AppContext.getBean("govdocElementManager");
			EdocElement element = gem.getByFieldName(getElementCode()==null?getCode():getElementCode());
			if(element != null && element.getMetadataId() !=null){
				return element.getMetadataId();
			}
		}
		return super.getEnumId();
	}

	@Override
	public String getLabel() {
		if(ResourceItemTypeEnums.govdocElements.getKey().equals(this.getItemType())){
			GovdocElementManager gem = (GovdocElementManager)AppContext.getBean("govdocElementManager");
			EdocElement element = gem.getByFieldName(getElementCode()==null?getCode():getElementCode());
			if(element != null){
				return ResourceUtil.getString(element.getName());
			}
		}
		return super.getLabel();
	}

	public String getElementCode() {
		return elementCode;
	}

	public void setElementCode(String elementCode) {
		this.elementCode = elementCode;
	}
	
	@Override
	public String getHbmAttr(Object dbValue) {
		if(this.getCode().equals("affairMemberId")){//统计范围要处理下
			String value = dbValue.toString();
			if(ResourceMemberTypeEnums.currentDepartment.name().equals(value)){
				return "affair.matchDepartmentId";
			}else if(value.indexOf("Department|") > -1){
				return "affair.matchDepartmentId";
			}
		}
		return super.getHbmAttr(dbValue);
	}

	@Override
	public Object getRealDBValue(ResourceExpressionEnums charEnum,Object dbValue){
		if(this.getCode().equals("affairMemberId")){//统计范围要处理下
			String value = dbValue.toString();
			if(ResourceMemberTypeEnums.currentDepartment.name().equals(value)){
				return AppContext.getCurrentUser().getDepartmentId();
			}else if(ResourceMemberTypeEnums.currentMember.name().equals(value)){
				return AppContext.currentUserId();
			}else if(value.indexOf("Department|") > -1){
				return Long.parseLong(value.replace("Department|", ""));
			}else {
				if(value.indexOf("Post|") > -1){//有可能是基准岗
					OrgManager orgManager = (OrgManager)AppContext.getBean("orgManager");
					try {
						List<V3xOrgMember> members = orgManager.getMembersByPost(Long.parseLong(value.replace("Post|", "")), AppContext.currentAccountId());
						List<Long> memberIds = null;
						if(Strings.isNotEmpty(members)){
							memberIds = new ArrayList<Long>();
							for(V3xOrgMember m:members){
								memberIds.add(m.getId());
							}
						}
						return memberIds;
					} catch (Exception e) {
						logger.error(e);
					}
				}
				List<Long> result = OrgHelper.getMemberIdsByTypeAndId(value, (OrgManager) AppContext.getBean("orgManager"));
				if(Strings.isEmpty(result)){
					result = null;
				}
				return result;
			}
		}else if(this.getCode().equals("turnRecUserId")){
			return null;// 是否为空
		}else if(this.getCode().equals("affairSubApp")){
			GovdocAffairSubAppIncludeOldExchangeEnums e = GovdocAffairSubAppIncludeOldExchangeEnums.findByKey(dbValue.toString());
			if(e!=null){
				dbValue = e.getValue();
			}
		}
		return super.getRealDBValue(charEnum, dbValue);
	}

	@Override
	public List<Map<String, Object>> findEnums(ResourceValueBO pvb) {
		if((ResourceSelectTypeEnums.select.getKey().equals(this.getSelectType()) || ResourceSelectTypeEnums.checkbox.getKey().equals(this.getSelectType())) && !ResourceDataTypeEnums.booler.getKey().equals(this.getDataType())){
			List<Map<String,Object>> eList = new ArrayList<Map<String,Object>>();
			if(this.getEnumId() == null){
				if(this.getCode().equals("govdocType")){//分类//
					Map<String,Object> eMap = new HashMap<String,Object>();
					eMap.put("value", GovdocAffairSubAppIncludeOldExchangeEnums.sendGov.getValue());
					eMap.put("text", GovdocAffairSubAppIncludeOldExchangeEnums.sendGov.getText());
					eList.add(eMap);
					eMap = new HashMap<String,Object>();
					eMap.put("value", GovdocAffairSubAppIncludeOldExchangeEnums.receiveGov.getValue()+","+GovdocAffairSubAppIncludeOldExchangeEnums.exchangeGov.getValue());
					eMap.put("text", GovdocAffairSubAppIncludeOldExchangeEnums.receiveGov.getText());
					eList.add(eMap);
					eMap = new HashMap<String,Object>();
					eMap.put("value", GovdocAffairSubAppIncludeOldExchangeEnums.signGov.getValue());
					eMap.put("text", GovdocAffairSubAppIncludeOldExchangeEnums.signGov.getText());
					eList.add(eMap);
				}else if("summaryState".equals(this.getCode())){//流程状态
					Map<String,Object> eMap = new HashMap<String,Object>();
					eMap.put("value", 0);
					eMap.put("text", ResourceUtil.getString("collaboration.unend"));
					eList.add(eMap);
					eMap = new HashMap<String,Object>();
					eMap.put("value", 1);
					eMap.put("text", ResourceUtil.getString("collaboration.terminated"));
					eList.add(eMap);
					eMap = new HashMap<String,Object>();
					eMap.put("value", 3);
					eMap.put("text", ResourceUtil.getString("collaboration.ended"));
					eList.add(eMap);
				}else if("affairNodePolicy".equals(this.getCode())){//节点权限 没法查
					//公文权限？
				}else if("subState".equals(this.getCode())){//状态
					if(pvb != null){
						Map<String,String> wsStateCons = new LinkedHashMap<String,String>();
						String wsBack = "2,16,18";
						String pendingBack = "7,16";
						String allBack = "2,7,16,18";
						wsStateCons.put("1", ResourceUtil.getString("govdoc.resource.resourceConfig.nodeStatus.draft"));
						wsStateCons.put(wsBack, ResourceUtil.getString("govdoc.resource.resourceConfig.nodeStatus.rollbackd"));
						wsStateCons.put("3", ResourceUtil.getString("govdoc.resource.resourceConfig.nodeStatus.revoke"));
						Map<String,String> pendingStateCons = new LinkedHashMap<String,String>();
						pendingStateCons.put("11,12,13", ResourceUtil.getString("govdoc.resource.resourceConfig.nodeStatus.daiban"));
						pendingStateCons.put(pendingBack, ResourceUtil.getString("govdoc.resource.resourceConfig.nodeStatus.rollbackd"));
						pendingStateCons.put("6", ResourceUtil.getString("govdoc.resource.resourceConfig.nodeStatus.takeBack"));
						boolean hasWSBack = false;
						for(ResourceItemValueBO itemValue : pvb.getDataSources()){
							if("affairState".equals(itemValue.getCode())){
								if("1".equals(itemValue.getDbValue())){//待发
									List<ResourceItemValueBO>  sItems = itemValue.getSubItem();
									Map<String,Object> eMap = null;
									if(Strings.isEmpty(sItems)){
										for(String key:wsStateCons.keySet()){
											eMap = new HashMap<String,Object>();
											eMap.put("value", key);
											eMap.put("text", wsStateCons.get(key));
											eList.add(eMap);
										}
										hasWSBack = true;
									}else{
										for(ResourceItemValueBO sb:sItems){
											eMap = new HashMap<String,Object>();
											eMap.put("value", sb.getDbValue());
											eMap.put("text", wsStateCons.get(sb.getDbValue()));
											eList.add(eMap);
											if(sb.getDbValue().equals(wsBack)){
												hasWSBack = true;
											}
										}
									}
								}else if("3".equals(itemValue.getDbValue())){//待办
									List<Map<String,Object>> eList2 = new ArrayList<Map<String,Object>>();
									List<ResourceItemValueBO>  sItems = itemValue.getSubItem();
									Map<String,Object> eMap = null;
									if(Strings.isEmpty(sItems)){
										for(String key:pendingStateCons.keySet()){
											eMap = new HashMap<String,Object>();
											eMap.put("value", key);
											eMap.put("text", pendingStateCons.get(key));
											eList2.add(eMap);
										}
									}else{
										for(ResourceItemValueBO sb:sItems){
											eMap = new HashMap<String,Object>();
											eMap.put("value", sb.getDbValue());
											eMap.put("text", pendingStateCons.get(sb.getDbValue()));
											eList2.add(eMap);
										}
									}
									if(hasWSBack){
										for(Map<String,Object> em2:eList2){
											if(pendingBack.equals(em2.get("value"))){
												for(Map<String,Object> em:eList){
													if(wsBack.equals(em.get("value"))){
														em.put("value", allBack);
													}
												}
												eList2.remove(em2);
												break;
											}
										}
									}
									eList.addAll(eList2);
								}
							}
						}
					}
				}else if(this.getCode().contains("list")){
					Map<String,Object> eMap = new HashMap<String,Object>();
					eMap.put("value", "");
					eMap.put("text", "");
					eList.add(eMap);
				}
				return eList;
			}else{
				if(this.getCode().contains("list")){//宽展字段
					Long enumId = this.getEnumId();
					if(enumId != null){
						EnumManager enumManagerNew = (EnumManager)AppContext.getBean("enumManagerNew");
						CtpEnumBean metadata = enumManagerNew.getEnum(enumId);
						if(metadata != null && metadata.getItems() != null){
							for(CtpEnumItem item:metadata.getItems()){
								Map<String,Object> eMap = new HashMap<String,Object>();
								eMap.put("value", item.getId());
								eMap.put("text", ResourceUtil.getString(item.getLabel()));
								eList.add(eMap);
							}
						} 
					}else{
						Map<String,Object> eMap = new HashMap<String,Object>();
						eMap.put("value", "");
						eMap.put("text", "");
						eList.add(eMap);
					}
					return eList;
				}
			}
		}
		return super.findEnums(pvb);
	}

}
